function [score outerProd scoreMat] = ...
    kalman_derivs(freeparams, allparams, free_index, ...
                        observedData, numFactors, tolerance, delta)
% derivatives.

%
% Greg Duffee
% June 2011
%

[T numObserved] = size(observedData); 

numFree = length(freeparams);

score = zeros(numFree, 1);
outerProd = zeros(numFree, numFree);
scoreMat = zeros(T, numFree);

for thisParam = 1:numFree
  freeCopy = freeparams;
  freeCopy(thisParam) = freeparams(thisParam) + delta;
  paramvec = poke_freeparams(freeCopy, allparams, free_index); 

  [mu K_vec Sigma_vec A B_vec errorSDs] = ...
      unpack_allparams(numFactors, numObserved, paramvec);

  [Kmat Sigma B H] = ...
      build_matrices(K_vec, Sigma_vec, B_vec, errorSDs); 

  [errors loglikesUp filtered_values cond_vars contemp_vars ...
   normSqrErrors eigFs] = ...
      kalman_recursion_noderiv(observedData, mu, Kmat, Sigma, H, A, B, ...
                               tolerance);
  
  freeCopy(thisParam) = freeparams(thisParam) - delta;
  paramvec = poke_freeparams(freeCopy, allparams, free_index); 

  [mu K_vec Sigma_vec A B_vec errorSDs] = ...
      unpack_allparams(numFactors, numObserved, paramvec);
  
  [Kmat Sigma B H] = ...
      build_matrices(K_vec, Sigma_vec, B_vec, errorSDs); 

  [errors loglikesDown filtered_values cond_vars contemp_vars ...
   normSqrErrors eigFs] = ...
      kalman_recursion_noderiv(observedData, mu, Kmat, Sigma, H, A, B, ...
                               tolerance);
  scoreMat(:,thisParam) = (loglikesUp - loglikesDown)/(2*delta);
end

score = sum(scoreMat);
outerProd = scoreMat' * scoreMat;
    
return

