function [freeparams bigparamvec free_index] = setup_initparams( ...
    mu_vals, k_vals, S_vals, A_vals, B_vals, ...
    errorSD_vals, mu_flag, k_flag, S_flag, ...
    A_flag, B_flag, errorSD_flag, ols_flag, olsStartIdx, allData);
%
% SETUP_INITPARAMS gets the initial vectors of free parameters and
% unrestricted parameters.  
%

%
% Greg Duffee
% June 2011
%

mu = mu_vals;
k_vec = k_vals; 
S = S_vals; 
A = A_vals;
B_vec = B_vals;
errorSDs = errorSD_vals;

if (ols_flag == 1)
  [T numObserved] = size(allData); 
  numfactors = length(mu); 
  usedData = allData(:,olsStartIdx);
  xmat = [ones(T,1) usedData];

  [mu K vcv Sigma vector_form mu_companion K_companion resids] = ...
      get_varparams(usedData, 1);
  k_vec = vec(K); 
  for i = 1:numObserved
    [coefs bint r rint stats] = regress(allData(:,i), xmat);
    A(i) = coefs(1);
    B(i,:) = coefs(2:end);
  end
  B_vec = vec(B); 
  
  index = 0; 
  for i = 1:numfactors
    for j = 1:i
      index = index + 1; 
      S(index) = Sigma(i,j);
    end
  end

end

[bigparamvec free_index] = pack_allparams( ...
    mu, k_vec, S, A, B_vec, errorSDs, ...
    mu_flag, k_flag, S_flag, A_flag, ...   
    B_flag, errorSD_flag); 

freeparams = pop_freeparams(bigparamvec, free_index); 

return

