function write_output(inputfile, outputfile, mu, K_vec, S, ...
                      A, B_vec, errorSD, loglike, exitflag) 
%
% WRITE_OUTPUT is a routine that writes the results of estimatng 
% a dynamic term structure model.  The results are written to 'outputfile'.

% Greg Duffee
% September 2009
%

numObserved = length(A);
numfactors = length(mu); 

%
% open the input and output files
%

f_input_id = fopen(inputfile, 'rt');
f_output_id = fopen(outputfile, 'wt'); 

%
% write the header
%

my_string = '% Output file for estimation of Kalman filter model'; 
fprintf(f_output_id, '%s\n', my_string); 
my_string = ['% ' datestr(now)]; 
fprintf(f_output_id, '%s\n', my_string); 

%
% copy input file until parameters are reached.  

inputline = textscan(f_input_id, '%[^\n]', 1, 'headerLines', 2); 
fprintf(f_output_id, '%s\n', char(inputline{1})); 

foundline = 0; 
while foundline == 0
    inputline = textscan(f_input_id, '%[^\n]', 1); 
    tmp = char(inputline{1}); 
    fprintf(f_output_id, '%s\n', tmp); 
    if ((length(tmp) > 19) & (tmp(1:20) == 'MU VECTOR (must be n'))
        foundline = 1; 
    end
end

inputlines = textscan(f_input_id, '%s %d %*f %f', double(numfactors)); 
param_names = char(inputlines{1}); 
param_flags = inputlines{2};
param_stddevs = inputlines{3};
for i = 1:numfactors
    fprintf(f_output_id, '%s %5d %+19.11e %+19.11e\n', param_names(i,:), ...
               param_flags(i), mu(i), param_stddevs(i)); 
end
  

inputline = textscan(f_input_id, '%[^\n]', 1); 
fprintf(f_output_id, '%s\n', char(inputline{1})); 

numlines = double(numfactors^2);
inputlines = textscan(f_input_id, '%s %d %*f %f', numlines); 
param_names = char(inputlines{1}); 
param_flags = inputlines{2};
param_stddevs = inputlines{3};
for i = 1:(numfactors^2)
    fprintf(f_output_id, '%s %5d %+19.11e %+19.11e\n', param_names(i,:), ...
               param_flags(i), K_vec(i), param_stddevs(i)); 
end

inputline = textscan(f_input_id, '%[^\n]', 1); 
fprintf(f_output_id, '%s\n', char(inputline{1})); 

numlines = double(numfactors*(1+numfactors)/2); 
inputlines = textscan(f_input_id, '%s %d %*f %f', numlines); 
param_names = char(inputlines{1}); 
param_flags = inputlines{2};
param_stddevs = inputlines{3};
for i = 1:(numfactors*(1+numfactors)/2)
    fprintf(f_output_id, '%s %5d %+19.11e %+19.11e\n', param_names(i,:), ...
               param_flags(i), S(i), param_stddevs(i)); 
end

inputline = textscan(f_input_id, '%[^\n]', 1); 
fprintf(f_output_id, '%s\n', char(inputline{1})); 

inputlines = textscan(f_input_id, '%s %d %*f %f', double(numObserved)); 
param_names = char(inputlines{1}); 
param_flags = inputlines{2};
param_stddevs = inputlines{3};
for i = 1:numObserved
    fprintf(f_output_id, '%s %5d %+19.11e %+19.11e\n', param_names(i,:), ...
               param_flags(i), A(i), param_stddevs(i)); 
end
  

inputline = textscan(f_input_id, '%[^\n]', 1); 
fprintf(f_output_id, '%s\n', char(inputline{1})); 

inputlines = textscan(f_input_id, '%s %d %*f %f', ...
                      double(numfactors*numObserved)); 
param_names = char(inputlines{1}); 
param_flags = inputlines{2};
param_stddevs = inputlines{3};
for i = 1:(numfactors*numObserved)
    fprintf(f_output_id, '%s %5d %+19.11e %+19.11e\n', param_names(i,:), ...
               param_flags(i), B_vec(i), param_stddevs(i)); 
end
  
inputline = textscan(f_input_id, '%[^\n]', 1); 
fprintf(f_output_id, '%s\n', char(inputline{1})); 

inputlines = textscan(f_input_id, '%s %d %*f %f', ...
                      double(numObserved)); 
param_names = char(inputlines{1}); 
param_flags = inputlines{2};
param_stddevs = inputlines{3};
for i = 1:numObserved
    fprintf(f_output_id, '%s %5d %+19.11e %+19.11e\n', param_names(i,:), ...
               param_flags(i), errorSD(i), param_stddevs(i)); 
end

%
% write the log-likelihood value and exit flag.  
%

my_string = 'LOG-LIKELIHOOD VALUE'; 
fprintf(f_output_id, '%s\n', my_string); 
fprintf(f_output_id, '%f\n', loglike); 

my_string = 'OPTIMIZATION EXIT FLAG'; 
fprintf(f_output_id, '%s\n', my_string); 
fprintf(f_output_id, '%d\n', exitflag); 

fclose(f_input_id);
fclose(f_output_id); 

return
