function [freeparams newseed] = generate_random_freeparams( ...
    numfactors, bigparamvec, stddevs, free_flags, ranseed)
%
% GENERATE_RANDOM_FREEPARAMS generates a random starting point.  The
% routine ensures the point is stationary.  NOTE ** this is a bit
% of a cheap way to check for stationarity.  I assume that the
% diagonal elements of D are always free parameters, so that the
% first 'numfactors' elements of the free parameter vector must be
% between -1 and 1.  
%

%
% Greg Duffee
% October 2005, revised June 2006
%

randn('state', ranseed); 

initial_freeparams = pop_freeparams(bigparamvec, free_flags); 
free_stddevs = pop_freeparams(stddevs, free_flags); 
num_freeparams = length(initial_freeparams); 

freeparams = initial_freeparams + free_stddevs .* randn(num_freeparams,1); 

newseed = randn('state'); 

end
