function [funval loglikes filtered_values] = ...
    kalman_only_loglike(freeparams, allparams, free_index, ...
                        observedData)
% KALMAN_ONLY_LOGLIKE returns the (negative of the) Kalman filter
% log-likelihood.  This is used for standard ML estimation.  

%
% Greg Duffee
% June 2011
%

tolerance = 1e-9; 

loglikes = [];
filtered_values = [];

[T numObserved] = size(observedData);
% order is one-quarter yield, all other yields (if any),
% then inflation, output.

numbonds = numObserved - 2; 

paramvec = poke_freeparams(freeparams, allparams, free_index); 

%
% build up model vectors and matrices. 
%

[means, bcm, stddev, Avec, Bvec, errorSD] = ...
    unpack_allparams(paramvec, numbonds); 

RE_output = solveRE(bcm, stddev); 

[delta0 delta1 mu K Sigma bailout] = buildRESolution(RE_output, ...
                                                  means);

%
% check beta coefficients on long run inflation.  
%

beta_bailout = 0; 
fail_amount = 0; 

% below is a kludge to see whether restrictions on coefficients of
% long-run inflation equation are useful.

L3 = 1 - bcm(8) - bcm(9); 
%if ( (bcm(8) < 0.0) | (bcm(9) < 0.0) | (L3 < 0.0) )
%  beta_bailout = 1;
%  fail_amount = (min(0, bcm(8))*bcm(8)*1e8) + ...
%      (min(0, bcm(9))*bcm(9)*1e8) + ...
%      (min(0, L3)*L3*1e8) + 1e8; 
%end


if (bailout == 1)
  funval = 1e8 + 1e8*RE_output.maxEig;
%  fprintf(1, 'Failure, max abs physical eig = %8.3f\n', RE_output.maxEig); 
else
  if (beta_bailout == 1) 
    funval = fail_amount; 
  else
  
    [A B H] = build_matrices(delta0, delta1, Avec, Bvec, errorSD);
    
    [errors loglikes filtered_values cond_vars contemp_vars ...
     normSqrErrors eigFs] = ...
      kalman_recursion_noderiv(observedData, mu, K, Sigma, H, A, B, ...
                               tolerance);
    funval = -sum(loglikes);
  end
  
end

%fprintf(1, '%20.5f\n', funval); 

return

