function output = recursion(B11, A11, B12, QZ)
%
% Implementation of the recursion method in Cho and Moreno (2003)
% to solve models of the form
%
% B_11 X_t = A_11 E_t X_{t+1} + B_12 X_{t-1} + \epsilon_t, 
%
% E_{t-1} \epsilon_t = 0.
%

% matrices mat1_k and mat2_k, k = 1, \dots, are the Phi_{k} and
% next matrix on page 26 of Cho and Moreno.  mat3_k is the matrix
% under those.

if (nargin<4); QZ = 1; end;

nn = size(B11,1); 

A = [A11 zeros(nn,nn); zeros(nn,nn) eye(nn)];
B = [B11 -B12; eye(nn) zeros(nn,nn)];
C1 = eye(nn); 

if QZ==1,            
  [Omegan,Geig]=qzc(A,B,seqa(1,1,nn),'McCallum');
  for i=1:nn, for j=1:nn                
      if abs(imag(Omegan(i,j)))<0.0000001, Omegan(i,j)=real(Omegan(i,j));end
    end, end       
    Omegan=real(Omegan);    
    Gamman=inv(B11-A11*Omegan)*(C1);
    
elseif QZ==2,             
  [Omegan,Geig,Gamman]=RS(80, A11, B11, B12);            
end

maxEig = max(abs(eig(Omegan))); 

error_lagState = B11 * Omegan - (A11 * Omegan^2) - B12;
error_shock = B11 * Gamman - (A11 * Omegan * Gamman) - eye(nn);

output.Omegan = Omegan;
output.Gamman = Gamman;
output.maxEig = maxEig;
output.error_lagState = error_lagState;
output.error_shock = error_shock;

return
