function output = solveRE(bcm, stddev) 
%
% Greg Duffee
% July 2011
%
% use the BCM recursion method.
%

% write model in terms of current = expected future + lagged +
% shock.  State vector is g, pi, r, pi^L  Everything is in
% deviations from means.
%
% B_11 X_t = A_11 E_t X_{t+1} + B_12 X_{t-1} + \epsilon_t, 
%
% \epsilon_t = F \epsilon_{t-1} + \omega_t, 
%
% E_{t-1} \omega_t = 0.

% altered scaling  

beta_g = bcm(1);
gamma_r = bcm(2); 
%gamma_r = bcm(2)*1e-5;
beta_pi = bcm(3);
gamma_pi = bcm(4); 
%gamma_pi = bcm(4)*1.0e-5;

beta_r = bcm(5);
gamma_rpi = bcm(6);
gamma_rg = bcm(7);

beta_L1 = bcm(8);
beta_L2 = bcm(9);

epsilon_vcv = diag(stddev .^2); 

B_11 = zeros(4,4);
A_11 = zeros(4,4);
B_12 = zeros(4,4); 
F = zeros(4,4); 

B_11(1,1) = 1;
A_11(1,1) = beta_g;
B_12(1,1) = 1 - beta_g;
B_11(1,3) = gamma_r;
A_11(1,2) = gamma_r; 

B_11(2,2) = 1; 
A_11(2,2) = beta_pi;
B_12(2,2) = 1 - beta_pi;
B_11(2,1) = -gamma_pi;

B_11(3,3) = 1;
B_12(3,3) = beta_r;
A_11(3,2) = (1 - beta_r)*gamma_rpi;
B_11(3,4) = (1 - beta_r)*gamma_rpi;
B_11(3,1) = -(1 - beta_r)*gamma_rg;

B_11(4,4) = 1;
A_11(4,4) = beta_L1;
B_12(4,4) = beta_L2;
B_11(4,2) = -(1 - beta_L1 - beta_L2);

QZ = 2; 
output = recursion(B_11, A_11, B_12, QZ);

Gamma = output.Gamman; 

fullVCV = Gamma * epsilon_vcv * Gamma';

output.fullVCV = fullVCV;
output.B11 = B_11;
output.A11 = A_11;
output.B12 = B_12; 


return
