function write_output(inputfile, outputfile, means, bcm, stddev, Avec, ...
                      Bvec, errorSD, loglike, exitflag) 
%
% WRITE_OUTPUT is a routine that writes the results of estimatng 
% a dynamic term structure model.  The results are written to 'outputfile'.

% Greg Duffee
% September 2009, revised July 2011
%

numUsedBonds = length(errorSD) - 2; 

%
% open the input and output files
%

f_input_id = fopen(inputfile, 'rt');
f_output_id = fopen(outputfile, 'wt'); 

%
% write the header
%

my_string = '% Output file for estimation of dynamic term structure model'; 
fprintf(f_output_id, '%s\n', my_string); 
my_string = ['% ' datestr(now)]; 
fprintf(f_output_id, '%s\n', my_string); 

%
% copy input file until parameters are reached.  

inputline = textscan(f_input_id, '%[^\n]', 1, 'headerLines', 2); 
fprintf(f_output_id, '%s\n', char(inputline{1})); 

foundline = 0; 
while foundline == 0
    inputline = textscan(f_input_id, '%[^\n]', 1); 
    tmp = char(inputline{1}); 
    fprintf(f_output_id, '%s\n', tmp); 
    if ((length(tmp) > 10) & (tmp(1:11) == 'MEAN VECTOR'))
        foundline = 1; 
    end
end

inputlines = textscan(f_input_id, '%s %d %*f %f', 3); 
param_names = char(inputlines{1}); 
param_flags = inputlines{2};
param_stddevs = inputlines{3};
for i = 1:3
    fprintf(f_output_id, '%s %5d %+19.11e %+19.11e\n', param_names(i,:), ...
               param_flags(i), means(i), param_stddevs(i)); 
end
  
inputline = textscan(f_input_id, '%[^\n]', 1); 
fprintf(f_output_id, '%s\n', char(inputline{1})); 

numlines = 9;
inputlines = textscan(f_input_id, '%s %d %*f %f', numlines); 
param_names = char(inputlines{1}); 
param_flags = inputlines{2};
param_stddevs = inputlines{3};
for i = 1:9
    fprintf(f_output_id, '%s %5d %+19.11e %+19.11e\n', param_names(i,:), ...
               param_flags(i), bcm(i), param_stddevs(i)); 
end

inputline = textscan(f_input_id, '%[^\n]', 1); 
fprintf(f_output_id, '%s\n', char(inputline{1})); 

numlines = 4;
inputlines = textscan(f_input_id, '%s %d %*f %f', numlines); 
param_names = char(inputlines{1}); 
param_flags = inputlines{2};
param_stddevs = inputlines{3};
for i = 1:4
    fprintf(f_output_id, '%s %5d %+19.11e %+19.11e\n', param_names(i,:), ...
               param_flags(i), stddev(i), param_stddevs(i)); 
end

if (numUsedBonds > 1) 

  inputline = textscan(f_input_id, '%[^\n]', 1); 
  fprintf(f_output_id, '%s\n', char(inputline{1})); 

  inputlines = textscan(f_input_id, '%s %d %*f %f', ...
                        double(numUsedBonds - 1)); 
  param_names = char(inputlines{1}); 
  param_flags = inputlines{2};
  param_stddevs = inputlines{3};
  for i = 1:(numUsedBonds - 1) 
    fprintf(f_output_id, '%s %5d %+19.11e %+19.11e\n', param_names(i,:), ...
            param_flags(i), Avec(i), param_stddevs(i)); 
  end

  inputline = textscan(f_input_id, '%[^\n]', 1); 
  fprintf(f_output_id, '%s\n', char(inputline{1})); 

  inputlines = textscan(f_input_id, '%s %d %*f %f', ...
                        double(4*(numUsedBonds - 1))); 
  param_names = char(inputlines{1}); 
  param_flags = inputlines{2};
  param_stddevs = inputlines{3};
  for i = 1:(4*(numUsedBonds - 1))
    fprintf(f_output_id, '%s %5d %+19.11e %+19.11e\n', param_names(i,:), ...
            param_flags(i), Bvec(i), param_stddevs(i)); 
  end

end

inputline = textscan(f_input_id, '%[^\n]', 1); 
fprintf(f_output_id, '%s\n', char(inputline{1})); 

numlines = length(errorSD);
inputlines = textscan(f_input_id, '%s %d %*f %f', numlines); 
param_names = char(inputlines{1}); 
param_flags = inputlines{2};
param_stddevs = inputlines{3};
for i = 1:numlines
    fprintf(f_output_id, '%s %5d %+19.11e %+19.11e\n', param_names(i,:), ...
               param_flags(i), errorSD(i), param_stddevs(i)); 
end


%
% write the log-likelihood value and exit flag.  
%

my_string = 'LOG-LIKELIHOOD VALUE'; 
fprintf(f_output_id, '%s\n', my_string); 
fprintf(f_output_id, '%f\n', loglike); 

my_string = 'OPTIMIZATION EXIT FLAG'; 
fprintf(f_output_id, '%s\n', my_string); 
fprintf(f_output_id, '%d\n', exitflag); 

fclose(f_input_id);
fclose(f_output_id); 

return
