function [latentModel observableModel] = ...
    buildBothModels(muStar, kMatStar, SigmaMatStar, rinfty, ...
                    kEquivMatLatent, A_f, B_fMat, ...
                    Pmat, maturities)
% buildBothModels constructs all parameters under both measures for
% the two rotations.

%
% Greg Duffee
% June 2011
%

numfactors = length(muStar);
numbonds = length(maturities);
numUsedMacro = length(A_f); 

zerovec = zeros(numfactors, 1); 
onevec = ones(numfactors, 1);

bigBmatLatent = createBigB(onevec, kEquivMatLatent, ...
                           max(maturities));
B_yMat = bigBmatLatent(maturities,:);


BMat = [B_yMat; B_fMat]; 

PBMat = Pmat*BMat;

invPBMat = inv(PBMat);

[SigmaMat cholError] = chol(invPBMat*SigmaMatStar*SigmaMatStar'*invPBMat', ...
                            'lower');

latentModel.cholError = cholError;
observableModel.cholError = cholError; 

if (cholError == 0) 

  bigALatent = createBigA(rinfty, zerovec, SigmaMat, ...
                               bigBmatLatent);
  A_y = bigALatent(maturities); 
  
  A = [A_y; A_f];
  
  rho0 = rinfty - onevec'*invPBMat*Pmat*A;
  rho1 = (onevec'*invPBMat)';
  
  kMatEquivStar = PBMat*kEquivMatLatent*invPBMat;
  muStarEquiv = (eye(numfactors) - kMatEquivStar)*Pmat*A;
  
  kMatLatent = invPBMat*kMatStar*PBMat;
  muLatent = invPBMat*(muStar - (eye(numfactors) - kMatStar)*Pmat*A);
  
  BMatStar = BMat*invPBMat;
  AStar = A - BMatStar*Pmat*A;
  
  latentModel.delta0 = rinfty;
  latentModel.delta1 = onevec;
  latentModel.mu = muLatent;
  latentModel.K = kMatLatent;
  latentModel.Sigma = SigmaMat;
  latentModel.muEquiv = zerovec;
  latentModel.KEquiv = kEquivMatLatent;
  latentModel.A = A;
  latentModel.B = BMat;
  
  observableModel.delta0 = rho0;
  observableModel.delta1 = rho1;
  observableModel.mu = muStar;
  observableModel.K = kMatStar;
  observableModel.Sigma = SigmaMatStar;
  observableModel.muEquiv = muStarEquiv;
  observableModel.KEquiv = kMatEquivStar;
  observableModel.A = AStar;
  observableModel.B = BMatStar;
  
end

return
