function [KequivLatent maxAbsEig] = ...
          buildKequivLatent(realSingEig, realRepEig, complexEig)
% BuildKequivLatent constructs equivalent martingale feedback
% matrix in the JSZ latent factor rotation

%
% Greg Duffee
% September 2009, revised June 2011
%

numfactors = length(realSingEig) + 2*length(realRepEig) + ...
    length(complexEig);


%
% kludge here.
%

realSingEig = abs(realSingEig); 

%
%
%

KequivLatent = zeros(numfactors, numfactors);
numRealSingEig = length(realSingEig); 
numRealRepEig = length(realRepEig); 
numComplexPairs = length(complexEig)/2; 

if (numRealSingEig > 0)
  KequivLatent(1:numRealSingEig,1:numRealSingEig) = ...
      diag(sort(realSingEig, 'descend')); 
end

if (numRealRepEig > 0)
  dupEigs = sort(realRepEig, 'descend'); 
  for i = 1:numRealRepEig
    Rmat = [dupEigs(i) 1; 0 dupEigs(i)];
    startIdx = numRealSingEig + 1 + 2*(i-1);
    endIdx = startIdx + 1; 
    KequivLatent(startIdx:endIdx,startIdx:endIdx) = Rmat;
  end
end

if (numComplexPairs > 0)
  alphas = complexEig(1:2:(2*numComplexPairs));
  betas = abs(complexEig(2:2:(2*numComplexPairs)));
  [tmp order] = sort(alphas, 'descend'); 
  alphas = alphas(order);
  betas = betas(order); 
  for i = 1:numComplexPairs
    Rmat = [alphas(i) -betas(i); betas(i) alphas(i)];
    startIdx = numRealSingEig + 2*(numRealRepEig) + 1 + 2*(i-1);
    endIdx = startIdx + 1; 
    KequivLatent(startIdx:endIdx,startIdx:endIdx) = Rmat;
  end
end

maxAbsEig = max(abs(eig(KequivLatent)));

return
