function [score outerProd scoreMat] = ...
    kalman_derivs(freeparams, allparams, free_index, ...
                  observedData, maturities, numFactors, ...
                  macroLoadings, Pmat, tolerance, delta)
% derivatives.

%
% Greg Duffee
% June 2011
%

numFree = length(freeparams);
[T dimData] = size(observedData); 

score = zeros(numFree, 1);
outerProd = zeros(numFree, numFree);
scoreMat = zeros(T, numFree);

numbonds = length(maturities);
numUsedMacro = dimData - numbonds;

for thisParam = 1:numFree
  freeCopy = freeparams;
  freeCopy(thisParam) = freeparams(thisParam) + delta;
  paramvec = poke_freeparams(freeCopy, allparams, free_index); 

  [muStar, kStar, SigmaStar, rinfty, eigs, A_f, B_f, errorSDs] = ...
      unpack_allparams(numFactors, numUsedMacro, paramvec);
  
  [kMatStar SigmaMatStar B_fMat ...
   kEquivMatLatent maxAbsEquivEig H] = ...
      build_matrices(kStar, SigmaStar, B_f, eigs, errorSDs);
  
  if (macroLoadings == 1) 
    [latentModel observableModel] = ...
        buildBothModels(muStar, kMatStar, SigmaMatStar, rinfty, ...
                        kEquivMatLatent, A_f, B_fMat, ...
                        Pmat, maturities); 
  else
    [latentModel observableModel] = ...
        buildBothModelsStar(muStar, kMatStar, SigmaMatStar, rinfty, ...
                            kEquivMatLatent, A_f, B_fMat, ...
                            Pmat, maturities); 
  end

  mu = observableModel.mu;
  Kmat = observableModel.K;
  Sigma = observableModel.Sigma;
  A = observableModel.A;
  B = observableModel.B;
  
  [errors loglikesUp filtered_values cond_vars contemp_vars ...
   normSqrErrors eigFs] = ...
      kalman_recursion_noderiv(observedData, mu, Kmat, Sigma, H, A, B, ...
                               tolerance);

  freeCopy(thisParam) = freeparams(thisParam) - delta;
  paramvec = poke_freeparams(freeCopy, allparams, free_index); 

  [muStar, kStar, SigmaStar, rinfty, eigs, A_f, B_f, errorSDs] = ...
      unpack_allparams(numFactors, numUsedMacro, paramvec);
  
  [kMatStar SigmaMatStar B_fMat ...
   kEquivMatLatent maxAbsEquivEig H] = ...
      build_matrices(kStar, SigmaStar, B_f, eigs, errorSDs);
  
  if (macroLoadings == 1) 
    [latentModel observableModel] = ...
        buildBothModels(muStar, kMatStar, SigmaMatStar, rinfty, ...
                        kEquivMatLatent, A_f, B_fMat, ...
                        Pmat, maturities); 
  else
    [latentModel observableModel] = ...
        buildBothModelsStar(muStar, kMatStar, SigmaMatStar, rinfty, ...
                            kEquivMatLatent, A_f, B_fMat, ...
                            Pmat, maturities); 
  end

  mu = observableModel.mu;
  Kmat = observableModel.K;
  Sigma = observableModel.Sigma;
  A = observableModel.A;
  B = observableModel.B;
  
  [errors loglikesDown filtered_values cond_vars contemp_vars ...
   normSqrErrors eigFs] = ...
      kalman_recursion_noderiv(observedData, mu, Kmat, Sigma, H, A, B, ...
                               tolerance);

  scoreMat(:,thisParam) = (loglikesUp - loglikesDown)/(2*delta);
end

score = sum(scoreMat);
outerProd = scoreMat' * scoreMat;
    
return

