function [funval loglikes filtered_values errors] = ...
    kalman_only_loglike(freeparams, allparams, free_index, ...
                        observedData, maturities, numFactors, ...
                        macroLoadings, Pmat, tolerance)
% KALMAN_ONLY_LOGLIKE returns the (negative of the) Kalman filter
% log-likelihood.  This is used for standard ML estimation.  

%
% Greg Duffee
% June 2011
%

[T dimData] = size(observedData); 
numbonds = length(maturities);
numUsedMacro = dimData - numbonds;

%disp('free parameters');
%disp(freeparams');

paramvec = poke_freeparams(freeparams, allparams, free_index); 

[muStar, kStar, SigmaStar, rinfty, eigs, A_f, B_f, errorSDs] = ...
    unpack_allparams(numFactors, numUsedMacro, paramvec);

[kMatStar SigmaMatStar B_fMat ...
          kEquivMatLatent maxAbsEquivEig H] = ...
    build_matrices(kStar, SigmaStar, B_f, eigs, errorSDs);


%fprintf(1, 'max abs equiv eigen = %8.3f\n', maxAbsEquivEig);


if (maxAbsEquivEig < 1.1) 

  if (macroLoadings == 1) 
    [latentModel observableModel] = ...
        buildBothModels(muStar, kMatStar, SigmaMatStar, rinfty, ...
                        kEquivMatLatent, A_f, B_fMat, ...
                        Pmat, maturities); 
  else
    [latentModel observableModel] = ...
        buildBothModelsStar(muStar, kMatStar, SigmaMatStar, rinfty, ...
                        kEquivMatLatent, A_f, B_fMat, ...
                        Pmat, maturities); 
  end

  if (latentModel.cholError == 0) 
  
    mu = observableModel.mu;
    Kmat = observableModel.K;
    Sigma = observableModel.Sigma;
    A = observableModel.A;
    B = observableModel.B;
    
    [errors loglikes filtered_values cond_vars contemp_vars ...
     normSqrErrors eigFs] = ...
        kalman_recursion_noderiv(observedData, mu, Kmat, Sigma, H, A, B, ...
                                 tolerance);
  else
    
    loglikes = -1e10; 
  end
  
else
  loglikes = -1e9*maxAbsEquivEig;
end

funval = -sum(loglikes);

return

