function [freeparams bigparamvec free_index] = setup_initparams( ...
    muStar_vals, kStar_vals, SigmaStar_vals, rinfty_val, eig_vals, ...
    A_f_vals, B_f_vals, ...
    errorSD_vals, muStar_flag, kStar_flag, SigmaStar_flag, ...
    rinfty_flag, eig_flag, A_f_flag, B_f_flag, errorSD_flag, ...
    observedData, ols_flag, Pmat)
%
% SETUP_INITPARAMS gets the initial vectors of free parameters and
% unrestricted parameters.  
%

%
% Greg Duffee
% June 2011
%

muStar = muStar_vals;
kStar = kStar_vals; 
SigmaStar = SigmaStar_vals; 
rinfty = rinfty_val;  

if (ols_flag == 1)
  rinfty = abs(mean(observedData(:,1))); 
  numobs = size(observedData, 1); 
  numfactors = length(muStar); 
  numUsedMacro = length(A_f_vals); 
  used_data = observedData*Pmat';
  used_macro = observedData(:,(end-numUsedMacro+1):end); 
  cnstnt = ones(numobs-1,1); 
  allResids = zeros(numobs-1, numfactors); 
  KmatStar = zeros(numfactors, numfactors); 
  for i = 1:numfactors
    [coefs bint resids] = regress(used_data(2:end,i), ...
                              [cnstnt used_data(1:(end-1),:)]);
    muStar(i) = coefs(1);
    KmatStar(i,:) = coefs(2:end)'; 
    allResids(:,i) = resids; 
  end
  eigenvals = eig(KmatStar); 
  Omega = cov(allResids, 1); 
  kStar = vec(KmatStar); 
  SigmaMatStar = (chol(Omega))';
  SigmaStar = [];
  for i = 1:numfactors
    for j = 1:i
      SigmaStar = [SigmaStar; SigmaMatStar(i,j)];
    end
  end
  if (max(abs(eigenvals)) > 1.0) 
    fprintf(1, 'Warning:  OLS setting is nonstationary.\n');
  end
end

[bigparamvec free_index] = pack_allparams( ...
    muStar, kStar, SigmaStar, rinfty, eig_vals, A_f_vals, B_f_vals, ...
    errorSD_vals, ...
    muStar_flag, kStar_flag, SigmaStar_flag, rinfty_flag, ...   
    eig_flag, A_f_flag, B_f_flag, errorSD_flag); 

freeparams = pop_freeparams(bigparamvec, free_index); 

return

