function write_output(inputfile, outputfile, muStar, kStar, SigmaStar, ...
                      rinfty, eigs, A_fStar, B_fStar, errorSDs, ...
                      loglike, exitflag) 
%
% WRITE_OUTPUT is a routine that writes the results of estimatng 
% a dynamic term structure model.  The results are written to 'outputfile'.

% Greg Duffee
% September 2009, revised June 2011
%

numfactors = length(eigs); 

%
% open the input and output files
%

f_input_id = fopen(inputfile, 'rt');
f_output_id = fopen(outputfile, 'wt'); 

%
% write the header
%

my_string = '% Output file for estimation of dynamic term structure model'; 
fprintf(f_output_id, '%s\n', my_string); 
my_string = ['% ' datestr(now)]; 
fprintf(f_output_id, '%s\n', my_string); 

%
% copy input file until parameters are reached.  

inputline = textscan(f_input_id, '%[^\n]', 1, 'headerLines', 2); 
fprintf(f_output_id, '%s\n', char(inputline{1})); 

foundline = 0; 
while foundline == 0
    inputline = textscan(f_input_id, '%[^\n]', 1); 
    tmp = char(inputline{1}); 
    fprintf(f_output_id, '%s\n', tmp); 
    if ((length(tmp) > 21) & (tmp(1:22) == 'CONSTANT IN PHYSICAL D'))
        foundline = 1; 
    end
end

inputlines = textscan(f_input_id, '%s %d %*f %f', double(numfactors)); 
param_names = char(inputlines{1}); 
param_flags = inputlines{2};
param_stddevs = inputlines{3};
for i = 1:numfactors
    fprintf(f_output_id, '%s %5d %+19.11e %+19.11e\n', param_names(i,:), ...
               param_flags(i), muStar(i), param_stddevs(i)); 
end
  
inputline = textscan(f_input_id, '%[^\n]', 1); 
fprintf(f_output_id, '%s\n', char(inputline{1})); 

numlines = double(numfactors^2);
inputlines = textscan(f_input_id, '%s %d %*f %f', numlines); 
param_names = char(inputlines{1}); 
param_flags = inputlines{2};
param_stddevs = inputlines{3};
for i = 1:(numfactors^2)
    fprintf(f_output_id, '%s %5d %+19.11e %+19.11e\n', param_names(i,:), ...
               param_flags(i), kStar(i), param_stddevs(i)); 
end

inputline = textscan(f_input_id, '%[^\n]', 1); 
fprintf(f_output_id, '%s\n', char(inputline{1})); 

numlines = double(numfactors*(1+numfactors)/2); 
inputlines = textscan(f_input_id, '%s %d %*f %f', numlines); 
param_names = char(inputlines{1}); 
param_flags = inputlines{2};
param_stddevs = inputlines{3};
for i = 1:(numfactors*(1+numfactors)/2)
    fprintf(f_output_id, '%s %5d %+19.11e %+19.11e\n', param_names(i,:), ...
               param_flags(i), SigmaStar(i), param_stddevs(i)); 
end

inputline = textscan(f_input_id, '%[^\n]', 1); 
fprintf(f_output_id, '%s\n', char(inputline{1})); 

inputlines = textscan(f_input_id, '%s %d %*f %f', 1); 
param_names = char(inputlines{1}); 
param_flags = inputlines{2};
param_stddevs = inputlines{3};
fprintf(f_output_id, '%s %5d %+19.11e %+19.11e\n', param_names(1,:), ...
        param_flags(1), rinfty, param_stddevs(1)); 


inputline = textscan(f_input_id, '%[^\n]', 1); 
fprintf(f_output_id, '%s\n', char(inputline{1})); 

inputlines = textscan(f_input_id, '%s %d %*f %f', double(numfactors)); 
param_names = char(inputlines{1}); 
param_flags = inputlines{2};
param_stddevs = inputlines{3};
for i = 1:numfactors
  fprintf(f_output_id, '%s %5d %+19.11e %+19.11e\n', param_names(i,:), ...
          param_flags(i), eigs(i), param_stddevs(i)); 
end

inputline = textscan(f_input_id, '%[^\n]', 1); 
fprintf(f_output_id, '%s\n', char(inputline{1})); 

numUsedMacro = length(A_fStar); 

inputlines = textscan(f_input_id, '%s %d %*f %f', double(numUsedMacro)); 
param_names = char(inputlines{1}); 
param_flags = inputlines{2};
param_stddevs = inputlines{3};
for i = 1:numUsedMacro
  fprintf(f_output_id, '%s %5d %+19.11e %+19.11e\n', param_names(i,:), ...
          param_flags(i), A_fStar(i), param_stddevs(i)); 
end

inputline = textscan(f_input_id, '%[^\n]', 1); 
fprintf(f_output_id, '%s\n', char(inputline{1})); 

inputlines = textscan(f_input_id, '%s %d %*f %f', ...
                      double(numfactors*numUsedMacro)); 
param_names = char(inputlines{1}); 
param_flags = inputlines{2};
param_stddevs = inputlines{3};
for i = 1:(numfactors*numUsedMacro)
  fprintf(f_output_id, '%s %5d %+19.11e %+19.11e\n', param_names(i,:), ...
          param_flags(i), B_fStar(i), param_stddevs(i)); 
end


numObserved = length(errorSDs); 

inputline = textscan(f_input_id, '%[^\n]', 1); 
fprintf(f_output_id, '%s\n', char(inputline{1})); 

inputlines = textscan(f_input_id, '%s %d %*f %f', ...
                      double(numObserved)); 
param_names = char(inputlines{1}); 
param_flags = inputlines{2};
param_stddevs = inputlines{3};
for i = 1:numObserved
    fprintf(f_output_id, '%s %5d %+19.11e %+19.11e\n', param_names(i,:), ...
               param_flags(i), errorSDs(i), param_stddevs(i)); 
end

%
% write the log-likelihood value and exit flag.  
%

my_string = 'LOG-LIKELIHOOD VALUE'; 
fprintf(f_output_id, '%s\n', my_string); 
fprintf(f_output_id, '%f\n', loglike); 

my_string = 'OPTIMIZATION EXIT FLAG'; 
fprintf(f_output_id, '%s\n', my_string); 
fprintf(f_output_id, '%d\n', exitflag); 

fclose(f_input_id);
fclose(f_output_id); 

return
