% Greg Duffee
%
% Construct ARMA-smoothed measures of inflation.  
%

%=======================================================================

clear
close all

%=======================================================================

outputFile = '..\excelFiles\inflation_Q_Measures.xls';

%=======================================================================

quarterlyData = xlsread('..\excelFiles\quarterlydata.xls', ...
                        'quarterlyData', 'A13:Y264');

date = quarterlyData(:,1);
inflation_nondurServ = quarterlyData(:,9);
inflation_GDP = quarterlyData(:,10); 
inflation_CPI = quarterlyData(:,11);
inflation_CPIexFE = quarterlyData(:,12);

yield_3m = quarterlyData(:,13); 


%=======================================================================
%
% ARMA filtering.  
%
%=======================================================================

inflationMeasures = NaN*zeros(length(yield_3m), 4);

% For C, GDP, and CPI indexes, filter using 1952Q1 through 2010Q4.
% Use one-step-ahead forecasts as current values.  For CPI-ex FE,
% use 1957Q2 through 2010Q4.  

OKsamp = isfinite(yield_3m); 

tmpOKsamp = isfinite([repmat(NaN, 16, 1); ones((252-16),1)]);

constant = 1; 
ar = 1;
ma = 1; 

y = inflation_nondurServ(tmpOKsamp); 
[parameters errors] = armaxfilter(y,constant,ar,ma); 
C_ARMA = parameters(1) + parameters(2)*y(2:end) + ...
    parameters(3)*errors;
y = inflation_GDP(tmpOKsamp); 
[parameters errors] = armaxfilter(y,constant,ar,ma); 
GDP_ARMA = parameters(1) + parameters(2)*y(2:end) + ...
    parameters(3)*errors;
y = inflation_CPI(tmpOKsamp); 
[parameters errors] = armaxfilter(y,constant,ar,ma); 
CPI_ARMA = parameters(1) + parameters(2)*y(2:end) + ...
    parameters(3)*errors;

inflationMeasures(OKsamp,1) = C_ARMA;
inflationMeasures(OKsamp,2) = GDP_ARMA;
inflationMeasures(OKsamp,3) = CPI_ARMA;

tmpOKsamp = isfinite(inflation_CPIexFE);

y = inflation_CPIexFE(tmpOKsamp); 
[parameters errors] = armaxfilter(y,constant,ar,ma); 
CPIexFE_ARMA = parameters(1) + parameters(2)*y(2:end) + ...
    parameters(3)*errors;

inflationMeasures(tmpOKsamp,4) = [NaN; CPIexFE_ARMA];

inflationCorrs1 = corr(inflationMeasures(:,1:3), 'rows', 'complete');
inflationCorrs2 = corr(inflationMeasures, 'rows', 'complete');

%---------------------------------------------------------------------%
%
%  Write quarterly data file.
%
%---------------------------------------------------------------------%

writeData = [date inflationMeasures];

header1 = {['1. One-q-ahead forecast of inflation in nondurables ' ...
   'plus services, from ARMA(1,1) model estimated for 1952Q1 through ' ...
   '2010Q4']};
header2 = {['2. One-q-ahead forecast of inflation in GDP, ' ...
   'from ARMA(1,1) model estimated for 1952Q1 through ' ...
   '2010Q4']};
header3 = {['3. One-q-ahead forecast of CPI inflation, ' ...
   'from ARMA(1,1) model estimated for 1952Q1 through ' ...
   '2010Q4']};
header4 = {['4. One-q-ahead forecast of CPI ex F and E inflation, ' ...
   'from ARMA(1,1) model estimated for 1957Q2 through ' ...
   '2010Q4']};

header = {'Date', '1', '2', '3', '4'};

xlswrite(outputFile, header1,  'quarterlyData',  'A1');
xlswrite(outputFile, header2,  'quarterlyData',  'A2');
xlswrite(outputFile, header3,  'quarterlyData',  'A3');
xlswrite(outputFile, header4,  'quarterlyData',  'A4');

xlswrite(outputFile, header, 'quarterlyData', 'A7:E7');

xlswrite(outputFile, writeData, 'quarterlyData', 'A8:E259');

