% Greg Duffee
%
% Correlations among quarterly measures of economic activity

%=======================================================================

clear
close all

%=======================================================================

% file names

cycleFile = char('..\excelFiles\cycleActivity_Q_Measures.xls');
quarterFile = char('..\excelFiles\quarterlydata.xls'); 
inflationFile = char('..\excelFiles\inflation_Q_measures.xls'); 

cyclePage = char('quarterlyData'); 

%=======================================================================

% load data

cycleData = xlsread(cycleFile, cyclePage, 'B32:F266');
someGrowthData = xlsread(cycleFile, cyclePage, 'G32:I266'); 

CFNAIData = xlsread(quarterFile, cyclePage, 'F30:F264');
numMissing = size(someGrowthData,1) - length(CFNAIData);
CFNAIData = [NaN*ones(numMissing, 1); CFNAIData];

LNData = xlsread(quarterFile, cyclePage, 'H30:H264');
LNData = [LNData; NaN*ones(12, 1)];
numMissing = size(someGrowthData,1) - length(LNData);
LNData = [NaN*ones(numMissing, 1); LNData];

growthData = [someGrowthData CFNAIData -LNData]; 

gapAndGrowthMeasures = [cycleData growthData];

%====================================================
%
% Correlations among measures.  Order is
%  [ output gap measures ]
%  detrended log real GDP
%  detrended log IP
%  HP filtered log real GDP
%  HP filtered log IP
%  demeaned capacity utilization
%  [economic growth measures]
%  log-differenced per capita real consumption on nondurables and
%  services
%  log-differenced real GDP
%  log-differenced IP
%  the CFNAI
%  the Ludvigson and Ng macroeconomic activity principal
%  component. 
%====================================================

corrMatrix = corr(gapAndGrowthMeasures, 'rows', 'pairwise'); 

fprintf(1, 'Correlation matrix, pairwise available data\n'); 
fprintf(1, '-------------------------------------------\n'); 
for i = 1:size(corrMatrix,1)
  for j = 1:size(corrMatrix,2)
    fprintf(1, ' %8.2f', corrMatrix(i,j));
  end
  fprintf(1, '\n'); 
end



%====================================================
%
% Related information, mentioned in the text, but not the table.
% The first-order autocorrelations of the variables.
%
%====================================================

fprintf(1, '\n'); 
fprintf(1, 'First-order autocorrelations\n'); 
fprintf(1, '----------------------------\n'); 
for i = 1:size(gapAndGrowthMeasures, 2)
  lead = gapAndGrowthMeasures(2:end,i);
  lag = gapAndGrowthMeasures(1:(end-1),i);
  validObs = isfinite(lead) & isfinite(lag);
  yvec = lead(validObs);
  xmat = [ones(length(yvec),1) lag(validObs)];
  regResults = ols(yvec, xmat);
  fprintf(1, ' %4d    %7.2f\n', i, regResults.beta(2));
end
fprintf(1, '\n');


