% Greg Duffee
%
% Table 12.

%=======================================================================

clear
close all

%=======================================================================

% file names

monthFile = char('..\excelFiles\monthlydata.xls');
predictorFile = char('..\excelFiles\monthlyPredictiveVars.xls'); 

monthPage = char('monthlyData'); 

%=======================================================================
%
% load data
%
%=======================================================================


longBondExcessReturn = xlsread(monthFile, monthPage, 'O67:O768');
stockExcessReturn = xlsread(monthFile, monthPage, 'P67:P768');
% both of these series are July 1952 through December 2010.  

predictorVars = xlsread(predictorFile, monthPage, 'B19:W720');
% variables from June 1952 through November 2010. 

%====================================================
%
% Set up forecasting regressions 
%
%====================================================

numObs = length(longBondExcessReturn); 

sample1 = (1:numObs)';
sample2 = (117:numObs)';
sample3 = (140:(numObs-35))';

useConstant = 1; 
useNW = 1; 
numLags = 5; 

%====================================================
%
% Predict long bond excess returns 
%
%====================================================

fprintf(1, '\n\n'); 
fprintf(1, ...
        'One-month-ahead predictions of excess bond returns\n'); 
fprintf(1, '\n'); 


% 1.  Slope

theseReturns = longBondExcessReturn(sample1);
xmat = predictorVars(sample1,1);

validObs = isfinite(theseReturns) & ...
    (sum(isfinite(xmat), 2) == size(xmat, 2));
yvec = theseReturns(validObs);
X = xmat(validObs,:); 

regOutput = ols_robust(theseReturns, X, useConstant, numLags, ...
                       useNW);


fprintf(1, '          '); 
fprintf(1, ' T  adj r^2  test stat  Coef (if one)\n'); 
fprintf(1, '----------'); 
fprintf(1, '-------------------------------------\n'); 
fprintf(1, 'Slope    %d %8.4f %8.3f %8.3f\n', ...
        length(yvec), regOutput.adjrsqr, ...
        regOutput.hansen_pval, regOutput.beta(2)); 


% 2. 12-month averages of five forward rates


xmat = predictorVars(sample1,7:11);

validObs = isfinite(theseReturns) & ...
    (sum(isfinite(xmat), 2) == size(xmat, 2));

yvec = theseReturns(validObs);
X = xmat(validObs,:); 

regOutput = ols_robust(yvec, X, useConstant, numLags, ...
                       useNW);


fprintf(1, 'Forwards %d %8.4f %8.3f     ----\n', ...
        length(yvec), regOutput.adjrsqr, ...
        regOutput.hansen_pval); 


% 3. Conditional SD of long-bond yields


xmat = predictorVars(sample1,14);

validObs = isfinite(theseReturns) & ...
    (sum(isfinite(xmat), 2) == size(xmat, 2));

yvec = theseReturns(validObs);
X = xmat(validObs,:); 

regOutput = ols_robust(yvec, X, useConstant, numLags, ...
                       useNW);


fprintf(1, 'Cond SD  %d %8.4f %8.3f %8.3f\n', ...
        length(yvec), regOutput.adjrsqr, ...
        regOutput.hansen_pval, regOutput.beta(2)); 



% 4. Log change of IP


xmat = predictorVars(sample1,12);

validObs = isfinite(theseReturns) & ...
    (sum(isfinite(xmat), 2) == size(xmat, 2));

yvec = theseReturns(validObs);
X = xmat(validObs,:); 

regOutput = ols_robust(yvec, X, useConstant, numLags, ...
                       useNW);


fprintf(1, 'DifF IP  %d %8.4f %8.3f %8.3f\n', ...
        length(yvec), regOutput.adjrsqr, ...
        regOutput.hansen_pval, regOutput.beta(2)); 


% 5. Average log change of IP


xmat = predictorVars(sample1,13);

validObs = isfinite(theseReturns) & ...
    (sum(isfinite(xmat), 2) == size(xmat, 2));

yvec = theseReturns(validObs);
X = xmat(validObs,:); 

regOutput = ols_robust(yvec, X, useConstant, numLags, ...
                       useNW);


fprintf(1, 'Avg IP   %d %8.4f %8.3f %8.3f\n', ...
        length(yvec), regOutput.adjrsqr, ...
        regOutput.hansen_pval, regOutput.beta(2)); 



% 6. LN PCs

theseReturns = longBondExcessReturn(sample3);

xmat = predictorVars(sample3,15:22);

validObs = isfinite(theseReturns) & ...
    (sum(isfinite(xmat), 2) == size(xmat, 2));

yvec = theseReturns(validObs);
X = xmat(validObs,:); 

regOutput = ols_robust(yvec, X, useConstant, numLags, ...
                       useNW);


fprintf(1, 'LN PCs   %d %8.4f %8.3f     ----\n', ...
        length(yvec), regOutput.adjrsqr, ...
        regOutput.hansen_pval); 


% 7. 12-month averages of five forward rates, shorter sample. 


theseReturns = longBondExcessReturn(sample3);

xmat = predictorVars(sample3,7:11);

validObs = isfinite(theseReturns) & ...
    (sum(isfinite(xmat), 2) == size(xmat, 2));

yvec = theseReturns(validObs);
X = xmat(validObs,:); 

regOutput = ols_robust(yvec, X, useConstant, numLags, ...
                       useNW);


fprintf(1, 'Forwards %d %8.4f %8.3f     ----\n', ...
        length(yvec), regOutput.adjrsqr, ...
        regOutput.hansen_pval); 


% 8. First LN PC


theseReturns = longBondExcessReturn(sample3);

xmat = predictorVars(sample3,15);

validObs = isfinite(theseReturns) & ...
    (sum(isfinite(xmat), 2) == size(xmat, 2));

yvec = theseReturns(validObs);
X = xmat(validObs,:); 

regOutput = ols_robust(yvec, X, useConstant, numLags, ...
                       useNW);


fprintf(1, '1st PC   %d %8.4f %8.3f %8.3f\n', ...
        length(yvec), regOutput.adjrsqr, ...
        regOutput.hansen_pval, regOutput.beta(2)); 


% 9. Log change of IP


theseReturns = longBondExcessReturn(sample3);

xmat = predictorVars(sample3,12);

validObs = isfinite(theseReturns) & ...
    (sum(isfinite(xmat), 2) == size(xmat, 2));

yvec = theseReturns(validObs);
X = xmat(validObs,:); 

regOutput = ols_robust(yvec, X, useConstant, numLags, ...
                       useNW);


fprintf(1, 'DifF IP  %d %8.4f %8.3f %8.3f\n', ...
        length(yvec), regOutput.adjrsqr, ...
        regOutput.hansen_pval, regOutput.beta(2)); 

%====================================================
%
% Predict stock excess returns 
%
%====================================================

fprintf(1, '\n\n'); 
fprintf(1, ...
        'One-month-ahead predictions of excess stock returns\n'); 
fprintf(1, '\n'); 


% 1.  Slope

theseReturns = stockExcessReturn(sample1);
xmat = predictorVars(sample1,1);

validObs = isfinite(theseReturns) & ...
    (sum(isfinite(xmat), 2) == size(xmat, 2));
yvec = theseReturns(validObs);
X = xmat(validObs,:); 

regOutput = ols_robust(theseReturns, X, useConstant, numLags, ...
                       useNW);


fprintf(1, '          '); 
fprintf(1, ' T  adj r^2  test stat  Coef (if one)\n'); 
fprintf(1, '----------'); 
fprintf(1, '-------------------------------------\n'); 
fprintf(1, 'Slope    %d %8.4f %8.3f %8.3f\n', ...
        length(yvec), regOutput.adjrsqr, ...
        regOutput.hansen_pval, regOutput.beta(2)); 


% 2. 12-month averages of five forward rates


xmat = predictorVars(sample1,7:11);

validObs = isfinite(theseReturns) & ...
    (sum(isfinite(xmat), 2) == size(xmat, 2));

yvec = theseReturns(validObs);
X = xmat(validObs,:); 

regOutput = ols_robust(yvec, X, useConstant, numLags, ...
                       useNW);


fprintf(1, 'Forwards %d %8.4f %8.3f     ----\n', ...
        length(yvec), regOutput.adjrsqr, ...
        regOutput.hansen_pval); 


% 3. Log change of IP


xmat = predictorVars(sample1,12);

validObs = isfinite(theseReturns) & ...
    (sum(isfinite(xmat), 2) == size(xmat, 2));

yvec = theseReturns(validObs);
X = xmat(validObs,:); 

regOutput = ols_robust(yvec, X, useConstant, numLags, ...
                       useNW);


fprintf(1, 'DifF IP  %d %8.4f %8.3f %8.3f\n', ...
        length(yvec), regOutput.adjrsqr, ...
        regOutput.hansen_pval, regOutput.beta(2)); 



% 4. LN PCs

theseReturns = stockExcessReturn(sample3);

xmat = predictorVars(sample3,15:22);

validObs = isfinite(theseReturns) & ...
    (sum(isfinite(xmat), 2) == size(xmat, 2));

yvec = theseReturns(validObs);
X = xmat(validObs,:); 

regOutput = ols_robust(yvec, X, useConstant, numLags, ...
                       useNW);


fprintf(1, 'LN PCs   %d %8.4f %8.3f     ----\n', ...
        length(yvec), regOutput.adjrsqr, ...
        regOutput.hansen_pval); 


% 5. First LN PC


theseReturns = stockExcessReturn(sample3);

xmat = predictorVars(sample3,15);

validObs = isfinite(theseReturns) & ...
    (sum(isfinite(xmat), 2) == size(xmat, 2));

yvec = theseReturns(validObs);
X = xmat(validObs,:); 

regOutput = ols_robust(yvec, X, useConstant, numLags, ...
                       useNW);


fprintf(1, '1st PC   %d %8.4f %8.3f %8.3f\n', ...
        length(yvec), regOutput.adjrsqr, ...
        regOutput.hansen_pval, regOutput.beta(2)); 


