% Greg Duffee
%
% Table 13.

%=======================================================================

clear
close all

%=======================================================================

% file names

cycleFile = char('..\excelFiles\cycleActivity_Q_Measures.xls');
consumptionFile = char('..\excelFiles\cGrowth_not_per_capita.xls');
quarterFile = char('..\excelFiles\quarterlydata.xls'); 

cyclePage = char('quarterlyData'); 

%=======================================================================
%
% load data
%
%=======================================================================

longBondExcessReturn = xlsread(quarterFile, cyclePage, 'X30:X264');
stockExcessReturn = xlsread(quarterFile, cyclePage, 'Y30:Y264'); 

CPIinflation = xlsread(quarterFile, cyclePage, 'K30:K264'); 
yield3m = xlsread(quarterFile, cyclePage, 'M30:M264');

someGrowthData = xlsread(cycleFile, cyclePage, 'G32:I266'); 

contempCgrowth_a = someGrowthData(:,1);

contempCgrowth_b = xlsread(consumptionFile, cyclePage, 'C23:C257');

%=======================================================================
%
% Construct ex-post real rate.  Lagged 3-month yield per quarter
% less current inflation. 
%
%=======================================================================

exPostRR = [NaN; yield3m(1:(end-1))/4] - CPIinflation;

sampleEnd2004 = (1:211)';
sampleEnd2010 = (1:235)'; 

phi = 0.97; 
maxLag = 40;
  
useConstant = 1; 
useNW = 1; 
numLags = 6; 

%======================================================================
% 
% Do this with two measures of real consumption growth.  One is
% per-capita consumption growth, the other is not per-capita.

for thisMeasure = 1:2

  fprintf(1, '\n'); 

  if (thisMeasure == 1) 
    contempCgrowth = contempCgrowth_a; 
    fprintf(1, 'Results for per capita consumption growth\n'); 
  end
  if (thisMeasure == 2) 
    contempCgrowth = contempCgrowth_b; 
    fprintf(1, 'Results for consumption growth (not per capita)\n'); 
  end

  %=======================================================================
  %
  % Construct a measure of surplus consumption, using growth of per
  % capita real consumption on nondurables and services.  
  %
  % I follow Wachter 2006.  However, she appears to use a lag of
  % consumption growth -- I do not.
  %
  %======================================================================
  
  surplus = contempCgrowth;
  for i = 1:(maxLag-1)
    surplus = surplus + (phi^i)*lag(contempCgrowth,i,NaN);
  end

  %=======================================================================
  %
  % Forecasting with surplus.
  %
  %======================================================================
  
  lagSurplus = [NaN; surplus(1:(end-1))];

  %
  % Predict ex-post real rate in quarter t using surplus consumption
  % in t-1. 
  %
  
  % short-sample regression 
  
  usedRR = exPostRR(sampleEnd2004);
  usedLagSurplus = lagSurplus(sampleEnd2004);
  
  validObs = isfinite(usedRR) & isfinite(usedLagSurplus);
  
  yvec = usedRR(validObs);
  xmat = usedLagSurplus(validObs);
  
  regResults1 = ols_robust(yvec, xmat, useConstant, numLags, ...
                           useNW);
  
  % long-sample regression 
  
  usedRR = exPostRR(sampleEnd2010);
  usedLagSurplus = lagSurplus(sampleEnd2010);
  
  validObs = isfinite(usedRR) & isfinite(usedLagSurplus);
  
  yvec = usedRR(validObs);
  xmat = usedLagSurplus(validObs);
  
  regResults2 = ols_robust(yvec, xmat, useConstant, numLags, ...
                           useNW);
  
  
  %
  % Predict excess bond return in quarter t using surplus consumption
  % in t-1. 
  %
  
  % short-sample regression 
  
  usedRet = longBondExcessReturn(sampleEnd2004);
  usedLagSurplus = lagSurplus(sampleEnd2004);
  
  validObs = isfinite(usedRet) & isfinite(usedLagSurplus);
  
  yvec = usedRet(validObs);
  xmat = usedLagSurplus(validObs);
  
  regResults3 = ols_robust(yvec, xmat, useConstant, 0, useNW);
  
  % long-sample regression 
  
  usedRet = longBondExcessReturn(sampleEnd2010);
  usedLagSurplus = lagSurplus(sampleEnd2010);
  
  validObs = isfinite(usedRR) & isfinite(usedLagSurplus);
  
  yvec = usedRet(validObs);
  xmat = usedLagSurplus(validObs);
  
  regResults4 = ols_robust(yvec, xmat, useConstant, 0, useNW);
  
  
  fprintf(1, '\n'); 
  
  fprintf(1, 'Dependent var   1962Q2--2004Q4');
  fprintf(1, '      1962Q2--2010Q4\n'); 
  fprintf(1, '------------------------------');
  fprintf(1, '--------------------\n'); 
  fprintf(1, 'Ex-post real rate   %8.3f     %8.3f\n', ...
          regResults1.beta(2), regResults2.beta(2)); 
  fprintf(1, '                     (%6.2f)     (%6.2f)\n', ...
          regResults1.hansen_tstat(2), ...
          regResults2.hansen_tstat(2));
  fprintf(1, '\n'); 
  fprintf(1, 'Excess bond return  %8.3f     %8.3f\n', ...
          regResults3.beta(2), regResults4.beta(2)); 
  fprintf(1, '                     (%6.2f)     (%6.2f)\n', ...
          regResults3.white_tstat(2), ...
          regResults4.white_tstat(2));
  fprintf(1, '\n'); 

end

