% Greg Duffee
%
% Table 2.

%=======================================================================

clear
close all

%=======================================================================

% file names

cycleFile = char('..\excelFiles\cycleActivity_Q_Measures.xls');
quarterFile = char('..\excelFiles\quarterlydata.xls'); 
inflationFile = char('..\excelFiles\inflation_Q_measures.xls'); 

cyclePage = char('quarterlyData'); 

%=======================================================================
%
% load data
%
%=======================================================================

cycleData = xlsread(cycleFile, cyclePage, 'B32:F266');
someGrowthData = xlsread(cycleFile, cyclePage, 'G32:I266'); 

CFNAIData = xlsread(quarterFile, cyclePage, 'F30:F264');
numMissing = size(someGrowthData,1) - length(CFNAIData);
CFNAIData = [NaN*ones(numMissing, 1); CFNAIData];

LNData = xlsread(quarterFile, cyclePage, 'H30:H264');
LNData = [LNData; NaN*ones(12, 1)];
numMissing = size(someGrowthData,1) - length(LNData);
LNData = [NaN*ones(numMissing, 1); LNData];

growthData = [someGrowthData CFNAIData -LNData]; 

inflationData = xlsread(quarterFile, cyclePage, 'J30:L264');
ARMAinflationData = xlsread(inflationFile, cyclePage, ...
                            'B25:D259');
% these data are one-quarter-ahead predictions of inflation from an
% ARMA(1,1) model.  The three series are inflation from an index of
% nondurables and services, inflation from the GDP deflator, and
% inflation from the CPI respectively.  

yields = xlsread(quarterFile, cyclePage, 'M30:R264');
numYields = size(yields, 2); 

sample_1952Q2_2010Q4 = (1:235)';
sample_1986Q1_2007Q4 = (136:223)';

%====================================================
%
% Projections of output gap on term structure
%
%====================================================

numSeries = size(cycleData, 2);
% full sample 

T = length(sample_1952Q2_2010Q4);
xmat = [ones(T,1) yields(sample_1952Q2_2010Q4,:)];

regR2s = zeros(numSeries, 1); 
residAR1s = zeros(numSeries, 1);

for i = 1:numSeries
  yvec = cycleData(sample_1952Q2_2010Q4,i);
  validObs = isfinite(yvec); 
  regResults = ols(yvec(validObs), xmat(validObs,:));
  regR2s(i) = regResults.rsqr;
  resids = regResults.resid;
  reg2 = ols(resids(2:end), [ones(length(resids(2:end)),1) ...
                      resids(1:(end-1))]);
  residAR1s(i) = reg2.beta(2);
end

fprintf(1, '\n');
fprintf(1, 'Projections of measures of the output gap on');
fprintf(1, ' the term structure\n\n'); 
fprintf(1, '1952Q2 through 2010Q4\n\n'); 
fprintf(1, 'Index      R2        AR(1) of resids\n'); 
fprintf(1, '------------------------------------\n'); 
for i = 1:numSeries
  fprintf(1, '%d %12.2f %12.2f\n', i, regR2s(i), residAR1s(i));
end

% shorter sample

T = length(sample_1986Q1_2007Q4);
xmat = [ones(T,1) yields(sample_1986Q1_2007Q4,:)];

regR2s = zeros(numSeries, 1); 
residAR1s = zeros(numSeries, 1);

for i = 1:numSeries
  yvec = cycleData(sample_1986Q1_2007Q4,i);
  validObs = isfinite(yvec); 
  regResults = ols(yvec(validObs), xmat(validObs,:));
  regR2s(i) = regResults.rsqr;
  resids = regResults.resid;
  reg2 = ols(resids(2:end), [ones(length(resids(2:end)),1) ...
                      resids(1:(end-1))]);
  residAR1s(i) = reg2.beta(2);
end

fprintf(1, '\n');
fprintf(1, '1986Q1 through 2007Q4\n\n'); 
fprintf(1, 'Index      R2        AR(1) of resids\n'); 
fprintf(1, '------------------------------------\n'); 
for i = 1:numSeries
  fprintf(1, '%d %12.2f %12.2f\n', i, regR2s(i), residAR1s(i));
end

%====================================================
%
% Projections of economic growth on term structure
%
%====================================================

numSeries = size(growthData, 2);

% full sample 

T = length(sample_1952Q2_2010Q4);
xmat = [ones(T,1) yields(sample_1952Q2_2010Q4,:)];

regR2s = zeros(numSeries, 1); 
residAR1s = zeros(numSeries, 1);

for i = 1:numSeries
  yvec = growthData(sample_1952Q2_2010Q4,i);
  validObs = isfinite(yvec); 
  regResults = ols(yvec(validObs), xmat(validObs,:));
  regR2s(i) = regResults.rsqr;
  resids = regResults.resid;
  reg2 = ols(resids(2:end), [ones(length(resids(2:end)),1) ...
                      resids(1:(end-1))]);
  residAR1s(i) = reg2.beta(2);
end

fprintf(1, '\n');
fprintf(1, 'Projections of measures of economic growth on');
fprintf(1, ' the term structure\n\n'); 
fprintf(1, '1952Q2 through 2010Q4\n\n'); 
fprintf(1, 'Index      R2        AR(1) of resids\n'); 
fprintf(1, '------------------------------------\n'); 
for i = 1:numSeries
  fprintf(1, '%d %12.2f %12.2f\n', i, regR2s(i), residAR1s(i));
end

% shorter sample

T = length(sample_1986Q1_2007Q4);
xmat = [ones(T,1) yields(sample_1986Q1_2007Q4,:)];

regR2s = zeros(numSeries, 1); 
residAR1s = zeros(numSeries, 1);

for i = 1:numSeries
  yvec = growthData(sample_1986Q1_2007Q4,i);
  validObs = isfinite(yvec); 
  regResults = ols(yvec(validObs), xmat(validObs,:));
  regR2s(i) = regResults.rsqr;
  resids = regResults.resid;
  reg2 = ols(resids(2:end), [ones(length(resids(2:end)),1) ...
                      resids(1:(end-1))]);
  residAR1s(i) = reg2.beta(2);
end

fprintf(1, '\n');
fprintf(1, '1986Q1 through 2007Q4\n\n'); 
fprintf(1, 'Index      R2        AR(1) of resids\n'); 
fprintf(1, '------------------------------------\n'); 
for i = 1:numSeries
  fprintf(1, '%d %12.2f %12.2f\n', i, regR2s(i), residAR1s(i));
end



%====================================================
%
% Projections of inflation on term structure
%
%====================================================

numSeries = size(inflationData, 2);

% full sample 

T = length(sample_1952Q2_2010Q4);
xmat = [ones(T,1) yields(sample_1952Q2_2010Q4,:)];

regR2s = zeros(numSeries, 1); 
residAR1s = zeros(numSeries, 1);

for i = 1:numSeries
  yvec = inflationData(sample_1952Q2_2010Q4,i);
  validObs = isfinite(yvec); 
  regResults = ols(yvec(validObs), xmat(validObs,:));
  regR2s(i) = regResults.rsqr;
  resids = regResults.resid;
  reg2 = ols(resids(2:end), [ones(length(resids(2:end)),1) ...
                      resids(1:(end-1))]);
  residAR1s(i) = reg2.beta(2);
end

fprintf(1, '\n');
fprintf(1, 'Projections of inflation on');
fprintf(1, ' the term structure\n\n'); 
fprintf(1, '1952Q2 through 2010Q4\n\n'); 
fprintf(1, 'Index      R2        AR(1) of resids\n'); 
fprintf(1, '------------------------------------\n'); 
for i = 1:numSeries
  fprintf(1, '%d %12.2f %12.2f\n', i, regR2s(i), residAR1s(i));
end

% shorter sample

T = length(sample_1986Q1_2007Q4);
xmat = [ones(T,1) yields(sample_1986Q1_2007Q4,:)];

regR2s = zeros(numSeries, 1); 
residAR1s = zeros(numSeries, 1);

for i = 1:numSeries
  yvec = inflationData(sample_1986Q1_2007Q4,i);
  validObs = isfinite(yvec); 
  regResults = ols(yvec(validObs), xmat(validObs,:));
  regR2s(i) = regResults.rsqr;
  resids = regResults.resid;
  reg2 = ols(resids(2:end), [ones(length(resids(2:end)),1) ...
                      resids(1:(end-1))]);
  residAR1s(i) = reg2.beta(2);
end

fprintf(1, '\n');
fprintf(1, '1986Q1 through 2007Q4\n\n'); 
fprintf(1, 'Index      R2        AR(1) of resids\n'); 
fprintf(1, '------------------------------------\n'); 
for i = 1:numSeries
  fprintf(1, '%d %12.2f %12.2f\n', i, regR2s(i), residAR1s(i));
end


