% Greg Duffee
%
% Table 3.

%=======================================================================

clear
close all

%=======================================================================

% file names

cycleFile = char('..\excelFiles\cycleActivity_Q_Measures.xls');
quarterFile = char('..\excelFiles\quarterlydata.xls'); 
inflationFile = char('..\excelFiles\inflation_Q_measures.xls'); 

cyclePage = char('quarterlyData'); 

%=======================================================================
%
% load data
%
%=======================================================================

cycleData = xlsread(cycleFile, cyclePage, 'B32:F266');
someGrowthData = xlsread(cycleFile, cyclePage, 'G32:I266'); 

CFNAIData = xlsread(quarterFile, cyclePage, 'F30:F264');
numMissing = size(someGrowthData,1) - length(CFNAIData);
CFNAIData = [NaN*ones(numMissing, 1); CFNAIData];

LNData = xlsread(quarterFile, cyclePage, 'H30:H264');
LNData = [LNData; NaN*ones(12, 1)];
numMissing = size(someGrowthData,1) - length(LNData);
LNData = [NaN*ones(numMissing, 1); LNData];

growthData = [someGrowthData CFNAIData -LNData]; 

inflationData = xlsread(quarterFile, cyclePage, 'J30:L264');
ARMAinflationData = xlsread(inflationFile, cyclePage, ...
                            'B25:D259');
% these data are one-quarter-ahead predictions of inflation from an
% ARMA(1,1) model.  The three series are inflation from an index of
% nondurables and services, inflation from the GDP deflator, and
% inflation from the CPI respectively.  

yields = xlsread(quarterFile, cyclePage, 'M30:R264');
numYields = size(yields, 2); 

sample_1952Q2_2010Q4 = (1:235)';
sample_1986Q1_2007Q4 = (136:223)';

%====================================================
%
% Project economic growth at t on variables dated t-2 and t-3.
%
%====================================================

numSeries = size(growthData, 2);

fullSample = (4:235)';
shortSample = (139:223)'; 

firstStageR2s = zeros(numSeries, 2);
secondStageR2s = zeros(numSeries, 2);
residAR1s = zeros(numSeries, 2);

for i = 1:numSeries
  contempGrowth = growthData(:,i);
  lag2Growth = [NaN; NaN; growthData(1:(end-2),i)];
  lag3Growth = [NaN; NaN; NaN; growthData(1:(end-3),i)];
  lag2CPI = [NaN; NaN; inflationData(1:(end-2),2)]; 
  lag3CPI = [NaN; NaN; NaN; inflationData(1:(end-3),2)]; 
  lag2yields = [repmat(NaN, 2, numYields); yields(1:(end-2),:)]; 
  bigMatrix = [lag2Growth lag3Growth lag2CPI lag3CPI ...
               lag2yields]; 

  for j = 1:2
    if (j == 1) thisSample = fullSample; end
    if (j == 2) thisSample = shortSample; end

    yvec = contempGrowth(thisSample); 
    xmat = [ones(length(thisSample), 1) bigMatrix(thisSample,:)];
    
    validObs = isfinite(yvec) & ...
        isfinite(xmat(:,2)) & isfinite(xmat(:,3));
    
    okYvec = yvec(validObs);
    okXmat = xmat(validObs,:); 
    
    reg1 = ols(okYvec, okXmat); 
    firstStageR2s(i,j) = reg1.rsqr; 
    
    projectedValues = okYvec - reg1.resid; 
    
    reg2 = ols(projectedValues, okXmat(:,[1 6:end]));
    secondStageR2s(i,j) = reg2.rsqr;
    resids2 = reg2.resid;
    reg3 = ols(resids2(2:end), [ones(length(resids2)-1, 1) ...
                        resids2(1:(end-1))]);
    residAR1s(i,j) = reg3.beta(2);
    
  end
  
end


fprintf(1, '\n');
fprintf(1, 'Two-stage projections of economic growth measures\n');
fprintf(1, 'First projection on twice-lagged');
fprintf(1, ' yield curve and other variables\n');
fprintf(1, 'Then project first projection on');
fprintf(1, 'twice-lagged yielc curve\n'); 
fprintf(1, '1952Q2 through 2010Q4\n\n'); 
fprintf(1, 'Index    1st R2    2nd R2 AR1, 2nd residuals\n');
fprintf(1, '--------------------------------------------\n'); 
for i = 1:numSeries
  fprintf(1, '%d %12.2f %12.2f %12.2f\n', i, firstStageR2s(i,1), ...
          secondStageR2s(i,1), residAR1s(i,1));
end
fprintf(1, '\n'); 
fprintf(1, '1986Q1 through 2007Q4\n\n'); 
fprintf(1, 'Index    1st R2    2nd R2 AR1, 2nd residuals\n');
fprintf(1, '--------------------------------------------\n'); 
for i = 1:numSeries
  fprintf(1, '%d %12.2f %12.2f %12.2f\n', i, firstStageR2s(i,2), ...
          secondStageR2s(i,2), residAR1s(i,2));
end

