% Greg Duffee
%
% Table 4.

%=======================================================================

clear
close all

%=======================================================================

% file names

cycleFile = char('..\excelFiles\cycleActivity_Q_Measures.xls');
quarterFile = char('..\excelFiles\quarterlydata.xls'); 
inflationFile = char('..\excelFiles\inflation_Q_measures.xls'); 

cyclePage = char('quarterlyData'); 

%=======================================================================
%
% load data
%
%=======================================================================

cycleData = xlsread(cycleFile, cyclePage, 'B32:F266');
someGrowthData = xlsread(cycleFile, cyclePage, 'G32:I266'); 

CFNAIData = xlsread(quarterFile, cyclePage, 'F30:F264');
numMissing = size(someGrowthData,1) - length(CFNAIData);
CFNAIData = [NaN*ones(numMissing, 1); CFNAIData];

LNData = xlsread(quarterFile, cyclePage, 'H30:H264');
LNData = [LNData; NaN*ones(12, 1)];
numMissing = size(someGrowthData,1) - length(LNData);
LNData = [NaN*ones(numMissing, 1); LNData];

growthData = [someGrowthData CFNAIData -LNData]; 

inflationData = xlsread(quarterFile, cyclePage, 'J30:L264');
ARMAinflationData = xlsread(inflationFile, cyclePage, ...
                            'B25:D259');
% these data are one-quarter-ahead predictions of inflation from an
% ARMA(1,1) model.  The three series are inflation from an index of
% nondurables and services, inflation from the GDP deflator, and
% inflation from the CPI respectively.  

yields = xlsread(quarterFile, cyclePage, 'M30:R264');
numYields = size(yields, 2); 

%====================================================
%
% Project bond yields on contemporaneous and lagged variables 
% that should pick up variation in yields.
%
%====================================================


% construct the matrix of explanatory variables.  Contemporaneous
% HP-filtered log real GDP, HP-filtered log IP, log-differenced IP
% for quarters t, t-1, t-2, and ARMA-smoothed CPI inflation for
% quarters t, t-1, t-2.  
%

explanatoryVars = [cycleData(:,3:4) ...
                   growthData(:,3) ...
                   [NaN; growthData(1:(end-1),3)] ...
                   [NaN; NaN; growthData(1:(end-2),3)] ...
                   ARMAinflationData(:,3) ...
                   [NaN; ARMAinflationData(1:(end-1),3)] ...
                   [NaN; NaN; ARMAinflationData(1:(end-2),3)]];

yieldR2s = zeros(numYields, 2);
yieldResidSDs = zeros(numYields, 2);
residAR1s = zeros(numYields, 2);

fullSample = (3:235)';
shortSample = (138:223)'; 

for i = 1:numYields
  for j = 1:2
    if (j == 1) thisSample = fullSample; end
    if (j == 2) thisSample = shortSample; end
    T = length(thisSample); 
    
    yvec = yields(thisSample,i); 
    xmat = [ones(T,1) explanatoryVars(thisSample,:)];
    
    validObs = isfinite(yvec) & (sum(isfinite(xmat),2)==size(xmat, ...
                                                      2));
    regResults = ols(yvec(validObs), xmat(validObs,:)); 
    yieldR2s(i,j) = regResults.rsqr; 
    resids = regResults.resid; 
    yieldResidSDs(i,j) = sqrt( ...
        var(resids,1)* ...
        (regResults.nobs/(regResults.nobs - regResults.nvar)));
    reg2 = ols(resids(2:end), [ones(length(resids)-1,1) resids(1: ...
                                                      (end-1))]);
    residAR1s(i,j) = reg2.beta(2);
  end
end

fprintf(1, '\n');
fprintf(1, 'Project yields on contemporaneous and lagged');
fprintf(1, ' measures of economic activity and inflation\n');
fprintf(1, '1952Q2 through 2010Q4\n\n'); 
fprintf(1, 'Index      R2        resid SD      resid AR1\n');
fprintf(1, '--------------------------------------------\n'); 
for i = 1:numYields
  fprintf(1, '%d %12.2f %12.2f %12.2f\n', i, yieldR2s(i,1), ...
          yieldResidSDs(i,1)*100, residAR1s(i,1));
end
fprintf(1, '\n'); 
fprintf(1, '1986Q1 through 2007Q4\n\n'); 
fprintf(1, 'Index      R2        resid SD      resid AR1\n');
fprintf(1, '--------------------------------------------\n'); 
for i = 1:numYields
  fprintf(1, '%d %12.2f %12.2f %12.2f\n', i, yieldR2s(i,2), ...
          yieldResidSDs(i,2)*100, residAR1s(i,2));
end


