function output = covarFunction2factor(freeparams, allparams, free_index, ...
                                       numFactors, numObserved, horizon, ...
                                       beta, gamma)
%
%
% Greg Duffee
% June 2011
%

paramvec = poke_freeparams(freeparams, allparams, free_index); 

[mu K_vec Sigma_vec A B_vec errorSDs] = ...
    unpack_allparams(numFactors, numObserved, paramvec);
[Kmat Sigma B H] = ...
    build_matrices(K_vec, Sigma_vec, B_vec, errorSDs); 


tmpM = inv(eye(numFactors) - beta*Kmat);
tmp2 = B(end,:)*tmpM;
tmp3 = B(end,:)*(eye(numFactors) - (1 - gamma)*tmpM);

macro_real_delta0 = - log(beta) + A(end) + ...
    B(end,:) * mu  ...
    + 0.5*(1 - gamma)^2 * ( tmp2*Sigma*Sigma'*tmp2' + H(end,end) ) ...
    - 0.5*( (gamma^2)*H(end,end) + tmp3*Sigma*Sigma'*tmp3' );
macro_real_delta1 = (B(end,:)*Kmat)';
macro_nom_delta0 = macro_real_delta0 + A(end-1) + B(end-1,:)*mu - ...
    0.5*B(end-1,:)*Sigma*Sigma'*B(end-1,:)' - 0.5*H(end-1,end-1) - ...
    tmp3*Sigma*Sigma'*B(end-1,:)';
macro_nom_delta1 = (macro_real_delta1' + B(end-1,:)*Kmat)';

macro_real_lambda0 = Sigma*Sigma'*tmp3';
macro_nom_lambda0 = macro_real_lambda0 + Sigma*Sigma'*B(end-1,:)';

lambda1 = 0;

real_bigBmat = createBigB(macro_real_delta1, Kmat, horizon);
real_bigAvec = createBigA(macro_real_delta0, (mu - macro_real_lambda0), ...
                          Sigma, real_bigBmat);

nom_bigBmat = createBigB(macro_nom_delta1, Kmat, horizon);
nom_bigAvec = createBigA(macro_nom_delta0, (mu - macro_nom_lambda0), ...
                          Sigma, nom_bigBmat);

stateMean = inv(eye(numFactors) - Kmat)*mu; 

macro_meanRealTS = real_bigAvec + real_bigBmat*stateMean;
macro_meanNomTS = nom_bigAvec + nom_bigBmat*stateMean;

riskPremiumHorizonPlus1 = ...
    (-horizon*nom_bigBmat(horizon,:))*macro_nom_lambda0;

%
% covariance of two pieces of nominal short rate with three pieces
% of the nominal stochastic discount factor.  The two short rate
% pieces are the innovations in the real rate and one-step-ahead
% inflation.  The three
% pieces of the nominal SDF are the innovation in immediate
% consumption growth, the innovation in the PV of expected future
% consumption growth, and the innovation in inflation.
%

sumVector = B(end,:)*inv(eye(numFactors) - beta*Kmat)*Sigma; 

avgState = 0; 
for j = 1:horizon
  avgState = avgState + Kmat^j;
end
avgState = avgState/horizon;
  
meanPiResponse = B(end-1,:)*avgState;
meanCResponse = B(end,:)*avgState; 
  
output.covCmeanPi = B(end,:)*Sigma*Sigma'*meanPiResponse';
output.covCmeanC = B(end,:)*Sigma*Sigma'*meanCResponse';
output.covSUMmeanPi = sumVector*Sigma'*meanPiResponse';
output.covSUMmeanC = sumVector*Sigma'*meanCResponse';
output.covPimeanPi = B(end-1,:)*Sigma*Sigma'*meanPiResponse';
output.covPimeanC = B(end-1,:)*Sigma*Sigma'*meanCResponse';
output.covCNomBondYield = B(end,:)*Sigma*Sigma'*nom_bigBmat(horizon,:)';
output.covCRealBondYield = B(end,:)*Sigma*Sigma'*real_bigBmat(horizon,:)';
output.covSUMNomBondYield = sumVector*Sigma'*nom_bigBmat(horizon,:)';
output.covSUMRealBondYield = sumVector*Sigma'*real_bigBmat(horizon,:)';
output.covPiNomBondYield = B(end-1,:)*Sigma*Sigma'*nom_bigBmat(horizon,:)';
output.covPiRealBondYield = B(end-1,:)*Sigma*Sigma'*real_bigBmat(horizon,:)';
output.riskPremiumHorizonPlus1 = riskPremiumHorizonPlus1;

output.macro_meanRealTS = macro_meanRealTS;
output.macro_meanNomTS = macro_meanNomTS;

return
