% Greg Duffee
% June 2011
%
% Study estimated 4-factor model, and get standard errors of values
% that are relevant.  Recursive utility setting. 

clear
close all

origPath = path;
path('..\codeToEstimateGeneralModel', origPath);


% pick your desired model by commenting out the others.  

twoBond_4factor = char('..\genModFlatFiles\q_4factor3m5yra.out');
%twoBond_4factor = char('..\genModFlatFiles\q_4factor3m5yrEarlya.out');
%twoBond_4factor = char('..\genModFlatFiles\q_4factor3m5yrEarly2a.out');
%twoBond_4factor = char('..\genModFlatFiles\q_4factor3m5yrEarly3a.out');
%twoBond_4factor = char('..\genModFlatFiles\q_4factor3m5yrShorta.out');

tolerance = 1e-12;

[yielddatafile numbonds numUsedBonds usedBondIdx ...
 macrodatafile numMacro numUsedMacro usedMacroIdx ...
 numfactors firstobs lastobs ...
 use_ols_start olsStartIdx ...
 numloops numSmallLoops numsimplex numderiv ...
 mu_names mu_flag mu mu_rand_stddev ...
 k_names k_flag k_vec k_rand_stddev ...
 S_names S_flag Sigma_vec S_rand_stddev ...
 A_name A_flag A A_rand_stddev ...
 B_names B_flag B_vec B_rand_stddev ...
 errorSD_name errorSD_flag errorSDs errorSD_rand_stddev ...
] = read_input(twoBond_4factor);

use_ols_start = 0;

observedData = ...
    read_data(yielddatafile, macrodatafile, numbonds, usedBondIdx, ...
              numMacro, usedMacroIdx, firstobs, lastobs);

[T numObserved] = size(observedData);

[freeparams bigparamvec free_index] = setup_initparams( ...
    mu, k_vec, Sigma_vec, A, B_vec, ...
    errorSDs, mu_flag, k_flag, S_flag, ...
    A_flag, B_flag, errorSD_flag, use_ols_start, olsStartIdx, ...
    observedData);

numfreeparams = length(freeparams); 

[Kmat Sigma B H] = ...
    build_matrices(k_vec, Sigma_vec, B_vec, errorSDs); 

stateMean = inv(eye(numfactors) - Kmat)*mu; 

[stateCovMat valid] = uncon_vcv_general(Kmat, Sigma); 

factor_SDs = sqrt(diag(stateCovMat)); 

[funval loglikes filtered_values errors] = ...
    kalman_only_loglike(freeparams, bigparamvec, free_index, ...
                        observedData, numfactors, tolerance); 

delta = 1e-7; 
[score outerProd scoreMat] = ...
    kalman_derivs(freeparams, bigparamvec, free_index, ...
                  observedData, numfactors, tolerance, ...
                  delta);

[OPinv ok] = invpd(outerProd);

if (ok == 1)
  paramSDs = sqrt(diag(OPinv));
end

%==============================================================
%
% Results.  remember order of observables is 3m, 5yr, pi, delta c
%
%==============================================================


beta = 0.999; 
gamma = 10; 

horizon = 1; 
output1 = covarFunction(freeparams, bigparamvec, free_index, ...
                       numfactors, numObserved, horizon, ...
                       beta, gamma); 
horizon = 19; 
output19 = covarFunction(freeparams, bigparamvec, free_index, ...
                       numfactors, numObserved, horizon, ...
                       beta, gamma); 

horizon = 20; 
output20 = covarFunction(freeparams, bigparamvec, free_index, ...
                       numfactors, numObserved, horizon, ...
                       beta, gamma); 

%============================================================
%
% Population regressions of nominal bond yields.
%
%=========================================================== 


regOutput = regressFun(freeparams, bigparamvec, free_index, ...
                                numfactors, numObserved); 

%==============================================================
%
% Standard errors for one-quarter horizon.
%
%==============================================================

horizon = 1; 

dCmeanC = zeros(numfreeparams, 1);
dCmeanPi = zeros(numfreeparams, 1); 
dPimeanC = zeros(numfreeparams, 1);
dPimeanPi = zeros(numfreeparams, 1);
dSUMmeanC = zeros(numfreeparams, 1);
dSUMmeanPi = zeros(numfreeparams, 1);
dCRealBondYield = zeros(numfreeparams, 1);
dCNomBondYield = zeros(numfreeparams, 1);
dPiRealBondYield = zeros(numfreeparams, 1);
dPiNomBondYield = zeros(numfreeparams, 1);
dSUMRealBondYield = zeros(numfreeparams, 1);
dSUMNomBondYield = zeros(numfreeparams, 1);
dRP = zeros(numfreeparams, 1); 

dCActNom3mYield = zeros(numfreeparams,1);
dPiActNom3mYield = zeros(numfreeparams,1);
dSUMActNom3mYield = zeros(numfreeparams,1);

for thisParam = 1:numfreeparams
  freeCopy = freeparams;

  freeCopy(thisParam) = freeparams(thisParam) + delta;
  outputUp = covarFunction(freeCopy, bigparamvec, free_index, ...
                          numfactors, numObserved, horizon, ...
                          beta, gamma); 

  freeCopy(thisParam) = freeparams(thisParam) - delta;
  outputDown = covarFunction(freeCopy, bigparamvec, free_index, ...
                          numfactors, numObserved, horizon, ...
                          beta, gamma); 

  dCmeanPi(thisParam) = (outputUp.covCmeanPi - ...
                         outputDown.covCmeanPi)/(2*delta); 
  dCmeanC(thisParam) = (outputUp.covCmeanC - ...
                         outputDown.covCmeanC)/(2*delta); 
  dSUMmeanPi(thisParam) = (outputUp.covSUMmeanPi - ...
                         outputDown.covSUMmeanPi)/(2*delta); 
  dSUMmeanC(thisParam) = (outputUp.covSUMmeanC - ...
                         outputDown.covSUMmeanC)/(2*delta); 
  dPimeanPi(thisParam) = (outputUp.covPimeanPi - ...
                         outputDown.covPimeanPi)/(2*delta); 
  dPimeanC(thisParam) = (outputUp.covPimeanC - ...
                         outputDown.covPimeanC)/(2*delta); 
  dCNomBondYield(thisParam) = (outputUp.covCNomBondYield - ...
                         outputDown.covCNomBondYield)/(2*delta); 
  dCRealBondYield(thisParam) = (outputUp.covCRealBondYield - ...
                         outputDown.covCRealBondYield)/(2*delta); 
  dSUMNomBondYield(thisParam) = (outputUp.covSUMNomBondYield - ...
                         outputDown.covSUMNomBondYield)/(2*delta); 
  dSUMRealBondYield(thisParam) = (outputUp.covSUMRealBondYield - ...
                         outputDown.covSUMRealBondYield)/(2*delta); 
  dPiNomBondYield(thisParam) = (outputUp.covPiNomBondYield - ...
                         outputDown.covPiNomBondYield)/(2*delta); 
  dPiRealBondYield(thisParam) = (outputUp.covPiRealBondYield - ...
                         outputDown.covPiRealBondYield)/(2*delta); 
  dCActNom3mYield(thisParam) = (outputUp.covCActNom3mYield - ...
                         outputDown.covCActNom3mYield)/(2*delta); 
  dSUMActNom3mYield(thisParam) = (outputUp.covSUMActNom3mYield - ...
                         outputDown.covSUMActNom3mYield)/(2*delta); 
  dPiActNom3mYield(thisParam) = (outputUp.covPiActNom3mYield - ...
                         outputDown.covPiActNom3mYield)/(2*delta);
  dRP(thisParam) = (outputUp.riskPremiumHorizonPlus1 - ...
         outputDown.riskPremiumHorizonPlus1)/(2*delta);
  
end


stdErr1 = sqrt(dCmeanC'*OPinv*dCmeanC);
stdErr2 = sqrt(dCmeanPi'*OPinv*dCmeanPi);
stdErr3 = sqrt(dPimeanC'*OPinv*dPimeanC);
stdErr4 = sqrt(dPimeanPi'*OPinv*dPimeanPi);
stdErr5 = sqrt(dSUMmeanC'*OPinv*dSUMmeanC);
stdErr6 = sqrt(dSUMmeanPi'*OPinv*dSUMmeanPi);
stdErr7 = sqrt(dCRealBondYield'*OPinv*dCRealBondYield);
stdErr8 = sqrt(dCNomBondYield'*OPinv*dCNomBondYield);
stdErr9 = sqrt(dPiRealBondYield'*OPinv*dPiRealBondYield);
stdErr10 = sqrt(dPiNomBondYield'*OPinv*dPiNomBondYield);
stdErr11 = sqrt(dSUMRealBondYield'*OPinv*dSUMRealBondYield);
stdErr12 = sqrt(dSUMNomBondYield'*OPinv*dSUMNomBondYield);
stdErr13 = sqrt(dCActNom3mYield'*OPinv*dCActNom3mYield);
stdErr14 = sqrt(dPiActNom3mYield'*OPinv*dPiActNom3mYield);
stdErr15 = sqrt(dSUMActNom3mYield'*OPinv*dSUMActNom3mYield);
stdErr16 = sqrt(dRP'*OPinv*dRP);

%
% below are standard errors for the covariance of the log SDF with
% the log bond yields.  Nominal, real, actual 3-month.  

tmp = dCNomBondYield + (gamma-1)*dSUMNomBondYield + dPiNomBondYield;
stdErrSum1 = sqrt(tmp'*OPinv*tmp);
tmp = dCRealBondYield + (gamma-1)*dSUMRealBondYield + dPiRealBondYield;
stdErrSum2 = sqrt(tmp'*OPinv*tmp);
tmp = dCActNom3mYield + (gamma-1)*dSUMActNom3mYield + dPiActNom3mYield;
stdErrSum3 = sqrt(tmp'*OPinv*tmp);


stdErrs_1 = [stdErr1; stdErr2; stdErr3; stdErr4; stdErr5; stdErr6; ...
             stdErr7; stdErr8; stdErr9; stdErr10; stdErr11; stdErr12; ...
             stdErr13; stdErr14; stdErr15; stdErr16];

stdErrsSums_1 = [stdErrSum1; stdErrSum2; stdErrSum3];

sum1 = (output1.covCNomBondYield + ...
              output20.covPiNomBondYield + ...
              (gamma - 1)*output20.covSUMNomBondYield);
sum2 = (output1.covCRealBondYield + ...
              output20.covPiRealBondYield + ...
              (gamma - 1)*output20.covSUMRealBondYield);
sum3 = (output1.covCActNom3mYield + ...
              output20.covPiActNom3mYield + ...
              (gamma - 1)*output20.covSUMActNom3mYield);
  
sums_1 = [sum1; sum2; sum3];

sumTstats_1 = sums_1 ./ stdErrsSums_1;


%==============================================================
%
% Standard errors for the five-year horizon.
%
%==============================================================

horizon = 20; 

dCmeanC = zeros(numfreeparams, 1);
dCmeanPi = zeros(numfreeparams, 1); 
dPimeanC = zeros(numfreeparams, 1);
dPimeanPi = zeros(numfreeparams, 1);
dSUMmeanC = zeros(numfreeparams, 1);
dSUMmeanPi = zeros(numfreeparams, 1);
dCRealBondYield = zeros(numfreeparams, 1);
dCNomBondYield = zeros(numfreeparams, 1);
dPiRealBondYield = zeros(numfreeparams, 1);
dPiNomBondYield = zeros(numfreeparams, 1);
dSUMRealBondYield = zeros(numfreeparams, 1);
dSUMNomBondYield = zeros(numfreeparams, 1);
dRP = zeros(numfreeparams, 1); 

dCActNom5yrYield = zeros(numfreeparams,1);
dPiActNom5yrYield = zeros(numfreeparams,1);
dSUMActNom5yrYield = zeros(numfreeparams,1);

for thisParam = 1:numfreeparams
  freeCopy = freeparams;

  freeCopy(thisParam) = freeparams(thisParam) + delta;
  horizon = 20; 
  outputUp = covarFunction(freeCopy, bigparamvec, free_index, ...
                          numfactors, numObserved, horizon, ...
                          beta, gamma); 
  horizon = 19; 
  altOutputUp = covarFunction(freeCopy, bigparamvec, free_index, ...
                          numfactors, numObserved, horizon, ...
                          beta, gamma); 

  freeCopy(thisParam) = freeparams(thisParam) - delta;
  horizon = 20; 
  outputDown = covarFunction(freeCopy, bigparamvec, free_index, ...
                          numfactors, numObserved, horizon, ...
                          beta, gamma); 
  horizon = 19; 
  altOutputDown = covarFunction(freeCopy, bigparamvec, free_index, ...
                          numfactors, numObserved, horizon, ...
                          beta, gamma); 

  dCmeanPi(thisParam) = (outputUp.covCmeanPi - ...
                         outputDown.covCmeanPi)/(2*delta); 
  dCmeanC(thisParam) = (outputUp.covCmeanC - ...
                         outputDown.covCmeanC)/(2*delta); 
  dSUMmeanPi(thisParam) = (outputUp.covSUMmeanPi - ...
                         outputDown.covSUMmeanPi)/(2*delta); 
  dSUMmeanC(thisParam) = (outputUp.covSUMmeanC - ...
                         outputDown.covSUMmeanC)/(2*delta); 
  dPimeanPi(thisParam) = (outputUp.covPimeanPi - ...
                         outputDown.covPimeanPi)/(2*delta); 
  dPimeanC(thisParam) = (outputUp.covPimeanC - ...
                         outputDown.covPimeanC)/(2*delta); 
  dCNomBondYield(thisParam) = (outputUp.covCNomBondYield - ...
                         outputDown.covCNomBondYield)/(2*delta); 
  dCRealBondYield(thisParam) = (outputUp.covCRealBondYield - ...
                         outputDown.covCRealBondYield)/(2*delta); 
  dSUMNomBondYield(thisParam) = (outputUp.covSUMNomBondYield - ...
                         outputDown.covSUMNomBondYield)/(2*delta); 
  dSUMRealBondYield(thisParam) = (outputUp.covSUMRealBondYield - ...
                         outputDown.covSUMRealBondYield)/(2*delta); 
  dPiNomBondYield(thisParam) = (outputUp.covPiNomBondYield - ...
                         outputDown.covPiNomBondYield)/(2*delta); 
  dPiRealBondYield(thisParam) = (outputUp.covPiRealBondYield - ...
                         outputDown.covPiRealBondYield)/(2*delta); 
  dCActNom5yrYield(thisParam) = (outputUp.covCActNom5yrYield - ...
                         outputDown.covCActNom5yrYield)/(2*delta); 
  dSUMActNom5yrYield(thisParam) = (outputUp.covSUMActNom5yrYield - ...
                         outputDown.covSUMActNom5yrYield)/(2*delta); 
  dPiActNom5yrYield(thisParam) = (outputUp.covPiActNom5yrYield - ...
                         outputDown.covPiActNom5yrYield)/(2*delta); 
  dRP(thisParam) = (altOutputUp.riskPremiumHorizonPlus1 - ...
         altOutputDown.riskPremiumHorizonPlus1)/(2*delta);
  
  
end


stdErr1 = sqrt(dCmeanC'*OPinv*dCmeanC);
stdErr2 = sqrt(dCmeanPi'*OPinv*dCmeanPi);
stdErr3 = sqrt(dPimeanC'*OPinv*dPimeanC);
stdErr4 = sqrt(dPimeanPi'*OPinv*dPimeanPi);
stdErr5 = sqrt(dSUMmeanC'*OPinv*dSUMmeanC);
stdErr6 = sqrt(dSUMmeanPi'*OPinv*dSUMmeanPi);
stdErr7 = sqrt(dCRealBondYield'*OPinv*dCRealBondYield);
stdErr8 = sqrt(dCNomBondYield'*OPinv*dCNomBondYield);
stdErr9 = sqrt(dPiRealBondYield'*OPinv*dPiRealBondYield);
stdErr10 = sqrt(dPiNomBondYield'*OPinv*dPiNomBondYield);
stdErr11 = sqrt(dSUMRealBondYield'*OPinv*dSUMRealBondYield);
stdErr12 = sqrt(dSUMNomBondYield'*OPinv*dSUMNomBondYield);
stdErr13 = sqrt(dCActNom5yrYield'*OPinv*dCActNom5yrYield);
stdErr14 = sqrt(dPiActNom5yrYield'*OPinv*dPiActNom5yrYield);
stdErr15 = sqrt(dSUMActNom5yrYield'*OPinv*dSUMActNom5yrYield);
stdErr16 = sqrt(dRP'*OPinv*dRP);

tmp = dCNomBondYield + (gamma-1)*dSUMNomBondYield + dPiNomBondYield;
stdErrSum1 = sqrt(tmp'*OPinv*tmp);
tmp = dCRealBondYield + (gamma-1)*dSUMRealBondYield + dPiRealBondYield;
stdErrSum2 = sqrt(tmp'*OPinv*tmp);
tmp = dCActNom5yrYield + (gamma-1)*dSUMActNom5yrYield + dPiActNom5yrYield;
stdErrSum3 = sqrt(tmp'*OPinv*tmp);


stdErrs_20 = [stdErr1; stdErr2; stdErr3; stdErr4; stdErr5; stdErr6; ...
             stdErr7; stdErr8; stdErr9; stdErr10; stdErr11; stdErr12; ...
             stdErr13; stdErr14; stdErr15; stdErr16];

stdErrsSums_20 = [stdErrSum1; stdErrSum2; stdErrSum3];

sum1 = (output20.covCNomBondYield + ...
              output20.covPiNomBondYield + ...
              (gamma - 1)*output20.covSUMNomBondYield);
sum2 = (output20.covCRealBondYield + ...
              output20.covPiRealBondYield + ...
              (gamma - 1)*output20.covSUMRealBondYield);
sum3 = (output20.covCActNom5yrYield + ...
              output20.covPiActNom5yrYield + ...
        (gamma - 1)*output20.covSUMActNom5yrYield);
  
sums_20 = [sum1; sum2; sum3];

sumTstats_20 = sums_20 ./ stdErrsSums_20;

%==============================================================
%
% Standard errors for population regressions
%
%==============================================================

dCoef3m_expC = zeros(numfreeparams, 1);
dCoef3m_expPi = zeros(numfreeparams, 1);
dCoef5yr_expC = zeros(numfreeparams, 1);
dCoef5yr_expPi = zeros(numfreeparams, 1);
dCoef5yr_3m = zeros(numfreeparams,1);

for thisParam = 1:numfreeparams
  freeCopy = freeparams;

  freeCopy(thisParam) = freeparams(thisParam) + delta;
  outputUp = regressFun(freeCopy, bigparamvec, free_index, ...
                          numfactors, numObserved); 

  freeCopy(thisParam) = freeparams(thisParam) - delta;
  outputDown = regressFun(freeCopy, bigparamvec, free_index, ...
                          numfactors, numObserved); 

  dCoef3m_expC(thisParam) = (outputUp.threeMcoef(1) - ...
                   outputDown.threeMcoef(1))/(2*delta);
  dCoef3m_expPi(thisParam) = (outputUp.threeMcoef(2) - ...
                   outputDown.threeMcoef(2))/(2*delta);
  dCoef5yr_expC(thisParam) = (outputUp.fiveYRcoef1(1) - ...
                   outputDown.fiveYRcoef1(1))/(2*delta);
  dCoef5yr_expPi(thisParam) = (outputUp.fiveYRcoef1(2) - ...
                   outputDown.fiveYRcoef1(2))/(2*delta);
  dCoef5yr_3m(thisParam) = (outputUp.fiveYRcoef2(1) - ...
                   outputDown.fiveYRcoef2(1))/(2*delta);
end

stdErr1 = sqrt(dCoef3m_expC'*OPinv*dCoef3m_expC);
stdErr2 = sqrt(dCoef3m_expPi'*OPinv*dCoef3m_expPi);
stdErr3 = sqrt(dCoef5yr_expC'*OPinv*dCoef5yr_expC);
stdErr4 = sqrt(dCoef5yr_expPi'*OPinv*dCoef5yr_expPi);
stdErr5 = sqrt(dCoef5yr_3m'*OPinv*dCoef5yr_3m);

regCoefSEs = [stdErr1; stdErr2; stdErr3; stdErr4; stdErr5];

%=========================================================
%
% Print results
%
%=========================================================


fprintf(1, '\nConditional Covariances\n'); 
fprintf(1,   '-----------------------\n'); 
fprintf(1, '\n'); 
fprintf(1, '         delta c      pi          infinite sum\n'); 
fprintf(1, '      --------------------------------------\n\n'); 
fprintf(1, 'next d-c %8.3f %8.3f %8.3f \n', ...
        1e4*output1.covCmeanC, 1e4*output1.covPimeanC, ...
        1e4*output1.covSUMmeanC);
fprintf(1, 'next  pi %8.3f %8.3f %8.3f \n', ...
        1e4*output1.covCmeanPi, 1e4*output1.covPimeanPi, ...
        1e4*output1.covSUMmeanPi);
fprintf(1, 'avg d-c  %8.3f %8.3f %8.3f \n', ...
        1e4*output20.covCmeanC, 1e4*output20.covPimeanC, ...
        1e4*output20.covSUMmeanC);
fprintf(1, 'avg pi   %8.3f %8.3f %8.3f \n', ...
        1e4*output20.covCmeanPi, 1e4*output20.covPimeanPi, ...
        1e4*output20.covSUMmeanPi);
fprintf(1, '\n\n'); 
fprintf(1, 'Model-implied covars with 3-month yields\n'); 
fprintf(1, 'real     %8.3f %8.3f %8.3f \n', ...
        1e4*output1.covCRealBondYield, 1e4*output1.covPiRealBondYield, ...
        1e4*output1.covSUMRealBondYield);
fprintf(1, 'nominal  %8.3f %8.3f %8.3f \n', ...
        1e4*output1.covCNomBondYield, 1e4*output1.covPiNomBondYield, ...
        1e4*output1.covSUMNomBondYield);
fprintf(1, '\n\n'); 
fprintf(1, 'Model-implied covars with 20-quarter yields\n'); 
fprintf(1, 'real     %8.3f %8.3f %8.3f \n', ...
        1e4*output20.covCRealBondYield, 1e4*output20.covPiRealBondYield, ...
        1e4*output20.covSUMRealBondYield);
fprintf(1, 'nominal  %8.3f %8.3f %8.3f \n', ...
        1e4*output20.covCNomBondYield, 1e4*output20.covPiNomBondYield, ...
        1e4*output20.covSUMNomBondYield);
fprintf(1, '\n\n'); 
fprintf(1, 'Actual covars with yields\n'); 
fprintf(1, '3 q     %8.3f %8.3f %8.3f \n', ...
        1e4*output1.covCActNom3mYield, 1e4*output1.covPiActNom3mYield, ...
        1e4*output1.covSUMActNom3mYield);
fprintf(1, '20 q    %8.3f %8.3f %8.3f \n', ...
        1e4*output20.covCActNom5yrYield, 1e4*output20.covPiActNom5yrYield, ...
        1e4*output20.covSUMActNom5yrYield);
fprintf(1, '\n\n'); 


fprintf(1, '\n    Standard errors\n'); 
fprintf(1,   '-----------------------\n'); 
fprintf(1, '\n'); 
fprintf(1, '         delta c      pi          infinite sum\n'); 
fprintf(1, '      --------------------------------------\n\n'); 
fprintf(1, 'next d-c %8.3f %8.3f %8.3f \n', ...
        1e4*stdErrs_1(1), 1e4*stdErrs_1(3), ...
        1e4*stdErrs_1(5));
fprintf(1, 'next  pi %8.3f %8.3f %8.3f \n', ...
        1e4*stdErrs_1(2), 1e4*stdErrs_1(4), ...
        1e4*stdErrs_1(6));
fprintf(1, 'avg d-c  %8.3f %8.3f %8.3f \n', ...
        1e4*stdErrs_20(1), 1e4*stdErrs_20(3), ...
        1e4*stdErrs_20(5));
fprintf(1, 'avg pi   %8.3f %8.3f %8.3f \n', ...
        1e4*stdErrs_20(2), 1e4*stdErrs_20(4), ...
        1e4*stdErrs_20(6));
fprintf(1, '\n\n'); 
fprintf(1, 'Model-implied covars with 3-month yields\n'); 
fprintf(1, 'real     %8.3f %8.3f %8.3f \n', ...
        1e4*stdErrs_1(7), 1e4*stdErrs_1(9), ...
        1e4*stdErrs_1(11));
fprintf(1, 'nominal  %8.3f %8.3f %8.3f \n', ...
        1e4*stdErrs_1(8), 1e4*stdErrs_1(10), ...
        1e4*stdErrs_1(12));
fprintf(1, '\n\n'); 
fprintf(1, 'Model-implied covars with 20-quarter yields\n'); 
fprintf(1, 'real     %8.3f %8.3f %8.3f \n', ...
        1e4*stdErrs_20(7), 1e4*stdErrs_20(9), ...
        1e4*stdErrs_20(11));
fprintf(1, 'nominal  %8.3f %8.3f %8.3f \n', ...
        1e4*stdErrs_20(8), 1e4*stdErrs_20(10), ...
        1e4*stdErrs_20(12));
fprintf(1, '\n\n'); 
fprintf(1, 'Actual covars with yields\n'); 
fprintf(1, '3 q     %8.3f %8.3f %8.3f \n', ...
        1e4*stdErrs_1(13), 1e4*stdErrs_1(14), ...
        1e4*stdErrs_1(15));
fprintf(1, '20 q    %8.3f %8.3f %8.3f \n', ...
        1e4*stdErrs_20(13), 1e4*stdErrs_20(14), ...
        1e4*stdErrs_20(15));
fprintf(1, '\n'); 


fprintf(1, ...
        'Risk premium on a 20-quarter bond, in percent/quarter\n');

fprintf(1, 'Value        Std Err\n');
fprintf(1, '%8.3f    %8.3f\n', output19.riskPremiumHorizonPlus1*100, ...
        100*stdErrs_20(16));

fprintf(1, '\n\n'); 


%
%
%


fprintf(1, 'Population regression of nominal short rate on\n');
fprintf(1, 'expected consumption growth and expected ');
fprintf(1, 'inflation\n');
fprintf(1, '\n'); 
fprintf(1, ...
        'Coef, SE on expected consumption growth are %7.2f, %7.2f\n', ...
        regOutput.threeMcoef(1), regCoefSEs(1));
fprintf(1, 'Coef, SE on expected inflation are %7.2f, %7.2f\n', ...
        regOutput.threeMcoef(2), regCoefSEs(2));
fprintf(1, 'Rsquare is %7.2f\n', regOutput.R2_3m); 
fprintf(1, '\n\n'); 

fprintf(1, 'Population regression of nominal 5yr on\n');
fprintf(1, 'expected 3month yield over 5 years\n');
fprintf(1, '\n'); 
fprintf(1, 'Coef, SE are %7.2f, %7.2f\n', regOutput.fiveYRcoef2, ...
        regCoefSEs(5));
fprintf(1, 'Rsquare is %7.2f\n', regOutput.R2_5YR_2); 
fprintf(1, '\n\n'); 

fprintf(1, 'Population regression of nominal 5yr on\n');
fprintf(1, 'expected 5-yr consumption growth and inflation\n');
fprintf(1, '\n'); 
fprintf(1, ...
        'Coef, SE on expected consumption growth are %7.2f, %7.2f\n', ...
        regOutput.fiveYRcoef1(1), regCoefSEs(3));
fprintf(1, 'Coef, SE on expected inflation is %7.2f, %7.2f\n', ...
        regOutput.fiveYRcoef1(2), regCoefSEs(4));
fprintf(1, 'Rsquare is %7.2f\n', regOutput.R2_5YR_1); 
fprintf(1, '\n\n'); 

%
%
%

path(origPath) 

