% Greg Duffee
%
% Table 10.  A very long file that goes model by model.

clear
close all

origPath = path;
path('..\codeToEstimateGeneralModel', origPath);

%==============================================================
%
% First derivative step size
%
%==============================================================

delta = 1e-8; 

%==============================================================
%
% Recursive utility parameters
%
%==============================================================

beta = 0.999; 
gamma = 10; 

fprintf(1, '\n'); 
fprintf(1, 'Mean excess return per quarter (percent)\n'); 
fprintf(1, '\n'); 

%==============================================================
%
% Get the 3-factor model estimated with macro data and the
% three-month yield, 1952Q2 -- 1985Q4.  
%
%==============================================================

estimatedModel = char('..\genModFlatFiles\q_3factorEarly3.out'); 

tolerance = 1e-12;

[yielddatafile numbonds numUsedBonds usedBondIdx ...
 macrodatafile numMacro numUsedMacro usedMacroIdx ...
 numfactors firstobs lastobs ...
 use_ols_start olsStartIdx ...
 numloops numSmallLoops numsimplex numderiv ...
 mu_names mu_flag mu mu_rand_stddev ...
 k_names k_flag k_vec k_rand_stddev ...
 S_names S_flag Sigma_vec S_rand_stddev ...
 A_name A_flag A A_rand_stddev ...
 B_names B_flag B_vec B_rand_stddev ...
 errorSD_name errorSD_flag errorSDs errorSD_rand_stddev ...
] = read_input(estimatedModel);

use_ols_start = 0;

observedData = ...
    read_data(yielddatafile, macrodatafile, numbonds, usedBondIdx, ...
              numMacro, usedMacroIdx, firstobs, lastobs);

[T numObserved] = size(observedData);

[freeparams bigparamvec free_index] = setup_initparams( ...
    mu, k_vec, Sigma_vec, A, B_vec, ...
    errorSDs, mu_flag, k_flag, S_flag, ...
    A_flag, B_flag, errorSD_flag, use_ols_start, olsStartIdx, ...
    observedData);

numfreeparams = length(freeparams); 

[score outerProd scoreMat] = ...
    kalman_derivs(freeparams, bigparamvec, free_index, ...
                  observedData, numfactors, tolerance, ...
                  delta);

[OPinv ok] = invpd(outerProd);

horizon = 19; 
output19 = covarFunction3factor(freeparams, bigparamvec, free_index, ...
                       numfactors, numObserved, horizon, ...
                       beta, gamma); 

horizon = 19; 

dRP = zeros(numfreeparams, 1); 

for thisParam = 1:numfreeparams
  freeCopy = freeparams;

  freeCopy(thisParam) = freeparams(thisParam) + delta;
  altOutputUp = covarFunction3factor(freeCopy, bigparamvec, free_index, ...
                          numfactors, numObserved, horizon, ...
                          beta, gamma); 

  freeCopy(thisParam) = freeparams(thisParam) - delta;
  altOutputDown = covarFunction3factor(freeCopy, bigparamvec, free_index, ...
                          numfactors, numObserved, horizon, ...
                          beta, gamma); 

  dRP(thisParam) = (altOutputUp.riskPremiumHorizonPlus1 - ...
         altOutputDown.riskPremiumHorizonPlus1)/(2*delta);
end

stdErr16 = sqrt(dRP'*OPinv*dRP);


fprintf(1,'3 factor, macro + 3mon yield, 1952-1985\n\n');

fprintf(1, '%8.3f\n', output19.riskPremiumHorizonPlus1*100)
fprintf(1, '(%7.3f)\n', 100*stdErr16);
fprintf(1, '\n'); 

%==============================================================
%
% Get the 4-factor model estimated with macro data and the
% three-month, 5 yr yields, 1952Q2 -- 1985Q4.  
%
%==============================================================

estimatedModel = char('..\genModFlatFiles\q_4factor3m5yrEarly3a.out'); 

tolerance = 1e-12;

[yielddatafile numbonds numUsedBonds usedBondIdx ...
 macrodatafile numMacro numUsedMacro usedMacroIdx ...
 numfactors firstobs lastobs ...
 use_ols_start olsStartIdx ...
 numloops numSmallLoops numsimplex numderiv ...
 mu_names mu_flag mu mu_rand_stddev ...
 k_names k_flag k_vec k_rand_stddev ...
 S_names S_flag Sigma_vec S_rand_stddev ...
 A_name A_flag A A_rand_stddev ...
 B_names B_flag B_vec B_rand_stddev ...
 errorSD_name errorSD_flag errorSDs errorSD_rand_stddev ...
] = read_input(estimatedModel);

use_ols_start = 0;

observedData = ...
    read_data(yielddatafile, macrodatafile, numbonds, usedBondIdx, ...
              numMacro, usedMacroIdx, firstobs, lastobs);

[T numObserved] = size(observedData);

[freeparams bigparamvec free_index] = setup_initparams( ...
    mu, k_vec, Sigma_vec, A, B_vec, ...
    errorSDs, mu_flag, k_flag, S_flag, ...
    A_flag, B_flag, errorSD_flag, use_ols_start, olsStartIdx, ...
    observedData);

numfreeparams = length(freeparams); 

[score outerProd scoreMat] = ...
    kalman_derivs(freeparams, bigparamvec, free_index, ...
                  observedData, numfactors, tolerance, ...
                  delta);

[OPinv ok] = invpd(outerProd);

horizon = 19; 
output19 = covarFunction(freeparams, bigparamvec, free_index, ...
                       numfactors, numObserved, horizon, ...
                       beta, gamma); 

horizon = 19; 

dRP = zeros(numfreeparams, 1); 

for thisParam = 1:numfreeparams
  freeCopy = freeparams;

  freeCopy(thisParam) = freeparams(thisParam) + delta;
  altOutputUp = covarFunction(freeCopy, bigparamvec, free_index, ...
                          numfactors, numObserved, horizon, ...
                          beta, gamma); 

  freeCopy(thisParam) = freeparams(thisParam) - delta;
  altOutputDown = covarFunction(freeCopy, bigparamvec, free_index, ...
                          numfactors, numObserved, horizon, ...
                          beta, gamma); 

  dRP(thisParam) = (altOutputUp.riskPremiumHorizonPlus1 - ...
         altOutputDown.riskPremiumHorizonPlus1)/(2*delta);
end

stdErr16 = sqrt(dRP'*OPinv*dRP);


fprintf(1,'4 factor, macro + 3mon, 5 yr yields, 1952-1985\n\n');

fprintf(1, '%8.3f\n', output19.riskPremiumHorizonPlus1*100)
fprintf(1, '(%7.3f)\n', 100*stdErr16);
fprintf(1, '\n'); 

%==============================================================
%
% Get the 2-factor model estimated with macro data, 1952Q2 --
% 1985Q4.
%
%==============================================================

estimatedModel = char('..\genModFlatFiles\q_2factor.out'); 

tolerance = 1e-12;

[yielddatafile numbonds numUsedBonds usedBondIdx ...
 macrodatafile numMacro numUsedMacro usedMacroIdx ...
 numfactors firstobs lastobs ...
 use_ols_start olsStartIdx ...
 numloops numSmallLoops numsimplex numderiv ...
 mu_names mu_flag mu mu_rand_stddev ...
 k_names k_flag k_vec k_rand_stddev ...
 S_names S_flag Sigma_vec S_rand_stddev ...
 A_name A_flag A A_rand_stddev ...
 B_names B_flag B_vec B_rand_stddev ...
 errorSD_name errorSD_flag errorSDs errorSD_rand_stddev ...
] = read_input(estimatedModel);

use_ols_start = 0;

observedData = ...
    read_data(yielddatafile, macrodatafile, numbonds, usedBondIdx, ...
              numMacro, usedMacroIdx, firstobs, lastobs);

[T numObserved] = size(observedData);

[freeparams bigparamvec free_index] = setup_initparams( ...
    mu, k_vec, Sigma_vec, A, B_vec, ...
    errorSDs, mu_flag, k_flag, S_flag, ...
    A_flag, B_flag, errorSD_flag, use_ols_start, olsStartIdx, ...
    observedData);

numfreeparams = length(freeparams); 

[score outerProd scoreMat] = ...
    kalman_derivs(freeparams, bigparamvec, free_index, ...
                  observedData, numfactors, tolerance, ...
                  delta);

[OPinv ok] = invpd(outerProd);

horizon = 19; 
output19 = covarFunction2factor(freeparams, bigparamvec, free_index, ...
                       numfactors, numObserved, horizon, ...
                       beta, gamma); 

horizon = 19; 

dRP = zeros(numfreeparams, 1); 

for thisParam = 1:numfreeparams
  freeCopy = freeparams;

  freeCopy(thisParam) = freeparams(thisParam) + delta;
  altOutputUp = covarFunction2factor(freeCopy, bigparamvec, free_index, ...
                          numfactors, numObserved, horizon, ...
                          beta, gamma); 

  freeCopy(thisParam) = freeparams(thisParam) - delta;
  altOutputDown = covarFunction2factor(freeCopy, bigparamvec, free_index, ...
                          numfactors, numObserved, horizon, ...
                          beta, gamma); 

  dRP(thisParam) = (altOutputUp.riskPremiumHorizonPlus1 - ...
         altOutputDown.riskPremiumHorizonPlus1)/(2*delta);
end

stdErr16 = sqrt(dRP'*OPinv*dRP);


fprintf(1,'2 factor, macro, 1952-2010\n\n');

fprintf(1, '%8.3f\n', output19.riskPremiumHorizonPlus1*100)
fprintf(1, '(%7.3f)\n', 100*stdErr16);
fprintf(1, '\n'); 

%==============================================================
%
% Get the 3-factor model estimated with macro data and the
% three-month yield, 1952Q2 -- 2010Q4.  
%
%==============================================================

estimatedModel = char('..\genModFlatFiles\q_3factor.out'); 

tolerance = 1e-12;

[yielddatafile numbonds numUsedBonds usedBondIdx ...
 macrodatafile numMacro numUsedMacro usedMacroIdx ...
 numfactors firstobs lastobs ...
 use_ols_start olsStartIdx ...
 numloops numSmallLoops numsimplex numderiv ...
 mu_names mu_flag mu mu_rand_stddev ...
 k_names k_flag k_vec k_rand_stddev ...
 S_names S_flag Sigma_vec S_rand_stddev ...
 A_name A_flag A A_rand_stddev ...
 B_names B_flag B_vec B_rand_stddev ...
 errorSD_name errorSD_flag errorSDs errorSD_rand_stddev ...
] = read_input(estimatedModel);

use_ols_start = 0;

observedData = ...
    read_data(yielddatafile, macrodatafile, numbonds, usedBondIdx, ...
              numMacro, usedMacroIdx, firstobs, lastobs);

[T numObserved] = size(observedData);

[freeparams bigparamvec free_index] = setup_initparams( ...
    mu, k_vec, Sigma_vec, A, B_vec, ...
    errorSDs, mu_flag, k_flag, S_flag, ...
    A_flag, B_flag, errorSD_flag, use_ols_start, olsStartIdx, ...
    observedData);

numfreeparams = length(freeparams); 

[score outerProd scoreMat] = ...
    kalman_derivs(freeparams, bigparamvec, free_index, ...
                  observedData, numfactors, tolerance, ...
                  delta);

[OPinv ok] = invpd(outerProd);

horizon = 19; 
output19 = covarFunction3factor(freeparams, bigparamvec, free_index, ...
                       numfactors, numObserved, horizon, ...
                       beta, gamma); 

horizon = 19; 

dRP = zeros(numfreeparams, 1); 

for thisParam = 1:numfreeparams
  freeCopy = freeparams;

  freeCopy(thisParam) = freeparams(thisParam) + delta;
  altOutputUp = covarFunction3factor(freeCopy, bigparamvec, free_index, ...
                          numfactors, numObserved, horizon, ...
                          beta, gamma); 

  freeCopy(thisParam) = freeparams(thisParam) - delta;
  altOutputDown = covarFunction3factor(freeCopy, bigparamvec, free_index, ...
                          numfactors, numObserved, horizon, ...
                          beta, gamma); 

  dRP(thisParam) = (altOutputUp.riskPremiumHorizonPlus1 - ...
         altOutputDown.riskPremiumHorizonPlus1)/(2*delta);
end

stdErr16 = sqrt(dRP'*OPinv*dRP);


fprintf(1,'3 factor, macro + 3mon yield, 1952-2010\n\n');

fprintf(1, '%8.3f\n', output19.riskPremiumHorizonPlus1*100)
fprintf(1, '(%7.3f)\n', 100*stdErr16);
fprintf(1, '\n'); 

%==============================================================
%
% Get the 4-factor model estimated with macro data and the
% three-month, 5 yr yields, 1952Q2 -- 2010Q4.  
%
%==============================================================

estimatedModel = char('..\genModFlatFiles\q_4factor3m5yra.out'); 

tolerance = 1e-12;

[yielddatafile numbonds numUsedBonds usedBondIdx ...
 macrodatafile numMacro numUsedMacro usedMacroIdx ...
 numfactors firstobs lastobs ...
 use_ols_start olsStartIdx ...
 numloops numSmallLoops numsimplex numderiv ...
 mu_names mu_flag mu mu_rand_stddev ...
 k_names k_flag k_vec k_rand_stddev ...
 S_names S_flag Sigma_vec S_rand_stddev ...
 A_name A_flag A A_rand_stddev ...
 B_names B_flag B_vec B_rand_stddev ...
 errorSD_name errorSD_flag errorSDs errorSD_rand_stddev ...
] = read_input(estimatedModel);

use_ols_start = 0;

observedData = ...
    read_data(yielddatafile, macrodatafile, numbonds, usedBondIdx, ...
              numMacro, usedMacroIdx, firstobs, lastobs);

[T numObserved] = size(observedData);

[freeparams bigparamvec free_index] = setup_initparams( ...
    mu, k_vec, Sigma_vec, A, B_vec, ...
    errorSDs, mu_flag, k_flag, S_flag, ...
    A_flag, B_flag, errorSD_flag, use_ols_start, olsStartIdx, ...
    observedData);

numfreeparams = length(freeparams); 

[score outerProd scoreMat] = ...
    kalman_derivs(freeparams, bigparamvec, free_index, ...
                  observedData, numfactors, tolerance, ...
                  delta);

[OPinv ok] = invpd(outerProd);

horizon = 19; 
output19 = covarFunction(freeparams, bigparamvec, free_index, ...
                       numfactors, numObserved, horizon, ...
                       beta, gamma); 

horizon = 19; 

dRP = zeros(numfreeparams, 1); 

for thisParam = 1:numfreeparams
  freeCopy = freeparams;

  freeCopy(thisParam) = freeparams(thisParam) + delta;
  altOutputUp = covarFunction(freeCopy, bigparamvec, free_index, ...
                          numfactors, numObserved, horizon, ...
                          beta, gamma); 

  freeCopy(thisParam) = freeparams(thisParam) - delta;
  altOutputDown = covarFunction(freeCopy, bigparamvec, free_index, ...
                          numfactors, numObserved, horizon, ...
                          beta, gamma); 

  dRP(thisParam) = (altOutputUp.riskPremiumHorizonPlus1 - ...
         altOutputDown.riskPremiumHorizonPlus1)/(2*delta);
end

stdErr16 = sqrt(dRP'*OPinv*dRP);


fprintf(1,'4 factor, macro + 3mon, 5 yr yields, 1952-2010\n\n');

fprintf(1, '%8.3f\n', output19.riskPremiumHorizonPlus1*100)
fprintf(1, '(%7.3f)\n', 100*stdErr16);
fprintf(1, '\n'); 

%==============================================================
%
% Get the 4-factor model estimated with macro data and the
% three-month, 5 yr yields, 1986Q1 -- 2007Q4.  
%
%==============================================================

estimatedModel = char('..\genModFlatFiles\q_4factor3m5yrShorta.out'); 

tolerance = 1e-12;

[yielddatafile numbonds numUsedBonds usedBondIdx ...
 macrodatafile numMacro numUsedMacro usedMacroIdx ...
 numfactors firstobs lastobs ...
 use_ols_start olsStartIdx ...
 numloops numSmallLoops numsimplex numderiv ...
 mu_names mu_flag mu mu_rand_stddev ...
 k_names k_flag k_vec k_rand_stddev ...
 S_names S_flag Sigma_vec S_rand_stddev ...
 A_name A_flag A A_rand_stddev ...
 B_names B_flag B_vec B_rand_stddev ...
 errorSD_name errorSD_flag errorSDs errorSD_rand_stddev ...
] = read_input(estimatedModel);

use_ols_start = 0;

observedData = ...
    read_data(yielddatafile, macrodatafile, numbonds, usedBondIdx, ...
              numMacro, usedMacroIdx, firstobs, lastobs);

[T numObserved] = size(observedData);

[freeparams bigparamvec free_index] = setup_initparams( ...
    mu, k_vec, Sigma_vec, A, B_vec, ...
    errorSDs, mu_flag, k_flag, S_flag, ...
    A_flag, B_flag, errorSD_flag, use_ols_start, olsStartIdx, ...
    observedData);

numfreeparams = length(freeparams); 

[score outerProd scoreMat] = ...
    kalman_derivs(freeparams, bigparamvec, free_index, ...
                  observedData, numfactors, tolerance, ...
                  delta);

[OPinv ok] = invpd(outerProd);

horizon = 19; 
output19 = covarFunction(freeparams, bigparamvec, free_index, ...
                       numfactors, numObserved, horizon, ...
                       beta, gamma); 

horizon = 19; 

dRP = zeros(numfreeparams, 1); 

for thisParam = 1:numfreeparams
  freeCopy = freeparams;

  freeCopy(thisParam) = freeparams(thisParam) + delta;
  altOutputUp = covarFunction(freeCopy, bigparamvec, free_index, ...
                          numfactors, numObserved, horizon, ...
                          beta, gamma); 

  freeCopy(thisParam) = freeparams(thisParam) - delta;
  altOutputDown = covarFunction(freeCopy, bigparamvec, free_index, ...
                          numfactors, numObserved, horizon, ...
                          beta, gamma); 

  dRP(thisParam) = (altOutputUp.riskPremiumHorizonPlus1 - ...
         altOutputDown.riskPremiumHorizonPlus1)/(2*delta);
end

stdErr16 = sqrt(dRP'*OPinv*dRP);


fprintf(1,'4 factor, macro + 3mon, 5 yr yields, 1986-2007\n\n');

fprintf(1, '%8.3f\n', output19.riskPremiumHorizonPlus1*100)
fprintf(1, '(%7.3f)\n', 100*stdErr16);
fprintf(1, '\n'); 

%
%
%

path(origPath) 

