% Greg Duffee
%
% Table 11

clear
close all

origPath = path;
path('..\codeToEstimateGeneralModel', origPath);

%==============================================================
%
% Recursive utility parameters
%
%==============================================================

beta = 0.999; 
gamma = 10; 

fprintf(1, '\n'); 
fprintf(1, 'Sample covariances for evaluation of risk premia models\n'); 
fprintf(1, '\n'); 

%==============================================================
%
% file names and sample periods
%
%==============================================================

modelForDataRetrieval = ...
    char('..\genModFlatFiles\q_4factor3m5yra.out');
TwoFactorModelForEstimation = ...
    char('..\genModFlatFiles\q_2factorEarly3.out');
ThreeFactorModelForEstimation = ...
    char('..\genModFlatFiles\q_3factorEarly3.out');

fullSample = (2:235)';
earlySample = (2:135)';
lateSample = (136:235)';

%==============================================================
%
% first get the entire data set, including the two bond yields
%
%==============================================================

tolerance = 1e-12;

[yielddatafile numbonds numUsedBonds usedBondIdx ...
 macrodatafile numMacro numUsedMacro usedMacroIdx ...
 numfactors firstobs lastobs ...
 use_ols_start olsStartIdx ...
 numloops numSmallLoops numsimplex numderiv ...
 mu_names mu_flag mu mu_rand_stddev ...
 k_names k_flag k_vec k_rand_stddev ...
 S_names S_flag Sigma_vec S_rand_stddev ...
 A_name A_flag A A_rand_stddev ...
 B_names B_flag B_vec B_rand_stddev ...
 errorSD_name errorSD_flag errorSDs errorSD_rand_stddev ...
] = read_input(modelForDataRetrieval);

allObservedData = ...
    read_data(yielddatafile, macrodatafile, numbonds, usedBondIdx, ...
              numMacro, usedMacroIdx, firstobs, lastobs);

% first-difference the yields
%

diffYields = [repmat(NaN, 1, 2); ...
              allObservedData(2:end, 1:2) - ...
              allObservedData(1:(end-1),1:2)];

%==============================================================
%
% Now get the model estimated with just macro data.
%
%==============================================================

tolerance = 1e-12;

[yielddatafile numbonds numUsedBonds usedBondIdx ...
 macrodatafile numMacro numUsedMacro usedMacroIdx ...
 numfactors firstobs lastobs ...
 use_ols_start olsStartIdx ...
 numloops numSmallLoops numsimplex numderiv ...
 mu_names mu_flag mu mu_rand_stddev ...
 k_names k_flag k_vec k_rand_stddev ...
 S_names S_flag Sigma_vec S_rand_stddev ...
 A_name A_flag A A_rand_stddev ...
 B_names B_flag B_vec B_rand_stddev ...
 errorSD_name errorSD_flag errorSDs errorSD_rand_stddev ...
] = read_input(TwoFactorModelForEstimation);

lastobs = 235; % to get filtered estimates throughout the sample.  

observedData = ...
    read_data(yielddatafile, macrodatafile, numbonds, usedBondIdx, ...
              numMacro, usedMacroIdx, firstobs, lastobs);

[freeparams bigparamvec free_index] = setup_initparams( ...
    mu, k_vec, Sigma_vec, A, B_vec, ...
    errorSDs, mu_flag, k_flag, S_flag, ...
    A_flag, B_flag, errorSD_flag, use_ols_start, olsStartIdx, ...
    observedData);

[Kmat Sigma B H] = ...
    build_matrices(k_vec, Sigma_vec, B_vec, errorSDs); 

stateMean = inv(eye(numfactors) - Kmat)*mu; 

[stateCovMat valid] = uncon_vcv_general(Kmat, Sigma); 

factor_SDs = sqrt(diag(stateCovMat)); 

[funval loglikes filtered_values errors] = ...
    kalman_only_loglike(freeparams, bigparamvec, free_index, ...
                        observedData, numfactors, tolerance); 

[T numObserved] = size(observedData);

filteredObs = repmat(A', T, 1) + filtered_values*B'; 

crossSectErrors = 400*(observedData - filteredObs);

[filteredContempStates smoothedStates ...
          filteredContempCov smoothedCov] = ...
    kalman_smoothing_recursion(observedData, mu, Kmat, Sigma, H, A, B); 

errorsUsingSmoothed = smoothedStates(2:end,:) - ...
    repmat(mu', T-1, 1) - smoothedStates(1:(end-1),:)*Kmat';
errorsUsingSmoothed = [(smoothedStates(1,:) - stateMean'); ...
                    errorsUsingSmoothed];

%  recursive utility
%
% remember order of observables is 3m, 5yr, pi, delta c

maxMat = 40; 

sumVec = B(end,:) * inv(eye(numfactors) - beta*Kmat);

smoothedSumConsShock = errorsUsingSmoothed*sumVec';

allSmoothedShocks = [errorsUsingSmoothed*B' smoothedSumConsShock];

earlySmoothedCovs = cov([diffYields(earlySample,:) ...
                    allSmoothedShocks(earlySample,:)]); 

lateSmoothedCovs = cov([diffYields(lateSample,:) ...
                    allSmoothedShocks(lateSample,:)]); 

fullSmoothedCovs = cov([diffYields(fullSample,:) ...
                    allSmoothedShocks(fullSample,:)]); 


fprintf(1, '\n');
fprintf(1, 'Two-factor model\n'); 
fprintf(1, '\n');
fprintf(1, 'Sample covariances between first-differenced\n');
fprintf(1, 'nominal bond yields and smoothed shocks\n\n'); 
fprintf(1, '              delta c   pi    infinite sum\n');
fprintf(1, '--------------------------------------------\n');
fprintf(1, 'Early sample\n'); 
fprintf(1, '3-month yield  %8.3f %8.3f %8.3f\n', ...
        1e4*earlySmoothedCovs(1,end-1), 1e4*earlySmoothedCovs(1,end-2), ...
        1e4*earlySmoothedCovs(1,end));
fprintf(1, 'five-yr yield  %8.3f %8.3f %8.3f\n', ...
        1e4*earlySmoothedCovs(2,end-1), 1e4*earlySmoothedCovs(2,end-2), ...
        1e4*earlySmoothedCovs(2,end));
fprintf(1, 'Late sample\n'); 
fprintf(1, '3-month yield  %8.3f %8.3f %8.3f\n', ...
        1e4*lateSmoothedCovs(1,end-1), 1e4*lateSmoothedCovs(1,end-2), ...
        1e4*lateSmoothedCovs(1,end));
fprintf(1, 'five-yr yield  %8.3f %8.3f %8.3f\n', ...
        1e4*lateSmoothedCovs(2,end-1), 1e4*lateSmoothedCovs(2,end-2), ...
        1e4*lateSmoothedCovs(2,end));
fprintf(1, 'Full sample\n'); 
fprintf(1, '3-month yield  %8.3f %8.3f %8.3f\n', ...
        1e4*fullSmoothedCovs(1,end-1), 1e4*fullSmoothedCovs(1,end-2), ...
        1e4*fullSmoothedCovs(1,end));
fprintf(1, 'five-yr yield  %8.3f %8.3f %8.3f\n', ...
        1e4*fullSmoothedCovs(2,end-1), 1e4*fullSmoothedCovs(2,end-2), ...
        1e4*fullSmoothedCovs(2,end));
fprintf(1, '\n\n'); 

%==============================================================
%
% Now get the model estimated with macro data, 3m yield.
%
%==============================================================

tolerance = 1e-12;

[yielddatafile numbonds numUsedBonds usedBondIdx ...
 macrodatafile numMacro numUsedMacro usedMacroIdx ...
 numfactors firstobs lastobs ...
 use_ols_start olsStartIdx ...
 numloops numSmallLoops numsimplex numderiv ...
 mu_names mu_flag mu mu_rand_stddev ...
 k_names k_flag k_vec k_rand_stddev ...
 S_names S_flag Sigma_vec S_rand_stddev ...
 A_name A_flag A A_rand_stddev ...
 B_names B_flag B_vec B_rand_stddev ...
 errorSD_name errorSD_flag errorSDs errorSD_rand_stddev ...
] = read_input(ThreeFactorModelForEstimation);

lastobs = 235; % to get filtered estimates throughout the sample.  

observedData = ...
    read_data(yielddatafile, macrodatafile, numbonds, usedBondIdx, ...
              numMacro, usedMacroIdx, firstobs, lastobs);

[freeparams bigparamvec free_index] = setup_initparams( ...
    mu, k_vec, Sigma_vec, A, B_vec, ...
    errorSDs, mu_flag, k_flag, S_flag, ...
    A_flag, B_flag, errorSD_flag, use_ols_start, olsStartIdx, ...
    observedData);

[Kmat Sigma B H] = ...
    build_matrices(k_vec, Sigma_vec, B_vec, errorSDs); 

stateMean = inv(eye(numfactors) - Kmat)*mu; 

[stateCovMat valid] = uncon_vcv_general(Kmat, Sigma); 

factor_SDs = sqrt(diag(stateCovMat)); 

[funval loglikes filtered_values errors] = ...
    kalman_only_loglike(freeparams, bigparamvec, free_index, ...
                        observedData, numfactors, tolerance); 

[T numObserved] = size(observedData);

filteredObs = repmat(A', T, 1) + filtered_values*B'; 

crossSectErrors = 400*(observedData - filteredObs);

[filteredContempStates smoothedStates ...
          filteredContempCov smoothedCov] = ...
    kalman_smoothing_recursion(observedData, mu, Kmat, Sigma, H, A, B); 

errorsUsingSmoothed = smoothedStates(2:end,:) - ...
    repmat(mu', T-1, 1) - smoothedStates(1:(end-1),:)*Kmat';
errorsUsingSmoothed = [(smoothedStates(1,:) - stateMean'); ...
                    errorsUsingSmoothed];

%  recursive utility
%
% remember order of observables is 3m, 5yr, pi, delta c

maxMat = 40; 

sumVec = B(end,:) * inv(eye(numfactors) - beta*Kmat);

smoothedSumConsShock = errorsUsingSmoothed*sumVec';

allSmoothedShocks = [errorsUsingSmoothed*B' smoothedSumConsShock];

earlySmoothedCovs = cov([diffYields(earlySample,:) ...
                    allSmoothedShocks(earlySample,:)]); 

lateSmoothedCovs = cov([diffYields(lateSample,:) ...
                    allSmoothedShocks(lateSample,:)]); 

fullSmoothedCovs = cov([diffYields(fullSample,:) ...
                    allSmoothedShocks(fullSample,:)]); 


fprintf(1, '\n');
fprintf(1, 'Three-factor model\n'); 
fprintf(1, '\n');
fprintf(1, 'Sample covariances between first-differenced\n');
fprintf(1, 'nominal bond yields and smoothed shocks\n\n'); 
fprintf(1, '              delta c   pi    infinite sum\n');
fprintf(1, '--------------------------------------------\n');
fprintf(1, 'Early sample\n'); 
fprintf(1, '3-month yield  %8.3f %8.3f %8.3f\n', ...
        1e4*earlySmoothedCovs(1,end-1), 1e4*earlySmoothedCovs(1,end-2), ...
        1e4*earlySmoothedCovs(1,end));
fprintf(1, 'five-yr yield  %8.3f %8.3f %8.3f\n', ...
        1e4*earlySmoothedCovs(2,end-1), 1e4*earlySmoothedCovs(2,end-2), ...
        1e4*earlySmoothedCovs(2,end));
fprintf(1, 'Late sample\n'); 
fprintf(1, '3-month yield  %8.3f %8.3f %8.3f\n', ...
        1e4*lateSmoothedCovs(1,end-1), 1e4*lateSmoothedCovs(1,end-2), ...
        1e4*lateSmoothedCovs(1,end));
fprintf(1, 'five-yr yield  %8.3f %8.3f %8.3f\n', ...
        1e4*lateSmoothedCovs(2,end-1), 1e4*lateSmoothedCovs(2,end-2), ...
        1e4*lateSmoothedCovs(2,end));
fprintf(1, 'Full sample\n'); 
fprintf(1, '3-month yield  %8.3f %8.3f %8.3f\n', ...
        1e4*fullSmoothedCovs(1,end-1), 1e4*fullSmoothedCovs(1,end-2), ...
        1e4*fullSmoothedCovs(1,end));
fprintf(1, 'five-yr yield  %8.3f %8.3f %8.3f\n', ...
        1e4*fullSmoothedCovs(2,end-1), 1e4*fullSmoothedCovs(2,end-2), ...
        1e4*fullSmoothedCovs(2,end));
fprintf(1, '\n\n'); 


%
%
%

path(origPath) 

