% Greg Duffee
%
% Table 9.

clear
close all

origPath = path;
path('..\codeToEstimateGeneralModel', origPath);

%==============================================================
%
% First derivative step size
%
%==============================================================

delta = 1e-8; 

%==============================================================
%
% Recursive utility parameters
%
%==============================================================

beta = 0.999; 
gamma = 10; 

%==============================================================
%
% Get the model estimated with just macro data.
%
%==============================================================

table9Model = char('..\genModFlatFiles\q_2factorEarly3.out');

tolerance = 1e-12;

[yielddatafile numbonds numUsedBonds usedBondIdx ...
 macrodatafile numMacro numUsedMacro usedMacroIdx ...
 numfactors firstobs lastobs ...
 use_ols_start olsStartIdx ...
 numloops numSmallLoops numsimplex numderiv ...
 mu_names mu_flag mu mu_rand_stddev ...
 k_names k_flag k_vec k_rand_stddev ...
 S_names S_flag Sigma_vec S_rand_stddev ...
 A_name A_flag A A_rand_stddev ...
 B_names B_flag B_vec B_rand_stddev ...
 errorSD_name errorSD_flag errorSDs errorSD_rand_stddev ...
] = read_input(table9Model);

use_ols_start = 0;

lastobs = 235;

observedData = ...
    read_data(yielddatafile, macrodatafile, numbonds, usedBondIdx, ...
              numMacro, usedMacroIdx, firstobs, lastobs);

[T numObserved] = size(observedData);

[freeparams bigparamvec free_index] = setup_initparams( ...
    mu, k_vec, Sigma_vec, A, B_vec, ...
    errorSDs, mu_flag, k_flag, S_flag, ...
    A_flag, B_flag, errorSD_flag, use_ols_start, olsStartIdx, ...
    observedData);

numfreeparams = length(freeparams); 

[score outerProd scoreMat] = ...
    kalman_derivs(freeparams, bigparamvec, free_index, ...
                  observedData, numfactors, tolerance, ...
                  delta);

[OPinv ok] = invpd(outerProd);

horizon = 1; 
output1 = covarFunction2factor(freeparams, bigparamvec, free_index, ...
                       numfactors, numObserved, horizon, ...
                       beta, gamma); 

horizon = 19; 
output19 = covarFunction2factor(freeparams, bigparamvec, free_index, ...
                       numfactors, numObserved, horizon, ...
                       beta, gamma); 

horizon = 20; 
output20 = covarFunction2factor(freeparams, bigparamvec, free_index, ...
                       numfactors, numObserved, horizon, ...
                       beta, gamma); 

%==============================================================
%
% Standard errors for one-quarter horizon.
%
%==============================================================

horizon = 1; 

dCmeanC = zeros(numfreeparams, 1);
dCmeanPi = zeros(numfreeparams, 1); 
dPimeanC = zeros(numfreeparams, 1);
dPimeanPi = zeros(numfreeparams, 1);
dSUMmeanC = zeros(numfreeparams, 1);
dSUMmeanPi = zeros(numfreeparams, 1);
dCRealBondYield = zeros(numfreeparams, 1);
dCNomBondYield = zeros(numfreeparams, 1);
dPiRealBondYield = zeros(numfreeparams, 1);
dPiNomBondYield = zeros(numfreeparams, 1);
dSUMRealBondYield = zeros(numfreeparams, 1);
dSUMNomBondYield = zeros(numfreeparams, 1);
dRP = zeros(numfreeparams, 1); 

for thisParam = 1:numfreeparams
  freeCopy = freeparams;

  freeCopy(thisParam) = freeparams(thisParam) + delta;
  outputUp = covarFunction2factor(freeCopy, bigparamvec, free_index, ...
                          numfactors, numObserved, horizon, ...
                          beta, gamma); 

  freeCopy(thisParam) = freeparams(thisParam) - delta;
  outputDown = covarFunction2factor(freeCopy, bigparamvec, free_index, ...
                          numfactors, numObserved, horizon, ...
                          beta, gamma); 

  dCmeanPi(thisParam) = (outputUp.covCmeanPi - ...
                         outputDown.covCmeanPi)/(2*delta); 
  dCmeanC(thisParam) = (outputUp.covCmeanC - ...
                         outputDown.covCmeanC)/(2*delta); 
  dSUMmeanPi(thisParam) = (outputUp.covSUMmeanPi - ...
                         outputDown.covSUMmeanPi)/(2*delta); 
  dSUMmeanC(thisParam) = (outputUp.covSUMmeanC - ...
                         outputDown.covSUMmeanC)/(2*delta); 
  dPimeanPi(thisParam) = (outputUp.covPimeanPi - ...
                         outputDown.covPimeanPi)/(2*delta); 
  dPimeanC(thisParam) = (outputUp.covPimeanC - ...
                         outputDown.covPimeanC)/(2*delta); 
  dCNomBondYield(thisParam) = (outputUp.covCNomBondYield - ...
                         outputDown.covCNomBondYield)/(2*delta); 
  dCRealBondYield(thisParam) = (outputUp.covCRealBondYield - ...
                         outputDown.covCRealBondYield)/(2*delta); 
  dSUMNomBondYield(thisParam) = (outputUp.covSUMNomBondYield - ...
                         outputDown.covSUMNomBondYield)/(2*delta); 
  dSUMRealBondYield(thisParam) = (outputUp.covSUMRealBondYield - ...
                         outputDown.covSUMRealBondYield)/(2*delta); 
  dPiNomBondYield(thisParam) = (outputUp.covPiNomBondYield - ...
                         outputDown.covPiNomBondYield)/(2*delta); 
  dPiRealBondYield(thisParam) = (outputUp.covPiRealBondYield - ...
                         outputDown.covPiRealBondYield)/(2*delta); 
  dRP(thisParam) = (outputUp.riskPremiumHorizonPlus1 - ...
         outputDown.riskPremiumHorizonPlus1)/(2*delta);
end


stdErr1 = sqrt(dCmeanC'*OPinv*dCmeanC);
stdErr2 = sqrt(dCmeanPi'*OPinv*dCmeanPi);
stdErr3 = sqrt(dPimeanC'*OPinv*dPimeanC);
stdErr4 = sqrt(dPimeanPi'*OPinv*dPimeanPi);
stdErr5 = sqrt(dSUMmeanC'*OPinv*dSUMmeanC);
stdErr6 = sqrt(dSUMmeanPi'*OPinv*dSUMmeanPi);
stdErr7 = sqrt(dCRealBondYield'*OPinv*dCRealBondYield);
stdErr8 = sqrt(dCNomBondYield'*OPinv*dCNomBondYield);
stdErr9 = sqrt(dPiRealBondYield'*OPinv*dPiRealBondYield);
stdErr10 = sqrt(dPiNomBondYield'*OPinv*dPiNomBondYield);
stdErr11 = sqrt(dSUMRealBondYield'*OPinv*dSUMRealBondYield);
stdErr12 = sqrt(dSUMNomBondYield'*OPinv*dSUMNomBondYield);
stdErr16 = sqrt(dRP'*OPinv*dRP);

%
% below are standard errors for the covariance of the log SDF with
% the log bond yields.  Nominal, real. 

tmp = dCNomBondYield + (gamma-1)*dSUMNomBondYield + dPiNomBondYield;
stdErrSum1 = sqrt(tmp'*OPinv*tmp);
tmp = dCRealBondYield + (gamma-1)*dSUMRealBondYield + dPiRealBondYield;
stdErrSum2 = sqrt(tmp'*OPinv*tmp);


stdErrs_1 = [stdErr1; stdErr2; stdErr3; stdErr4; stdErr5; stdErr6; ...
             stdErr7; stdErr8; stdErr9; stdErr10; stdErr11; stdErr12; ...
            stdErr16];

stdErrsSums_1 = [stdErrSum1; stdErrSum2];

sum1 = (output1.covCNomBondYield + ...
              output20.covPiNomBondYield + ...
              (gamma - 1)*output20.covSUMNomBondYield);
sum2 = (output1.covCRealBondYield + ...
              output20.covPiRealBondYield + ...
              (gamma - 1)*output20.covSUMRealBondYield);
  
sums_1 = [sum1; sum2];

sumTstats_1 = sums_1 ./ stdErrsSums_1;

%==============================================================
%
% Standard errors for the five-year horizon.
%
%==============================================================

horizon = 20; 

dCmeanC = zeros(numfreeparams, 1);
dCmeanPi = zeros(numfreeparams, 1); 
dPimeanC = zeros(numfreeparams, 1);
dPimeanPi = zeros(numfreeparams, 1);
dSUMmeanC = zeros(numfreeparams, 1);
dSUMmeanPi = zeros(numfreeparams, 1);
dCRealBondYield = zeros(numfreeparams, 1);
dCNomBondYield = zeros(numfreeparams, 1);
dPiRealBondYield = zeros(numfreeparams, 1);
dPiNomBondYield = zeros(numfreeparams, 1);
dSUMRealBondYield = zeros(numfreeparams, 1);
dSUMNomBondYield = zeros(numfreeparams, 1);
dRP = zeros(numfreeparams, 1); 

for thisParam = 1:numfreeparams
  freeCopy = freeparams;

  freeCopy(thisParam) = freeparams(thisParam) + delta;

  horizon = 20; 
  outputUp = covarFunction2factor(freeCopy, bigparamvec, free_index, ...
                          numfactors, numObserved, horizon, ...
                          beta, gamma); 
  horizon = 19; 
  altOutputUp = covarFunction2factor(freeCopy, bigparamvec, free_index, ...
                          numfactors, numObserved, horizon, ...
                          beta, gamma); 

  freeCopy(thisParam) = freeparams(thisParam) - delta;
  horizon = 20; 
  outputDown = covarFunction2factor(freeCopy, bigparamvec, free_index, ...
                          numfactors, numObserved, horizon, ...
                          beta, gamma); 
  horizon = 19; 
  altOutputDown = covarFunction2factor(freeCopy, bigparamvec, free_index, ...
                          numfactors, numObserved, horizon, ...
                          beta, gamma); 
  
  dCmeanPi(thisParam) = (outputUp.covCmeanPi - ...
                         outputDown.covCmeanPi)/(2*delta); 
  dCmeanC(thisParam) = (outputUp.covCmeanC - ...
                         outputDown.covCmeanC)/(2*delta); 
  dSUMmeanPi(thisParam) = (outputUp.covSUMmeanPi - ...
                         outputDown.covSUMmeanPi)/(2*delta); 
  dSUMmeanC(thisParam) = (outputUp.covSUMmeanC - ...
                         outputDown.covSUMmeanC)/(2*delta); 
  dPimeanPi(thisParam) = (outputUp.covPimeanPi - ...
                         outputDown.covPimeanPi)/(2*delta); 
  dPimeanC(thisParam) = (outputUp.covPimeanC - ...
                         outputDown.covPimeanC)/(2*delta); 
  dCNomBondYield(thisParam) = (outputUp.covCNomBondYield - ...
                         outputDown.covCNomBondYield)/(2*delta); 
  dCRealBondYield(thisParam) = (outputUp.covCRealBondYield - ...
                         outputDown.covCRealBondYield)/(2*delta); 
  dSUMNomBondYield(thisParam) = (outputUp.covSUMNomBondYield - ...
                         outputDown.covSUMNomBondYield)/(2*delta); 
  dSUMRealBondYield(thisParam) = (outputUp.covSUMRealBondYield - ...
                         outputDown.covSUMRealBondYield)/(2*delta); 
  dPiNomBondYield(thisParam) = (outputUp.covPiNomBondYield - ...
                         outputDown.covPiNomBondYield)/(2*delta); 
  dPiRealBondYield(thisParam) = (outputUp.covPiRealBondYield - ...
                         outputDown.covPiRealBondYield)/(2*delta); 
  dRP(thisParam) = (altOutputUp.riskPremiumHorizonPlus1 - ...
         altOutputDown.riskPremiumHorizonPlus1)/(2*delta);
end


stdErr1 = sqrt(dCmeanC'*OPinv*dCmeanC);
stdErr2 = sqrt(dCmeanPi'*OPinv*dCmeanPi);
stdErr3 = sqrt(dPimeanC'*OPinv*dPimeanC);
stdErr4 = sqrt(dPimeanPi'*OPinv*dPimeanPi);
stdErr5 = sqrt(dSUMmeanC'*OPinv*dSUMmeanC);
stdErr6 = sqrt(dSUMmeanPi'*OPinv*dSUMmeanPi);
stdErr7 = sqrt(dCRealBondYield'*OPinv*dCRealBondYield);
stdErr8 = sqrt(dCNomBondYield'*OPinv*dCNomBondYield);
stdErr9 = sqrt(dPiRealBondYield'*OPinv*dPiRealBondYield);
stdErr10 = sqrt(dPiNomBondYield'*OPinv*dPiNomBondYield);
stdErr11 = sqrt(dSUMRealBondYield'*OPinv*dSUMRealBondYield);
stdErr12 = sqrt(dSUMNomBondYield'*OPinv*dSUMNomBondYield);
stdErr16 = sqrt(dRP'*OPinv*dRP);

tmp = dCNomBondYield + (gamma-1)*dSUMNomBondYield + dPiNomBondYield;
stdErrSum1 = sqrt(tmp'*OPinv*tmp);
tmp = dCRealBondYield + (gamma-1)*dSUMRealBondYield + dPiRealBondYield;
stdErrSum2 = sqrt(tmp'*OPinv*tmp);


stdErrs_20 = [stdErr1; stdErr2; stdErr3; stdErr4; stdErr5; stdErr6; ...
             stdErr7; stdErr8; stdErr9; stdErr10; stdErr11; stdErr12; ...
             stdErr16];

stdErrsSums_20 = [stdErrSum1; stdErrSum2];

sum1 = (output20.covCNomBondYield + ...
              output20.covPiNomBondYield + ...
              (gamma - 1)*output20.covSUMNomBondYield);
sum2 = (output20.covCRealBondYield + ...
              output20.covPiRealBondYield + ...
              (gamma - 1)*output20.covSUMRealBondYield);

sums_20 = [sum1; sum2];

sumTstats_20 = sums_20 ./ stdErrsSums_20;

fprintf(1, '\n'); 
fprintf(1, 'Properties of an estimated factor model\n'); 
fprintf(1, '1952Q2 through 1985Q4\n'); 
fprintf(1, '\n'); 
fprintf(1, 'beta = %8.3f\n', beta); 
fprintf(1, 'gamma = %8.3f\n', gamma); 
fprintf(1, '\n'); 
fprintf(1, 'A. Covariances among innovations\n\n'); 

fprintf(1, '         delta c      pi          infinite sum\n'); 
fprintf(1, '      --------------------------------------\n\n'); 
fprintf(1, 'next d-c %8.3f %8.3f %8.3f \n', ...
        1e4*output1.covCmeanC, 1e4*output1.covPimeanC, ...
        1e4*output1.covSUMmeanC);
fprintf(1, '        (%7.3f) (%7.3f)  (%7.3f) \n', ...
        1e4*stdErrs_1(1), 1e4*stdErrs_1(3), ...
        1e4*stdErrs_1(5));
fprintf(1, 'next  pi %8.3f %8.3f %8.3f \n', ...
        1e4*output1.covCmeanPi, 1e4*output1.covPimeanPi, ...
        1e4*output1.covSUMmeanPi);
fprintf(1, '        (%7.3f) (%7.3f)  (%7.3f) \n', ...
        1e4*stdErrs_1(2), 1e4*stdErrs_1(4), ...
        1e4*stdErrs_1(6));
fprintf(1, 'avg d-c  %8.3f %8.3f %8.3f \n', ...
        1e4*output20.covCmeanC, 1e4*output20.covPimeanC, ...
        1e4*output20.covSUMmeanC);
fprintf(1, '        (%7.3f) (%7.3f)  (%7.3f) \n', ...
        1e4*stdErrs_20(1), 1e4*stdErrs_20(3), ...
        1e4*stdErrs_20(5));
fprintf(1, 'avg pi   %8.3f %8.3f %8.3f \n', ...
        1e4*output20.covCmeanPi, 1e4*output20.covPimeanPi, ...
        1e4*output20.covSUMmeanPi);
fprintf(1, '        (%7.3f) (%7.3f)  (%7.3f) \n', ...
        1e4*stdErrs_20(2), 1e4*stdErrs_20(4), ...
        1e4*stdErrs_20(6));
fprintf(1, '\n\n'); 
fprintf(1, ...
        'B. Risk premium on a 20-quarter bond, in percent/quarter\n');

fprintf(1, 'Value (SE)\n');
fprintf(1, '----------\n'); 
fprintf(1, '%8.3f \n', output19.riskPremiumHorizonPlus1*100);
fprintf(1, '(%7.3f) \n', 100*stdErrs_20(end));

fprintf(1, '\n\n'); 

%
%
%

path(origPath) 

