% Greg Duffee
%
% Table 14


clear
close all

%===============================================================
%
% First produce results for the general factor model.  Then go to
% the New Keynesian model
%
%===============================================================

origPath = path;
path('..\codeToEstimateGeneralModel', origPath);


%===============================================================
%
% File names for the general factor model
%
%===============================================================

model3factor_3mon = char('..\genModFlatFiles\IP_3factor3mShort.out');
model4factor_3mon = char('..\genModFlatFiles\IP_4factor3mShort.out'); 
model4factor_3mon5yr = char('..\genModFlatFiles\IP_4factor3m5yrShorta.out');
model4factor_manyBonds = char('..\genModFlatFiles\IP_4factorManyShorta.out');

%===============================================================
%
% Parameter
%
%===============================================================

tolerance = 1e-12;

%===============================================================
%
% Three factors, 3-month yield
%
%===============================================================

[yielddatafile numbonds numUsedBonds usedBondIdx ...
 macrodatafile numMacro numUsedMacro usedMacroIdx ...
 numfactors firstobs lastobs ...
 use_ols_start olsStartIdx ...
 numloops numSmallLoops numsimplex numderiv ...
 mu_names mu_flag mu mu_rand_stddev ...
 k_names k_flag k_vec k_rand_stddev ...
 S_names S_flag Sigma_vec S_rand_stddev ...
 A_name A_flag A A_rand_stddev ...
 B_names B_flag B_vec B_rand_stddev ...
 errorSD_name errorSD_flag errorSDs errorSD_rand_stddev ...
] = read_input(model3factor_3mon);

use_ols_start = 0;
numObserved = numUsedBonds + numUsedMacro; 

observedData = ...
    read_data(yielddatafile, macrodatafile, numbonds, usedBondIdx, ...
              numMacro, usedMacroIdx, firstobs, lastobs);

[freeparams bigparamvec free_index] = setup_initparams( ...
    mu, k_vec, Sigma_vec, A, B_vec, ...
    errorSDs, mu_flag, k_flag, S_flag, ...
    A_flag, B_flag, errorSD_flag, use_ols_start, olsStartIdx, ...
    observedData);


[Kmat Sigma B H] = ...
    build_matrices(k_vec, Sigma_vec, B_vec, errorSDs); 

stateMean = inv(eye(numfactors) - Kmat)*mu; 

meanObservables = A + B * stateMean; 

[stateCovMat valid] = uncon_vcv_general(Kmat, Sigma); 

factor_SDs = sqrt(diag(stateCovMat)); 

[funval loglikes_3_1bond filtered_values errors] = ...
    kalman_only_loglike(freeparams, bigparamvec, free_index, ...
                        observedData, numfactors, tolerance); 

[T numObserved] = size(observedData);

filteredObs = repmat(A', T, 1) + filtered_values*B'; 

crossSectErrors = 400*(observedData - filteredObs);

RMSE_CSErrors_3factor_3mon = sqrt(mean(crossSectErrors .^2));

filteredDeviations = filtered_values - repmat(stateMean', T, 1);

stepsAhead = 1; 
stateForecast = repmat(stateMean', T, 1) + ...
    filteredDeviations*((Kmat^stepsAhead)');
stateErrors = [filtered_values((1+stepsAhead):end,:) - ...
               stateForecast(1:(end-stepsAhead),:); ...
               repmat(NaN, stepsAhead, numfactors)];
obsForecast = repmat(A', T, 1) + stateForecast*B';
obsErrors = [ observedData((1+stepsAhead):end,:) - ...
              obsForecast(1:(end-stepsAhead),:); ...
              repmat(NaN, stepsAhead, numObserved) ];
RMSE_Forecast_3factor_3mon = 400*sqrt(nanmean( obsErrors .^2)); 

%===============================================================
%
% Four factors, 3 month yield
%
%===============================================================

[yielddatafile numbonds numUsedBonds usedBondIdx ...
 macrodatafile numMacro numUsedMacro usedMacroIdx ...
 numfactors firstobs lastobs ...
 use_ols_start olsStartIdx ...
 numloops numSmallLoops numsimplex numderiv ...
 mu_names mu_flag mu mu_rand_stddev ...
 k_names k_flag k_vec k_rand_stddev ...
 S_names S_flag Sigma_vec S_rand_stddev ...
 A_name A_flag A A_rand_stddev ...
 B_names B_flag B_vec B_rand_stddev ...
 errorSD_name errorSD_flag errorSDs errorSD_rand_stddev ...
] = read_input(model4factor_3mon);

use_ols_start = 0;
numObserved = numUsedBonds + numUsedMacro; 

observedData = ...
    read_data(yielddatafile, macrodatafile, numbonds, usedBondIdx, ...
              numMacro, usedMacroIdx, firstobs, lastobs);

[freeparams bigparamvec free_index] = setup_initparams( ...
    mu, k_vec, Sigma_vec, A, B_vec, ...
    errorSDs, mu_flag, k_flag, S_flag, ...
    A_flag, B_flag, errorSD_flag, use_ols_start, olsStartIdx, ...
    observedData);


[Kmat Sigma B H] = ...
    build_matrices(k_vec, Sigma_vec, B_vec, errorSDs); 

stateMean = inv(eye(numfactors) - Kmat)*mu; 

meanObservables = A + B * stateMean; 

[stateCovMat valid] = uncon_vcv_general(Kmat, Sigma); 

factor_SDs = sqrt(diag(stateCovMat)); 

[funval loglikes_3_1bond filtered_values errors] = ...
    kalman_only_loglike(freeparams, bigparamvec, free_index, ...
                        observedData, numfactors, tolerance); 

[T numObserved] = size(observedData);

filteredObs = repmat(A', T, 1) + filtered_values*B'; 

crossSectErrors = 400*(observedData - filteredObs);

RMSE_CSErrors_4factor_3mon = sqrt(mean(crossSectErrors .^2));

filteredDeviations = filtered_values - repmat(stateMean', T, 1);

stepsAhead = 1; 
stateForecast = repmat(stateMean', T, 1) + ...
    filteredDeviations*((Kmat^stepsAhead)');
stateErrors = [filtered_values((1+stepsAhead):end,:) - ...
               stateForecast(1:(end-stepsAhead),:); ...
               repmat(NaN, stepsAhead, numfactors)];
obsForecast = repmat(A', T, 1) + stateForecast*B';
obsErrors = [ observedData((1+stepsAhead):end,:) - ...
              obsForecast(1:(end-stepsAhead),:); ...
              repmat(NaN, stepsAhead, numObserved) ];
RMSE_Forecast_4factor_3mon = 400*sqrt(nanmean( obsErrors .^2)); 

%===============================================================
%
% Four factors, 3-month, 5-year yields.
%
%===============================================================

[yielddatafile numbonds numUsedBonds usedBondIdx ...
 macrodatafile numMacro numUsedMacro usedMacroIdx ...
 numfactors firstobs lastobs ...
 use_ols_start olsStartIdx ...
 numloops numSmallLoops numsimplex numderiv ...
 mu_names mu_flag mu mu_rand_stddev ...
 k_names k_flag k_vec k_rand_stddev ...
 S_names S_flag Sigma_vec S_rand_stddev ...
 A_name A_flag A A_rand_stddev ...
 B_names B_flag B_vec B_rand_stddev ...
 errorSD_name errorSD_flag errorSDs errorSD_rand_stddev ...
] = read_input(model4factor_3mon5yr);

use_ols_start = 0;
numObserved = numUsedBonds + numUsedMacro; 

observedData = ...
    read_data(yielddatafile, macrodatafile, numbonds, usedBondIdx, ...
              numMacro, usedMacroIdx, firstobs, lastobs);

[freeparams bigparamvec free_index] = setup_initparams( ...
    mu, k_vec, Sigma_vec, A, B_vec, ...
    errorSDs, mu_flag, k_flag, S_flag, ...
    A_flag, B_flag, errorSD_flag, use_ols_start, olsStartIdx, ...
    observedData);


[Kmat Sigma B H] = ...
    build_matrices(k_vec, Sigma_vec, B_vec, errorSDs); 

stateMean = inv(eye(numfactors) - Kmat)*mu; 

meanObservables = A + B * stateMean; 

[stateCovMat valid] = uncon_vcv_general(Kmat, Sigma); 

factor_SDs = sqrt(diag(stateCovMat)); 

[funval loglikes_3_1bond filtered_values errors] = ...
    kalman_only_loglike(freeparams, bigparamvec, free_index, ...
                        observedData, numfactors, tolerance); 

[T numObserved] = size(observedData);

filteredObs = repmat(A', T, 1) + filtered_values*B'; 

crossSectErrors = 400*(observedData - filteredObs);

RMSE_CSErrors_4factor_3mon5yr = sqrt(mean(crossSectErrors .^2));

filteredDeviations = filtered_values - repmat(stateMean', T, 1);

stepsAhead = 1; 
stateForecast = repmat(stateMean', T, 1) + ...
    filteredDeviations*((Kmat^stepsAhead)');
stateErrors = [filtered_values((1+stepsAhead):end,:) - ...
               stateForecast(1:(end-stepsAhead),:); ...
               repmat(NaN, stepsAhead, numfactors)];
obsForecast = repmat(A', T, 1) + stateForecast*B';
obsErrors = [ observedData((1+stepsAhead):end,:) - ...
              obsForecast(1:(end-stepsAhead),:); ...
              repmat(NaN, stepsAhead, numObserved) ];
RMSE_Forecast_4factor_3mon5yr = 400*sqrt(nanmean( obsErrors .^2)); 

%===============================================================
%
% get results for model the six-bond case. 
%
%===============================================================

[yielddatafile numbonds numUsedBonds usedBondIdx ...
 macrodatafile numMacro numUsedMacro usedMacroIdx ...
 numfactors firstobs lastobs ...
 use_ols_start olsStartIdx ...
 numloops numSmallLoops numsimplex numderiv ...
 mu_names mu_flag mu mu_rand_stddev ...
 k_names k_flag k_vec k_rand_stddev ...
 S_names S_flag Sigma_vec S_rand_stddev ...
 A_name A_flag A A_rand_stddev ...
 B_names B_flag B_vec B_rand_stddev ...
 errorSD_name errorSD_flag errorSDs errorSD_rand_stddev ...
] = read_input(model4factor_manyBonds);

use_ols_start = 0;
numObserved = numUsedBonds + numUsedMacro; 

observedData = ...
    read_data(yielddatafile, macrodatafile, numbonds, usedBondIdx, ...
              numMacro, usedMacroIdx, firstobs, lastobs);

[freeparams bigparamvec free_index] = setup_initparams( ...
    mu, k_vec, Sigma_vec, A, B_vec, ...
    errorSDs, mu_flag, k_flag, S_flag, ...
    A_flag, B_flag, errorSD_flag, use_ols_start, olsStartIdx, ...
    observedData);


[Kmat Sigma B H] = ...
    build_matrices(k_vec, Sigma_vec, B_vec, errorSDs); 

stateMean = inv(eye(numfactors) - Kmat)*mu; 

meanObservables = A + B * stateMean; 

[stateCovMat valid] = uncon_vcv_general(Kmat, Sigma); 

factor_SDs = sqrt(diag(stateCovMat)); 

[funval loglikes_3_1bond filtered_values errors] = ...
    kalman_only_loglike(freeparams, bigparamvec, free_index, ...
                        observedData, numfactors, tolerance); 

[T numObserved] = size(observedData);

filteredObs = repmat(A', T, 1) + filtered_values*B'; 

crossSectErrors = 400*(observedData - filteredObs);

RMSE_CSErrors_4factor_manyBonds = sqrt(mean(crossSectErrors .^2));

filteredDeviations = filtered_values - repmat(stateMean', T, 1);

stepsAhead = 1; 
stateForecast = repmat(stateMean', T, 1) + ...
    filteredDeviations*((Kmat^stepsAhead)');
stateErrors = [filtered_values((1+stepsAhead):end,:) - ...
               stateForecast(1:(end-stepsAhead),:); ...
               repmat(NaN, stepsAhead, numfactors)];
obsForecast = repmat(A', T, 1) + stateForecast*B';
obsErrors = [ observedData((1+stepsAhead):end,:) - ...
              obsForecast(1:(end-stepsAhead),:); ...
              repmat(NaN, stepsAhead, numObserved) ];
RMSE_Forecast_4factor_manyBonds = 400*sqrt(nanmean( obsErrors .^2)); 



%===============================================================
%
% Display results.
%
%===============================================================


fprintf(1, '\n');
fprintf(1, 'A.  Unrestricted models\n\n'); 
fprintf(1, '               3-fac, 3-mon  ');
fprintf(1, '  4-fac, 3-mon  4-fac, 3m,5yr  4-fac, many\n'); 
fprintf(1, '\n');
fprintf(1, 'Cross-sectional fitting errors\n\n');
fprintf(1, 'Three-month yield ');
fprintf(1,'    %8.3f  %8.3f  %8.3f %8.3f \n', ...
        RMSE_CSErrors_3factor_3mon(1), ...
        RMSE_CSErrors_4factor_3mon(1), ...
        RMSE_CSErrors_4factor_3mon5yr(1), ...
        RMSE_CSErrors_4factor_manyBonds(1));
fprintf(1, 'Inflation         ');
fprintf(1,'    %8.3f  %8.3f  %8.3f %8.3f \n', ...
        RMSE_CSErrors_3factor_3mon(end-1), ...
        RMSE_CSErrors_4factor_3mon(end-1), ...
        RMSE_CSErrors_4factor_3mon5yr(end-1), ...
        RMSE_CSErrors_4factor_manyBonds(end-1));
fprintf(1, 'log IP            ');
fprintf(1,'    %8.3f  %8.3f  %8.3f %8.3f \n', ...
        RMSE_CSErrors_3factor_3mon(end), ...
        RMSE_CSErrors_4factor_3mon(end), ...
        RMSE_CSErrors_4factor_3mon5yr(end), ...
        RMSE_CSErrors_4factor_manyBonds(end));
fprintf(1, '\n');
fprintf(1, 'One-quarter-ahead forecasting errors\n\n');
fprintf(1, 'Three-month yield ');
fprintf(1,'    %8.3f  %8.3f  %8.3f %8.3f \n', ...
        RMSE_Forecast_3factor_3mon(1), ...
        RMSE_Forecast_4factor_3mon(1), ...
        RMSE_Forecast_4factor_3mon5yr(1), ...
        RMSE_Forecast_4factor_manyBonds(1));
fprintf(1, 'Inflation         ');
fprintf(1,'    %8.3f  %8.3f  %8.3f %8.3f \n', ...
        RMSE_Forecast_3factor_3mon(end-1), ...
        RMSE_Forecast_4factor_3mon(end-1), ...
        RMSE_Forecast_4factor_3mon5yr(end-1), ...
        RMSE_Forecast_4factor_manyBonds(end-1));
fprintf(1, 'log IP            ');
fprintf(1,'    %8.3f  %8.3f  %8.3f %8.3f \n', ...
        RMSE_Forecast_3factor_3mon(end), ...
        RMSE_Forecast_4factor_3mon(end), ...
        RMSE_Forecast_4factor_3mon5yr(end), ...
        RMSE_Forecast_4factor_manyBonds(end));
fprintf(1, '\n');

%
% wrap up
%


path(origPath) 

%===============================================================
%
% Now turn to the the New Keynesian model
%
%===============================================================


origPath = path;
path('..\codeToEstimateKeynesian', origPath);


%===============================================================
%
% File names for the New Keynesian model
%
%===============================================================

Keynes_3mon = char('..\keynesianFlatFiles\noBonds02Shortb.out');
Keynes_3mon5yr = char('..\keynesianFlatFiles\oneBond01Shortc.out');
Keynes_manyBonds = char('..\keynesianFlatFiles\manyBonds01Shorta.out');

%===============================================================
%
% Parameters 
%
%===============================================================

tolerance = 1e-12;
numfactors = 4; 

%===============================================================
%
% get results for model with 3-month yield.  
%
%===============================================================

[yielddatafile numbonds numUsedBonds usedBondIdx ...
 macrodatafile numMacro usedMacroIdx ...
 firstobs lastobs  ...
 numloops numSmallLoops numsimplex numderiv ...
 means_names means_flag means means_rand_stddev ...
 bcm_names bcm_flag bcm bcm_rand_stddev ...
 stddev_names stddev_flag stddev stddev_rand_stddev ...
 A_names A_flag Avec A_rand_stddev ...
 B_names B_flag Bvec B_rand_stddev ...
 errorSD_names errorSD_flag errorSD errorSD_rand_stddev ...
] = read_input(Keynes_3mon);

[freeparams allparams free_index] = setup_initparams( ...
    means, bcm, stddev, Avec, Bvec, errorSD, ...
    means_flag, bcm_flag, stddev_flag, ...
    A_flag, B_flag, errorSD_flag);

observedData = ...
    read_data(yielddatafile, ...
              macrodatafile, numbonds, usedBondIdx, ...
              numMacro, usedMacroIdx, firstobs, lastobs); 

[T numObserved] = size(observedData);

RE_output = solveRE(bcm, stddev); 

[delta0 delta1 mu K Sigma bailout] = buildRESolution(RE_output, ...
                                                  means);

[A B H] = build_matrices(delta0, delta1, Avec, Bvec, errorSD);

[errors loglikes filtered_values cond_vars contemp_vars ...
 normSqrErrors eigFs] = ...
    kalman_recursion_noderiv(observedData, mu, K, Sigma, H, A, B, ...
                             tolerance);

stateMean = inv(eye(4) - K)*mu; 
[stateCovMat valid] = uncon_vcv_general(K, Sigma); 

filteredObs = repmat(A', T, 1) + filtered_values*B'; 

crossSectErrors = 400*(observedData - filteredObs);

RMSE_CSErrors_3mon = sqrt(mean(crossSectErrors .^2));

filteredDeviations = filtered_values - repmat(stateMean', T, 1);

stepsAhead = 1; 
stateForecast = repmat(stateMean', T, 1) + ...
    filteredDeviations*((K^stepsAhead)');
stateErrors = [filtered_values((1+stepsAhead):end,:) - ...
               stateForecast(1:(end-stepsAhead),:); ...
               repmat(NaN, stepsAhead, numfactors)];
obsForecast = repmat(A', T, 1) + stateForecast*B';
obsErrors = [ observedData((1+stepsAhead):end,:) - ...
              obsForecast(1:(end-stepsAhead),:); ...
              repmat(NaN, stepsAhead, numObserved) ];
RMSE_Forecast_3mon = 400*sqrt(nanmean( obsErrors .^2)); 

%===============================================================
%
% get results for model with 3 month, 5-year yields.
%
%===============================================================

[yielddatafile numbonds numUsedBonds usedBondIdx ...
 macrodatafile numMacro usedMacroIdx ...
 firstobs lastobs  ...
 numloops numSmallLoops numsimplex numderiv ...
 means_names means_flag means means_rand_stddev ...
 bcm_names bcm_flag bcm bcm_rand_stddev ...
 stddev_names stddev_flag stddev stddev_rand_stddev ...
 A_names A_flag Avec A_rand_stddev ...
 B_names B_flag Bvec B_rand_stddev ...
 errorSD_names errorSD_flag errorSD errorSD_rand_stddev ...
] = read_input(Keynes_3mon5yr);

[freeparams allparams free_index] = setup_initparams( ...
    means, bcm, stddev, Avec, Bvec, errorSD, ...
    means_flag, bcm_flag, stddev_flag, ...
    A_flag, B_flag, errorSD_flag);

observedData = ...
    read_data(yielddatafile, ...
              macrodatafile, numbonds, usedBondIdx, ...
              numMacro, usedMacroIdx, firstobs, lastobs); 

[T numObserved] = size(observedData);

RE_output = solveRE(bcm, stddev); 

[delta0 delta1 mu K Sigma bailout] = buildRESolution(RE_output, ...
                                                  means);

[A B H] = build_matrices(delta0, delta1, Avec, Bvec, errorSD);

[errors loglikes filtered_values cond_vars contemp_vars ...
 normSqrErrors eigFs] = ...
    kalman_recursion_noderiv(observedData, mu, K, Sigma, H, A, B, ...
                             tolerance);

stateMean = inv(eye(4) - K)*mu; 
[stateCovMat valid] = uncon_vcv_general(K, Sigma); 

filteredObs = repmat(A', T, 1) + filtered_values*B'; 

crossSectErrors = 400*(observedData - filteredObs);

RMSE_CSErrors_3mon5yr = sqrt(mean(crossSectErrors .^2));

filteredDeviations = filtered_values - repmat(stateMean', T, 1);

stepsAhead = 1; 
stateForecast = repmat(stateMean', T, 1) + ...
    filteredDeviations*((K^stepsAhead)');
stateErrors = [filtered_values((1+stepsAhead):end,:) - ...
               stateForecast(1:(end-stepsAhead),:); ...
               repmat(NaN, stepsAhead, numfactors)];
obsForecast = repmat(A', T, 1) + stateForecast*B';
obsErrors = [ observedData((1+stepsAhead):end,:) - ...
              obsForecast(1:(end-stepsAhead),:); ...
              repmat(NaN, stepsAhead, numObserved) ];
RMSE_Forecast_3mon5yr = 400*sqrt(nanmean( obsErrors .^2)); 

%===============================================================
%
% get results for model with six bonds
%
%===============================================================

[yielddatafile numbonds numUsedBonds usedBondIdx ...
 macrodatafile numMacro usedMacroIdx ...
 firstobs lastobs  ...
 numloops numSmallLoops numsimplex numderiv ...
 means_names means_flag means means_rand_stddev ...
 bcm_names bcm_flag bcm bcm_rand_stddev ...
 stddev_names stddev_flag stddev stddev_rand_stddev ...
 A_names A_flag Avec A_rand_stddev ...
 B_names B_flag Bvec B_rand_stddev ...
 errorSD_names errorSD_flag errorSD errorSD_rand_stddev ...
] = read_input(Keynes_manyBonds);

[freeparams allparams free_index] = setup_initparams( ...
    means, bcm, stddev, Avec, Bvec, errorSD, ...
    means_flag, bcm_flag, stddev_flag, ...
    A_flag, B_flag, errorSD_flag);

observedData = ...
    read_data(yielddatafile, ...
              macrodatafile, numbonds, usedBondIdx, ...
              numMacro, usedMacroIdx, firstobs, lastobs); 

[T numObserved] = size(observedData);

RE_output = solveRE(bcm, stddev); 

[delta0 delta1 mu K Sigma bailout] = buildRESolution(RE_output, ...
                                                  means);

[A B H] = build_matrices(delta0, delta1, Avec, Bvec, errorSD);

[errors loglikes filtered_values cond_vars contemp_vars ...
 normSqrErrors eigFs] = ...
    kalman_recursion_noderiv(observedData, mu, K, Sigma, H, A, B, ...
                             tolerance);

stateMean = inv(eye(4) - K)*mu; 
[stateCovMat valid] = uncon_vcv_general(K, Sigma); 

filteredObs = repmat(A', T, 1) + filtered_values*B'; 

crossSectErrors = 400*(observedData - filteredObs);

RMSE_CSErrors_manyBonds = sqrt(mean(crossSectErrors .^2));

filteredDeviations = filtered_values - repmat(stateMean', T, 1);

stepsAhead = 1; 
stateForecast = repmat(stateMean', T, 1) + ...
    filteredDeviations*((K^stepsAhead)');
stateErrors = [filtered_values((1+stepsAhead):end,:) - ...
               stateForecast(1:(end-stepsAhead),:); ...
               repmat(NaN, stepsAhead, numfactors)];
obsForecast = repmat(A', T, 1) + stateForecast*B';
obsErrors = [ observedData((1+stepsAhead):end,:) - ...
              obsForecast(1:(end-stepsAhead),:); ...
              repmat(NaN, stepsAhead, numObserved) ];
RMSE_Forecast_manyBonds = 400*sqrt(nanmean( obsErrors .^2)); 

%===============================================================
%
% Display results.
%
%===============================================================


fprintf(1, '\n');
fprintf(1, 'B.  Restricted 4-factor models\n\n'); 
fprintf(1, '                    no long bonds  ');
fprintf(1, '  One long bond    Five long bonds\n'); 
fprintf(1, '\n');
fprintf(1, 'Cross-sectional fitting errors\n\n');
fprintf(1, 'Three-month yield ');
fprintf(1,'    %8.3f        %8.3f %8.3f \n', ...
        RMSE_CSErrors_3mon(1), ...
        RMSE_CSErrors_3mon5yr(1), ...
        RMSE_CSErrors_manyBonds(1));
fprintf(1, 'Inflation         ');
fprintf(1,'    %8.3f        %8.3f %8.3f \n', ...
        RMSE_CSErrors_3mon(end-1), ...
        RMSE_CSErrors_3mon5yr(end-1), ...
        RMSE_CSErrors_manyBonds(end-1));
fprintf(1, 'log IP            ');
fprintf(1,'    %8.3f        %8.3f %8.3f \n', ...
        RMSE_CSErrors_3mon(end), ...
        RMSE_CSErrors_3mon5yr(end), ...
        RMSE_CSErrors_manyBonds(end));
fprintf(1, '\n');
fprintf(1, 'One-quarter-ahead forecasting errors\n\n');
fprintf(1, 'Three-month yield ');
fprintf(1,'    %8.3f        %8.3f %8.3f \n', ...
        RMSE_Forecast_3mon(1), ...
        RMSE_Forecast_3mon5yr(1), ...
        RMSE_Forecast_manyBonds(1));
fprintf(1, 'Inflation         ');
fprintf(1,'    %8.3f        %8.3f %8.3f \n', ...
        RMSE_Forecast_3mon(end-1), ...
        RMSE_Forecast_3mon5yr(end-1), ...
        RMSE_Forecast_manyBonds(end-1));
fprintf(1, 'log IP            ');
fprintf(1,'    %8.3f        %8.3f %8.3f \n', ...
        RMSE_Forecast_3mon(end), ...
        RMSE_Forecast_3mon5yr(end), ...
        RMSE_Forecast_manyBonds(end));
fprintf(1, '\n');


%
% wrap up
%


origPath = path;
