% Greg Duffee
%
% Table 5

clear
close all

%============================================================
%
% How many steps ahead to forecast? 
%
%============================================================

stepsAhead = 1; 

%============================================================
%
% First the results for two general factor models.
%
%============================================================

origPath = path;
path('..\codeToEstimateGeneralModel', origPath);


twoBond_4factor = char('..\genModFlatFiles\q_4factor3m5yr.out');
sixBond_4factor = char('..\genModFlatFiles\q_4factorManya.out');

%
% get results for the two-bond case. 
%

tolerance = 1e-12;

[yielddatafile numbonds numUsedBonds usedBondIdx ...
 macrodatafile numMacro numUsedMacro usedMacroIdx ...
 numfactors firstobs lastobs ...
 use_ols_start olsStartIdx ...
 numloops numSmallLoops numsimplex numderiv ...
 mu_names mu_flag mu mu_rand_stddev ...
 k_names k_flag k_vec k_rand_stddev ...
 S_names S_flag Sigma_vec S_rand_stddev ...
 A_name A_flag A A_rand_stddev ...
 B_names B_flag B_vec B_rand_stddev ...
 errorSD_name errorSD_flag errorSDs errorSD_rand_stddev ...
] = read_input(twoBond_4factor);

use_ols_start = 0;
numObserved = numUsedBonds + numUsedMacro; 

observedData = ...
    read_data(yielddatafile, macrodatafile, numbonds, usedBondIdx, ...
              numMacro, usedMacroIdx, firstobs, lastobs);

[freeparams bigparamvec free_index] = setup_initparams( ...
    mu, k_vec, Sigma_vec, A, B_vec, ...
    errorSDs, mu_flag, k_flag, S_flag, ...
    A_flag, B_flag, errorSD_flag, use_ols_start, olsStartIdx, ...
    observedData);


[Kmat Sigma B H] = ...
    build_matrices(k_vec, Sigma_vec, B_vec, errorSDs); 

stateMean = inv(eye(numfactors) - Kmat)*mu; 

meanObservables = A + B * stateMean; 

unconMean_2bond = meanObservables; 


[stateCovMat valid] = uncon_vcv_general(Kmat, Sigma); 

factor_SDs = sqrt(diag(stateCovMat)); 

[funval loglikes filtered_values errors] = ...
    kalman_only_loglike(freeparams, bigparamvec, free_index, ...
                        observedData, numfactors, tolerance); 

filtered_values_2bond = filtered_values; 

sampleMean_2bond = A + B * mean(filtered_values_2bond)';

[T numObserved] = size(observedData);

filteredObs = repmat(A', T, 1) + filtered_values*B'; 

filtered_inflation_2bond = filteredObs(:,end-1);
filtered_cgrowth_2bond = filteredObs(:,end);

numNegCgrowth = sum(observedData(:,end) < 0);
numFilteredNegCgrowth = sum(filtered_cgrowth_2bond < 0); 

tmpmat = eye(numfactors);
if (stepsAhead > 1)
  for i = 1:(stepsAhead-1)
    tmpmat = tmpmat + Kmat^i;
  end
end

stateForecastAhead_2bond = repmat((tmpmat*mu)', T, 1) + ...
    filtered_values*(Kmat^stepsAhead)';

obsForecastAhead_2bond = repmat(A', T, 1) + ...
    stateForecastAhead_2bond*B';

crossSectErrors = 400*(observedData - filteredObs);

obsForecastErrors = obsForecastAhead_2bond(1:(end-stepsAhead),:) ...
    - observedData((1+stepsAhead):end,:);

RMSE_CSErrors_2bond = sqrt(mean(crossSectErrors .^2));
rmse_of_errors_2bond = sqrt(mean(obsForecastErrors.^2)); 


%
% get results for the six-bond case. 
%

[yielddatafile numbonds numUsedBonds usedBondIdx ...
 macrodatafile numMacro numUsedMacro usedMacroIdx ...
 numfactors firstobs lastobs ...
 use_ols_start olsStartIdx ...
 numloops numSmallLoops numsimplex numderiv ...
 mu_names mu_flag mu mu_rand_stddev ...
 k_names k_flag k_vec k_rand_stddev ...
 S_names S_flag Sigma_vec S_rand_stddev ...
 A_name A_flag A A_rand_stddev ...
 B_names B_flag B_vec B_rand_stddev ...
 errorSD_name errorSD_flag errorSDs errorSD_rand_stddev ...
] = read_input(sixBond_4factor);

use_ols_start = 0;
numObserved = numUsedBonds + numUsedMacro; 

observedData = ...
    read_data(yielddatafile, macrodatafile, numbonds, usedBondIdx, ...
              numMacro, usedMacroIdx, firstobs, lastobs);

meanData = mean(observedData);
stdData = std(observedData); 


maturities = [0.25; 1; 2; 3; 4; 5];

[freeparams bigparamvec free_index] = setup_initparams( ...
    mu, k_vec, Sigma_vec, A, B_vec, ...
    errorSDs, mu_flag, k_flag, S_flag, ...
    A_flag, B_flag, errorSD_flag, use_ols_start, olsStartIdx, ...
    observedData);


[Kmat Sigma B H] = ...
    build_matrices(k_vec, Sigma_vec, B_vec, errorSDs); 

stateMean = inv(eye(numfactors) - Kmat)*mu; 

meanObservables = A + B * stateMean; 

unconMean_6bond = meanObservables; 

[stateCovMat valid] = uncon_vcv_general(Kmat, Sigma); 

factor_SDs = sqrt(diag(stateCovMat)); 

[funval loglikes filtered_values errors] = ...
    kalman_only_loglike(freeparams, bigparamvec, free_index, ...
                        observedData, numfactors, tolerance); 

filtered_values_6bond = filtered_values; 

sampleMean_6bond = A + B * mean(filtered_values_6bond)';

[T numObserved] = size(observedData);

filteredObs = repmat(A', T, 1) + filtered_values*B'; 

filtered_inflation_6bond = filteredObs(:,end-1);
filtered_cgrowth_6bond = filteredObs(:,end);

tmpmat = eye(numfactors);
if (stepsAhead > 1)
  for i = 1:(stepsAhead-1)
    tmpmat = tmpmat + Kmat^i;
  end
end

stateForecastAhead_6bond = repmat((tmpmat*mu)', T, 1) + ...
    filtered_values*((Kmat^stepsAhead)');

obsForecastAhead_6bond = repmat(A', T, 1) + ...
    stateForecastAhead_6bond*B';

SD_states_6bond = std(filtered_values_6bond);
mean_states_6bond = mean(filtered_values_6bond);

crossSectErrors = 400*(observedData - filteredObs);

obsForecastErrors = obsForecastAhead_6bond(1:(end-stepsAhead),:) ...
    - observedData((1+stepsAhead):end,:);

RMSE_CSErrors_6bond_factor = sqrt(mean(crossSectErrors .^2));
rmse_of_errors_6bond_factor = sqrt(mean(obsForecastErrors .^2)); 


path(origPath) 


%============================================================
%
% Second, the results for the two no-arbitrage models.
%
%============================================================


path('..\codeToEstimateNoArb', origPath);


sixBond_4factor = char('..\noarbFlatFiles\noarbQ4Manyb.out');
sixBond_4macro = char('..\noarbFlatFiles\noarbQ4Macrob.out');

%===============================================================
%
% get results for the six-bond case. 
%
%===============================================================


[yielddatafile maturitydatafile ...
 numbonds numUsedBonds usedBondIdx ...
 macrodatafile numMacro numUsedMacro usedMacroIdx ...
 numfactors firstobs lastobs macroLoadings ...
 use_ols_start use_pc factor_indexes ...
 numloops numSmallLoops numsimplex numderiv ...
 muStar_names muStar_flag muStar_initialval muStar_rand_stddev ...
 kStar_names kStar_flag kStar_initialval kStar_rand_stddev ...
 SigmaStar_names SigmaStar_flag SigmaStar_initialval ...
 SigmaStar_rand_stddev ...
 rinfty_name rinfty_flag rinfty_initialval rinfty_rand_stddev ...
 eig_names eig_flag eig_initialval eig_rand_stddev ...
 A_f_names A_f_flag A_f_initialval A_f_rand_stddev ...
 B_f_names B_f_flag B_f_initialval B_f_rand_stddev ...
 errorSD_names errorSD_flag errorSD_initialval errorSD_rand_stddev ...
] = read_input(sixBond_4factor);

numSeries = numUsedBonds + numUsedMacro; 
use_ols_start = 0;

[observedData maturities] = ...
    read_data(yielddatafile, maturitydatafile, ...
              macrodatafile, numbonds, usedBondIdx, ...
              numMacro, usedMacroIdx, firstobs, lastobs);


if (use_pc) 
  pcLoadings = pcacov(cov(observedData)); 
  Pmat = pcLoadings(:,1:numfactors)'; 
  Emat = pcLoadings(:,(numfactors+1):end)'; 
else
  other_indexes = setdiff((1:numSeries), factor_indexes); 
  Pmat = zeros(numfactors, numSeries);
  Emat = zeros(numSeries-numfactors, numSeries); 
  for i = 1:numfactors
    Pmat(i,factor_indexes(i)) = 1;
  end
  for i = 1:(numSeries - numfactors)
    Emat(i,other_indexes(i)) = 1;
  end
end

if (macroLoadings == 1)
  [freeparams bigparamvec free_index] = setup_initparams( ...
      muStar_initialval, kStar_initialval, SigmaStar_initialval, ...
      rinfty_initialval, eig_initialval, ...
      A_f_initialval, B_f_initialval, ...
      errorSD_initialval, muStar_flag, kStar_flag, ...
      SigmaStar_flag, rinfty_flag, eig_flag, A_f_flag, B_f_flag, ...
      errorSD_flag, ...
      observedData, use_ols_start, Pmat); 
else
  [freeparams bigparamvec free_index] = setup_initparamsStar( ...
      muStar_initialval, kStar_initialval, SigmaStar_initialval, ...
      rinfty_initialval, eig_initialval, ...
      A_f_initialval, B_f_initialval, ...
      errorSD_initialval, muStar_flag, kStar_flag, ...
      SigmaStar_flag, rinfty_flag, eig_flag, A_f_flag, B_f_flag, ...
      errorSD_flag, ...
      observedData, use_ols_start, Pmat); 
end

meanData = mean(observedData);
stdData = std(observedData); 


[muStar, kStar, SigmaStar, rinfty, eigs, A_f, B_f, errorSDs] = ...
    unpack_allparams(numfactors, numUsedMacro, bigparamvec);

[kMatStar SigmaMatStar B_fMat ...
          kEquivMatLatent maxAbsEquivEig H] = ...
    build_matrices(kStar, SigmaStar, B_f, eigs, errorSDs);

if (macroLoadings == 1) 
  [latentModel observableModel] = ...
      buildBothModels(muStar, kMatStar, SigmaMatStar, rinfty, ...
                      kEquivMatLatent, A_f, B_fMat, ...
                      Pmat, maturities); 
else
  [latentModel observableModel] = ...
      buildBothModelsStar(muStar, kMatStar, SigmaMatStar, rinfty, ...
                          kEquivMatLatent, A_f, B_fMat, ...
                          Pmat, maturities); 
end
  
mu = observableModel.mu;
Kmat = observableModel.K;
Sigma = observableModel.Sigma;
A = observableModel.A;
B = observableModel.B;

tolerance = 1e-12; 

[errors loglikes filtered_values cond_vars contemp_vars ...
 normSqrErrors eigFs] = ...
    kalman_recursion_noderiv(observedData, mu, Kmat, Sigma, H, A, B, ...
                             tolerance);

stateMean = inv(eye(numfactors) - Kmat)*mu; 

meanObservables = A + B * stateMean; 

unconMean_6bond = meanObservables; 

[stateCovMat valid] = uncon_vcv_general(Kmat, Sigma); 

factor_SDs = sqrt(diag(stateCovMat)); 

filtered_values_6bond = filtered_values; 

sampleMean_6bond = A + B * mean(filtered_values_6bond)';

[T numObserved] = size(observedData);

filteredObs = repmat(A', T, 1) + filtered_values*B'; 

filtered_inflation_6bond = filteredObs(:,end-1);
filtered_cgrowth_6bond = filteredObs(:,end);

numNegCgrowth = sum(observedData(:,end) < 0);
numFilteredNegCgrowth = sum(filtered_cgrowth_6bond < 0); 

tmpmat = eye(numfactors);
if (stepsAhead > 1)
  for i = 1:(stepsAhead-1)
    tmpmat = tmpmat + Kmat^i;
  end
end

stateForecastAhead_6bond = repmat((tmpmat*mu)', T, 1) + ...
    filtered_values*((Kmat^stepsAhead)');

obsForecastAhead_6bond = repmat(A', T, 1) + ...
    stateForecastAhead_6bond*B';

SD_states_6bond = std(filtered_values_6bond);
mean_states_6bond = mean(filtered_values_6bond);

crossSectErrors = 400*(observedData - filteredObs);

obsForecastErrors = obsForecastAhead_6bond(1:(end-stepsAhead),:) ...
    - observedData((1+stepsAhead):end,:);

RMSE_CSErrors_6bond_noarb = sqrt(mean(crossSectErrors .^2));
rmse_of_errors_6bond_noarb = sqrt(mean(obsForecastErrors.^2)); 

%===============================================================
%
% get results for the six-bond case that uses
% exactly-observed ARMA C growth and inflation.  
%
%===============================================================

[yielddatafile maturitydatafile ...
 numbonds numUsedBonds usedBondIdx ...
 macrodatafile numMacro numUsedMacro usedMacroIdx ...
 numfactors firstobs lastobs macroLoadings ...
 use_ols_start use_pc factor_indexes ...
 numloops numSmallLoops numsimplex numderiv ...
 muStar_names muStar_flag muStar_initialval muStar_rand_stddev ...
 kStar_names kStar_flag kStar_initialval kStar_rand_stddev ...
 SigmaStar_names SigmaStar_flag SigmaStar_initialval ...
 SigmaStar_rand_stddev ...
 rinfty_name rinfty_flag rinfty_initialval rinfty_rand_stddev ...
 eig_names eig_flag eig_initialval eig_rand_stddev ...
 A_f_names A_f_flag A_f_initialval A_f_rand_stddev ...
 B_f_names B_f_flag B_f_initialval B_f_rand_stddev ...
 errorSD_names errorSD_flag errorSD_initialval errorSD_rand_stddev ...
] = read_input(sixBond_4macro);

numSeries = numUsedBonds + numUsedMacro; 
use_ols_start = 0;

[observedDataMacro maturities] = ...
    read_data(yielddatafile, maturitydatafile, ...
              macrodatafile, numbonds, usedBondIdx, ...
              numMacro, usedMacroIdx, firstobs, lastobs);


if (use_pc) 
  pcLoadings = pcacov(cov(observedDataMacro)); 
  Pmat = pcLoadings(:,1:numfactors)'; 
  Emat = pcLoadings(:,(numfactors+1):end)'; 
else
  other_indexes = setdiff((1:numSeries), factor_indexes); 
  Pmat = zeros(numfactors, numSeries);
  Emat = zeros(numSeries-numfactors, numSeries); 
  for i = 1:numfactors
    Pmat(i,factor_indexes(i)) = 1;
  end
  for i = 1:(numSeries - numfactors)
    Emat(i,other_indexes(i)) = 1;
  end
end

if (macroLoadings == 1)
  [freeparams bigparamvec free_index] = setup_initparams( ...
      muStar_initialval, kStar_initialval, SigmaStar_initialval, ...
      rinfty_initialval, eig_initialval, ...
      A_f_initialval, B_f_initialval, ...
      errorSD_initialval, muStar_flag, kStar_flag, ...
      SigmaStar_flag, rinfty_flag, eig_flag, A_f_flag, B_f_flag, ...
      errorSD_flag, ...
      observedDataMacro, use_ols_start, Pmat); 
else
  [freeparams bigparamvec free_index] = setup_initparamsStar( ...
      muStar_initialval, kStar_initialval, SigmaStar_initialval, ...
      rinfty_initialval, eig_initialval, ...
      A_f_initialval, B_f_initialval, ...
      errorSD_initialval, muStar_flag, kStar_flag, ...
      SigmaStar_flag, rinfty_flag, eig_flag, A_f_flag, B_f_flag, ...
      errorSD_flag, ...
      observedDataMacro, use_ols_start, Pmat); 
end

meanDataMacro = mean(observedDataMacro);
stdDataMacro = std(observedDataMacro); 

[muStar, kStar, SigmaStar, rinfty, eigs, A_f, B_f, errorSDs] = ...
    unpack_allparams(numfactors, numUsedMacro, bigparamvec);

[kMatStar SigmaMatStar B_fMat ...
          kEquivMatLatent maxAbsEquivEig H] = ...
    build_matrices(kStar, SigmaStar, B_f, eigs, errorSDs);

if (macroLoadings == 1) 
  [latentModel observableModel] = ...
      buildBothModels(muStar, kMatStar, SigmaMatStar, rinfty, ...
                      kEquivMatLatent, A_f, B_fMat, ...
                      Pmat, maturities); 
else
  [latentModel observableModel] = ...
      buildBothModelsStar(muStar, kMatStar, SigmaMatStar, rinfty, ...
                          kEquivMatLatent, A_f, B_fMat, ...
                          Pmat, maturities); 
end
  
mu = observableModel.mu;
Kmat = observableModel.K;
Sigma = observableModel.Sigma;
A = observableModel.A;
B = observableModel.B;

tolerance = 1e-12; 

[errors loglikes filtered_values cond_vars contemp_vars ...
 normSqrErrors eigFs] = ...
    kalman_recursion_noderiv(observedDataMacro, mu, Kmat, Sigma, H, A, B, ...
                             tolerance);

stateMean = inv(eye(numfactors) - Kmat)*mu; 

meanObservables = A + B * stateMean; 

unconMean_Macro = meanObservables; 

[stateCovMat valid] = uncon_vcv_general(Kmat, Sigma); 

factor_SDs = sqrt(diag(stateCovMat)); 

filtered_values_Macro = filtered_values; 

sampleMean_Macro = A + B * mean(filtered_values_Macro)';

[T numObserved] = size(observedDataMacro);

filteredObs = repmat(A', T, 1) + filtered_values*B'; 

filtered_inflation_Macro = filteredObs(:,end-1);
filtered_cgrowth_Macro = filteredObs(:,end);

tmpmat = eye(numfactors);
if (stepsAhead > 1)
  for i = 1:(stepsAhead-1)
    tmpmat = tmpmat + Kmat^i;
  end
end

stateForecastAhead_Macro = repmat((tmpmat*mu)', T, 1) + ...
    filtered_values*((Kmat^stepsAhead)');

obsForecastAhead_Macro = repmat(A', T, 1) + ...
    stateForecastAhead_Macro*B';

SD_states_Macro = std(filtered_values_Macro);
mean_states_Macro = mean(filtered_values_Macro);

crossSectErrors = 400*(observedData - filteredObs);

obsForecastErrors_Macro = obsForecastAhead_Macro(1:(end-stepsAhead),:) ...
    - observedData((1+stepsAhead):end,:);

RMSE_CSErrors_macro_noarb = sqrt(mean(crossSectErrors .^2));
rmse_of_errors_Macro = sqrt(mean(obsForecastErrors_Macro.^2)); 


%=========================================================
%
% Print cross-sectional deviations 
%
%=========================================================


fprintf(1, 'RMSE, cross-sectional deviations\n'); 
fprintf(1, '   two-bond      six-bond six, no-arb six, ARMA   \n'); 
fprintf(1, '--------------------------------------------------\n'); 
for i = 1:length(RMSE_CSErrors_macro_noarb)
  if (i == 1) 
    fprintf(1, '%d %7.2f      %7.2f  %7.2f %7.2f\n', i, ...
            RMSE_CSErrors_2bond(i), ...
            RMSE_CSErrors_6bond_factor(i), ...
            RMSE_CSErrors_6bond_noarb(i), ...
            RMSE_CSErrors_macro_noarb(i));
  else
    if (i > 5) 
    fprintf(1, '%d %7.2f      %7.2f  %7.2f %7.2f\n', i, ...
            RMSE_CSErrors_2bond(i-4), ...
            RMSE_CSErrors_6bond_factor(i), ...
            RMSE_CSErrors_6bond_noarb(i), ...
            RMSE_CSErrors_macro_noarb(i));
    else
      fprintf(1, '%d              %7.2f  %7.2f %7.2f\n', i, ...
            RMSE_CSErrors_6bond_factor(i), ...
            RMSE_CSErrors_6bond_noarb(i), ...
            RMSE_CSErrors_macro_noarb(i));
    end
  end
end

%=========================================================
%
% Print forecasting results.
%
%=========================================================

fprintf(1, '\n'); 
fprintf(1, 'RMSE, forecasted values\n'); 
fprintf(1, '%d steps ahead\n', stepsAhead); 
fprintf(1, '   two-bond      six-bond six, no-arb six, ARMA   \n'); 
fprintf(1, '--------------------------------------------------\n'); 
for i = 1:length(rmse_of_errors_6bond_factor)
  if (i == 1) 
    fprintf(1, '%d %7.2f      %7.2f  %7.2f %7.2f\n', i, ...
            rmse_of_errors_2bond(i)*400, ...
            rmse_of_errors_6bond_factor(i)*400, ...
            rmse_of_errors_6bond_noarb(i)*400, ...
            rmse_of_errors_Macro(i)*400);
  else
    if (i > 5) 
    fprintf(1, '%d %7.2f      %7.2f  %7.2f %7.2f\n', i, ...
            rmse_of_errors_2bond(i-4)*400, ...
            rmse_of_errors_6bond_factor(i)*400, ...
            rmse_of_errors_6bond_noarb(i)*400, ...
            rmse_of_errors_Macro(i)*400);
    else
      fprintf(1, '%d              %7.2f  %7.2f %7.2f\n', i, ...
              rmse_of_errors_6bond_factor(i)*400, ...
              rmse_of_errors_6bond_noarb(i)*400, ...
              rmse_of_errors_Macro(i)*400);
    end
  end
end


origPath = path;
