% Greg Duffee
%
% Table 6.

clear
close all


origPath = path;
path('..\codeToEstimateNoArb', origPath);


%=========================================================
%
%  Used models.
%
%=========================================================

whiteNoiseModel = char('..\noarbFlatFiles\noarbQ4Manyb.out');
exactObservedModel = char('..\noarbFlatFiles\noarbQ4Macrob.out');

%=========================================================
%
%  Step size for first derivatives.
%
%=========================================================

delta = 1e-7; 

%=========================================================
%
%  First get the results for the model that has all variables
%  observed with white-noise components.
%
%=========================================================

noarbModel = whiteNoiseModel;

[yielddatafile maturitydatafile ...
 numbonds numUsedBonds usedBondIdx ...
 macrodatafile numMacro numUsedMacro usedMacroIdx ...
 numfactors firstobs lastobs macroLoadings ...
 use_ols_start use_pc factor_indexes ...
 numloops numSmallLoops numsimplex numderiv ...
 muStar_names muStar_flag muStar_vals muStar_rand_stddev ...
 kStar_names kStar_flag kStar_vals kStar_rand_stddev ...
 SigmaStar_names SigmaStar_flag SigmaStar_vals ...
 SigmaStar_rand_stddev ...
 rinfty_name rinfty_flag rinfty_val rinfty_rand_stddev ...
 eig_names eig_flag eig_vals eig_rand_stddev ...
 A_f_names A_f_flag A_f_vals A_f_rand_stddev ...
 B_f_names B_f_flag B_f_vals B_f_rand_stddev ...
 errorSD_names errorSD_flag errorSD_vals errorSD_rand_stddev ...
] = read_input(noarbModel);

use_ols_start = 0;
numSeries = numUsedBonds + numUsedMacro; 

[observedData maturities] = ...
    read_data(yielddatafile, maturitydatafile, ...
              macrodatafile, numbonds, usedBondIdx, ...
              numMacro, usedMacroIdx, firstobs, lastobs);


if (use_pc) 
  pcLoadings = pcacov(cov(observedData)); 
  Pmat = pcLoadings(:,1:numfactors)'; 
  Emat = pcLoadings(:,(numfactors+1):end)'; 
else
  other_indexes = setdiff((1:numSeries), factor_indexes); 
  Pmat = zeros(numfactors, numSeries);
  Emat = zeros(numSeries-numfactors, numSeries); 
  for i = 1:numfactors
    Pmat(i,factor_indexes(i)) = 1;
  end
  for i = 1:(numSeries - numfactors)
    Emat(i,other_indexes(i)) = 1;
  end
end

if (macroLoadings == 1)
  [freeparams bigparamvec free_index] = setup_initparams( ...
      muStar_vals, kStar_vals, SigmaStar_vals, ...
      rinfty_val, eig_vals, ...
      A_f_vals, B_f_vals, ...
      errorSD_vals, muStar_flag, kStar_flag, ...
      SigmaStar_flag, rinfty_flag, eig_flag, A_f_flag, B_f_flag, ...
      errorSD_flag, ...
      observedData, use_ols_start, Pmat); 
else
  [freeparams bigparamvec free_index] = setup_initparamsStar( ...
      muStar_vals, kStar_vals, SigmaStar_vals, ...
      rinfty_val, eig_vals, ...
      A_f_vals, B_f_vals, ...
      errorSD_vals, muStar_flag, kStar_flag, ...
      SigmaStar_flag, rinfty_flag, eig_flag, A_f_flag, B_f_flag, ...
      errorSD_flag, ...
      observedData, use_ols_start, Pmat); 
end


tolerance = 1e-16;

[score outerProd scoreMat] = ...
    kalman_derivs(freeparams, bigparamvec, free_index, ...
                  observedData, maturities, numfactors, ...
                  macroLoadings, Pmat, tolerance, delta); 

[OPinv ok] = invpd(outerProd);

if (ok == 1)
  paramSDs = sqrt(diag(OPinv));
end

numfreeparams = length(freeparams); 

[meanShortNom meanShortReal meanSlope meanRealSlope ...
          inflationComp consumptionComp] = ...
    modelFeatures(freeparams, bigparamvec, free_index, ...
                  maturities, numfactors, numUsedMacro, ...
                  macroLoadings, Pmat); 

dmeanShortNom = zeros(numfreeparams, 1);
dmeanShortReal = zeros(numfreeparams, 1);
dmeanSlope = zeros(numfreeparams, 1);
dmeanRealSlope = zeros(numfreeparams, 1);
dinflationComp = zeros(numfreeparams, 1);
dconsumptionComp = zeros(numfreeparams, 1);

for thisParam = 1:numfreeparams
  freeCopy = freeparams;

  freeCopy(thisParam) = freeparams(thisParam) + delta;

  [meanShortNomUp meanShortRealUp meanSlopeUp meanRealSlopeUp ...
          inflationCompUp consumptionCompUp] = ...
    modelFeatures(freeCopy, bigparamvec, free_index, ...
                  maturities, numfactors, numUsedMacro, ...
                  macroLoadings, Pmat); 

  freeCopy(thisParam) = freeparams(thisParam) - delta;

  [meanShortNomDown meanShortRealDown meanSlopeDown meanRealSlopeDown ...
   inflationCompDown consumptionCompDown] = ...
      modelFeatures(freeCopy, bigparamvec, free_index, ...
                    maturities, numfactors, numUsedMacro, ...
                    macroLoadings, Pmat); 

  dmeanShortNom(thisParam) = (meanShortNomUp - meanShortNomDown)/ ...
      (2*delta);
  dmeanShortReal(thisParam) = (meanShortRealUp - meanShortRealDown)/ ...
      (2*delta);
  dmeanSlope(thisParam) = (meanSlopeUp - meanSlopeDown)/ ...
      (2*delta);
  dmeanRealSlope(thisParam) = (meanRealSlopeUp - meanRealSlopeDown)/ ...
      (2*delta);
  dinflationComp(thisParam) = (inflationCompUp - inflationCompDown)/ ...
      (2*delta);
  dconsumptionComp(thisParam) = (consumptionCompUp - consumptionCompDown)/ ...
      (2*delta);
  
end


stdErr1 = sqrt(dmeanShortNom'*OPinv*dmeanShortNom);
stdErr2 = sqrt(dmeanShortReal'*OPinv*dmeanShortReal);
stdErr3 = sqrt(dmeanSlope'*OPinv*dmeanSlope);
stdErr4 = sqrt(dmeanRealSlope'*OPinv*dmeanRealSlope);
stdErr5 = sqrt(dinflationComp'*OPinv*dinflationComp);
stdErr6 = sqrt(dconsumptionComp'*OPinv*dconsumptionComp);

stdErrs = [stdErr1; stdErr2; stdErr3; stdErr4; stdErr5; stdErr6];


fprintf(1, '\n');
fprintf(1, 'Features of model with white-noise components\n\n');
fprintf(1, '                                Value     SE\n'); 
fprintf(1, '-----------------------------------------------\n'); 
fprintf(1, 'Mean short nominal rate  %9.2f %9.2f \n', meanShortNom, ...
        stdErr1);
fprintf(1, 'Mean 5yr - 3m nominal    %9.2f %9.2f \n', meanSlope, ...
        stdErr3);
fprintf(1, 'Inflation compensation   %9.2f %9.2f \n', inflationComp, ...
        stdErr5);
fprintf(1, 'Consumption compensation %9.2f %9.2f \n', consumptionComp, ...
        stdErr6);
fprintf(1, 'Mean short real rate     %9.2f %9.2f \n', meanShortReal, ...
        stdErr2);
fprintf(1, 'Mean 5yr - 3m real       %9.2f %9.2f \n', meanRealSlope, ...
        stdErr4);



%=========================================================
%
%  second get the results for the model that has macro variables
%  observed with no error.
%
%=========================================================

noarbModel = exactObservedModel;

[yielddatafile maturitydatafile ...
 numbonds numUsedBonds usedBondIdx ...
 macrodatafile numMacro numUsedMacro usedMacroIdx ...
 numfactors firstobs lastobs macroLoadings ...
 use_ols_start use_pc factor_indexes ...
 numloops numSmallLoops numsimplex numderiv ...
 muStar_names muStar_flag muStar_vals muStar_rand_stddev ...
 kStar_names kStar_flag kStar_vals kStar_rand_stddev ...
 SigmaStar_names SigmaStar_flag SigmaStar_vals ...
 SigmaStar_rand_stddev ...
 rinfty_name rinfty_flag rinfty_val rinfty_rand_stddev ...
 eig_names eig_flag eig_vals eig_rand_stddev ...
 A_f_names A_f_flag A_f_vals A_f_rand_stddev ...
 B_f_names B_f_flag B_f_vals B_f_rand_stddev ...
 errorSD_names errorSD_flag errorSD_vals errorSD_rand_stddev ...
] = read_input(noarbModel);

use_ols_start = 0;
numSeries = numUsedBonds + numUsedMacro; 

[observedData maturities] = ...
    read_data(yielddatafile, maturitydatafile, ...
              macrodatafile, numbonds, usedBondIdx, ...
              numMacro, usedMacroIdx, firstobs, lastobs);


if (use_pc) 
  pcLoadings = pcacov(cov(observedData)); 
  Pmat = pcLoadings(:,1:numfactors)'; 
  Emat = pcLoadings(:,(numfactors+1):end)'; 
else
  other_indexes = setdiff((1:numSeries), factor_indexes); 
  Pmat = zeros(numfactors, numSeries);
  Emat = zeros(numSeries-numfactors, numSeries); 
  for i = 1:numfactors
    Pmat(i,factor_indexes(i)) = 1;
  end
  for i = 1:(numSeries - numfactors)
    Emat(i,other_indexes(i)) = 1;
  end
end

if (macroLoadings == 1)
  [freeparams bigparamvec free_index] = setup_initparams( ...
      muStar_vals, kStar_vals, SigmaStar_vals, ...
      rinfty_val, eig_vals, ...
      A_f_vals, B_f_vals, ...
      errorSD_vals, muStar_flag, kStar_flag, ...
      SigmaStar_flag, rinfty_flag, eig_flag, A_f_flag, B_f_flag, ...
      errorSD_flag, ...
      observedData, use_ols_start, Pmat); 
else
  [freeparams bigparamvec free_index] = setup_initparamsStar( ...
      muStar_vals, kStar_vals, SigmaStar_vals, ...
      rinfty_val, eig_vals, ...
      A_f_vals, B_f_vals, ...
      errorSD_vals, muStar_flag, kStar_flag, ...
      SigmaStar_flag, rinfty_flag, eig_flag, A_f_flag, B_f_flag, ...
      errorSD_flag, ...
      observedData, use_ols_start, Pmat); 
end


tolerance = 1e-16;

[score outerProd scoreMat] = ...
    kalman_derivs(freeparams, bigparamvec, free_index, ...
                  observedData, maturities, numfactors, ...
                  macroLoadings, Pmat, tolerance, delta); 

[OPinv ok] = invpd(outerProd);

if (ok == 1)
  paramSDs = sqrt(diag(OPinv));
end

numfreeparams = length(freeparams); 

[meanShortNom meanShortReal meanSlope meanRealSlope ...
          inflationComp consumptionComp] = ...
    modelFeatures(freeparams, bigparamvec, free_index, ...
                  maturities, numfactors, numUsedMacro, ...
                  macroLoadings, Pmat); 

dmeanShortNom = zeros(numfreeparams, 1);
dmeanShortReal = zeros(numfreeparams, 1);
dmeanSlope = zeros(numfreeparams, 1);
dmeanRealSlope = zeros(numfreeparams, 1);
dinflationComp = zeros(numfreeparams, 1);
dconsumptionComp = zeros(numfreeparams, 1);

for thisParam = 1:numfreeparams
  freeCopy = freeparams;

  freeCopy(thisParam) = freeparams(thisParam) + delta;

  [meanShortNomUp meanShortRealUp meanSlopeUp meanRealSlopeUp ...
          inflationCompUp consumptionCompUp] = ...
    modelFeatures(freeCopy, bigparamvec, free_index, ...
                  maturities, numfactors, numUsedMacro, ...
                  macroLoadings, Pmat); 

  freeCopy(thisParam) = freeparams(thisParam) - delta;

  [meanShortNomDown meanShortRealDown meanSlopeDown meanRealSlopeDown ...
   inflationCompDown consumptionCompDown] = ...
      modelFeatures(freeCopy, bigparamvec, free_index, ...
                    maturities, numfactors, numUsedMacro, ...
                    macroLoadings, Pmat); 

  dmeanShortNom(thisParam) = (meanShortNomUp - meanShortNomDown)/ ...
      (2*delta);
  dmeanShortReal(thisParam) = (meanShortRealUp - meanShortRealDown)/ ...
      (2*delta);
  dmeanSlope(thisParam) = (meanSlopeUp - meanSlopeDown)/ ...
      (2*delta);
  dmeanRealSlope(thisParam) = (meanRealSlopeUp - meanRealSlopeDown)/ ...
      (2*delta);
  dinflationComp(thisParam) = (inflationCompUp - inflationCompDown)/ ...
      (2*delta);
  dconsumptionComp(thisParam) = (consumptionCompUp - consumptionCompDown)/ ...
      (2*delta);
  
end


stdErr1 = sqrt(dmeanShortNom'*OPinv*dmeanShortNom);
stdErr2 = sqrt(dmeanShortReal'*OPinv*dmeanShortReal);
stdErr3 = sqrt(dmeanSlope'*OPinv*dmeanSlope);
stdErr4 = sqrt(dmeanRealSlope'*OPinv*dmeanRealSlope);
stdErr5 = sqrt(dinflationComp'*OPinv*dinflationComp);
stdErr6 = sqrt(dconsumptionComp'*OPinv*dconsumptionComp);

stdErrs = [stdErr1; stdErr2; stdErr3; stdErr4; stdErr5; stdErr6];


fprintf(1, '\n');
fprintf(1, 'Features of model with perfectly-observed components\n\n');
fprintf(1, '                                Value     SE\n'); 
fprintf(1, '-----------------------------------------------\n'); 
fprintf(1, 'Mean short nominal rate  %9.2f %9.2f \n', meanShortNom, ...
        stdErr1);
fprintf(1, 'Mean 5yr - 3m nominal    %9.2f %9.2f \n', meanSlope, ...
        stdErr3);
fprintf(1, 'Inflation compensation   %9.2f %9.2f \n', inflationComp, ...
        stdErr5);
fprintf(1, 'Consumption compensation %9.2f %9.2f \n', consumptionComp, ...
        stdErr6);
fprintf(1, 'Mean short real rate     %9.2f %9.2f \n', meanShortReal, ...
        stdErr2);
fprintf(1, 'Mean 5yr - 3m real       %9.2f %9.2f \n', meanRealSlope, ...
        stdErr4);


%
% wrap up
%


origPath = path;
